#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Part of the Human Connectome - Lifespan Project Task fMRI Battery
Psychopy implementation by Erik K Kastman, Constanza M Vidal Bustamente, and Leah H Somerville
**********************************************************************************************
Tested with Psychopy 1.83.04; Psyexp Experiment Builder XML available upon request
  Peirce, JW (2007) PsychoPy - Psychophysics software in Python. Journal of Neuroscience Methods, 162(1-2), 8-13.
  Peirce, JW (2009) Generating stimuli for neuroscience using PsychoPy. Frontiers in Neuroinformatics, 2:10. doi: 10.3389/neuro.11.010.2008
"""

from __future__ import division  # so that 1/3=0.333 instead of 1/3=0
import datetime
from psychopy import locale_setup, visual, core, data, event, logging, sound, gui
from psychopy.constants import *  # things like STARTED, FINISHED
import numpy as np  # whole numpy lib is available, prepend 'np.'
from numpy import sin, cos, tan, log, log10, pi, average, sqrt, std, deg2rad, rad2deg, linspace, asarray
from numpy.random import random, randint, normal, shuffle
import os  # handy system and path functions
import sys # to get file system encoding

# Common HCP Tools
import hcpcommon
hcpcommon.checkScreenCount()
# Ensure that relative paths start from the same directory as this script
_thisDir = os.path.dirname(os.path.abspath(__file__)).decode(sys.getfilesystemencoding())
os.chdir(_thisDir)

# Store info about the experiment session
expName = 'CARIT'  # from the Builder filename that created this script
expInfo = {u'runNumber': u'practice', u'sessionID': u'', u'mriMode': u'Off',  u'shape': u'circle'}
dlg = gui.DlgFromDict(dictionary=expInfo, title=expName)
if dlg.OK == False: core.quit()  # user pressed cancel
expInfo['date'] = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')  # add a simple timestamp
expInfo['expName'] = expName

# Data file name stem = absolute path + name; later add .psyexp, .csv, .log, etc
filename = _thisDir + os.sep + 'data' + os.path.sep + '_'.join([expName, expInfo['sessionID'], 'run%s' % expInfo['runNumber'], expInfo['date']])

# An ExperimentHandler isn't essential but helps with data saving
thisExp = data.ExperimentHandler(name=expName, version='',
    extraInfo=expInfo, runtimeInfo=None,
    originPath=None,
    savePickle=True, saveWideText=False,
    dataFileName=filename)
#save a log file for detail verbose info
logFile = logging.LogFile(filename+'.log', level=logging.EXP)
logging.console.setLevel(logging.WARNING)  # this outputs to the screen, not a file

endExpNow = False  # flag for 'escape' or other condition => quit the exp

# Start Code - component code to be run before the window creation
## EKK Screen Edit of compiled script
import pyglet
display = pyglet.window.get_platform().get_default_display()
screens = display.get_screens()
resolution = [screens[-1].width, screens[-1].height]
import yaml
from psychopy import monitors
if not os.path.exists('siteConfig.yaml'): raise IOError('Missing siteConfig.yaml - Please copy configuration text file')
with open('siteConfig.yaml') as f:
    config = yaml.safe_load(f)
mon = monitors.Monitor('newMonitor')
mon.setWidth(config['monitor']['width'])
mon.setDistance(config['monitor']['distance'])
mon.setSizePix(resolution)

# Setup the Window
win = visual.Window(size=resolution, fullscr=False, screen=config['monitor']['screen'], allowGUI=False, allowStencil=False,
    monitor=mon, color=[-1,-1,-1], colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg')
# store frame rate of monitor if we can measure it successfully
expInfo['frameRate']=win.getActualFrameRate()
if expInfo['frameRate']!=None:
    frameDur = 1.0/round(expInfo['frameRate'])
else:
    frameDur = 1.0/60.0 # couldn't get a reliable measure so guess

# Initialize components for Routine "intro"
introClock = core.Clock()
import yaml
import itertools
import hcpcommon

if os.path.exists('siteConfig.yaml'):
    with open('siteConfig.yaml') as f:
        config = yaml.safe_load(f)
else:
    copyMsg = ('Please copy configuration text file '
               '"siteConfig.yaml.example" to "siteConfig.yaml" '
               'and edit it with your trigger and buttons.')
    raise IOError(copyMsg)

logging.exp('Using Site Configuration: %s' % config)
allowed_keys = list(itertools.chain.from_iterable(
    config['keys'].values()))

runNumber = expInfo['runNumber']

if 'nRuns' not in expInfo.keys():
    expInfo['nRuns'] = 1

if runNumber == 'practice':
    nRuns = 1
    runNumber = 0
    mode = 'practice'
else:
    nRuns = expInfo['nRuns']
    try:
        runNumber = int(runNumber)
    except ValueError:
        raise ValueError('runNumber must be either "practice" or a whole number, but it was "%s"\n' % runNumber)
    mode = 'scan'

if 'shape' not in expInfo.keys() or expInfo['shape'] in ['na', 'n']:
    # 'shape' removed from CARIT in Aging Protocol
    expInfo['prevRewShape'] = 'na'
    rewShapes = dict(prevRewNogo='circle.png',
                     neutralNogo='square.png')

elif expInfo['shape'] in ['circle', 'c']:
    expInfo['prevRewShape'] = 'circle.png'
    rewShapes = dict(prevRewNogo='circle.png',
                     neutralNogo='square.png')
    del(expInfo['shape'])  # Cleanup expInfo for log

elif expInfo['shape'] in ['square', 's']:
    expInfo['prevRewShape'] = 'square.png'
    rewShapes = dict(prevRewNogo='square.png',
                     neutralNogo='circle.png')
    del(expInfo['shape'])  # Cleanup expInfo for log

else:
    raise ValueError('counterbalance must be either "circle", "square", or "na"')

titleLetterSize = config['style']['titleLetterSize']  # 3
subtitleLetterSize = config['style']['subtitleLetterSize']  # .7
textLetterSize = config['style']['textLetterSize']  # 1.5
fixLetterSize = config['style']['fixLetterSize']  # 2.5
wrapWidth = config['style']['wrapWidth']  # 30
verbalColor = config['style']['verbalColor']  # '#3EB4F0'
frameRateRange = config['frameRateRange']  # [59, 121]

shapeSize = [28, 20]

introText = visual.TextStim(win=win, ori=0, name='introText',
    text='SHAPES',    font='Arial',
    pos=[0, 2], height=titleLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
introText2 = visual.TextStim(win=win, ori=0, name='introText2',
    text='GAME',    font='Arial',
    pos=[0, -2], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
# Setup the RA Experimenter Window
raWin = visual.Window(size=[1280,800], fullscr=False, allowGUI=True, allowStencil=False,
    monitor=u'testMonitor', color=u'black', colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg',
    screen=0)

raText = visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[0, -3], height=textLetterSize * 1.5, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raVerbalText = visual.TextStim(win=raWin, ori=0, name='raVerbalText',
    text='', font='Arial',
    pos=[0,3], height=textLetterSize * 1.5, wrapWidth=30,
    color=verbalColor, colorSpace='rgb', opacity=1,
    depth=0.0, italic=True)

def frameRateWarning():
    dlg = gui.Dlg(title='Frame Rate Warning')
    dlg.addText('WARNING: Frame rate seems too high or too low.')
    dlg.addText('Press "OK" to continue or "Cancel" to stop.')
    dlg.addFixedField('Measured Frame Rate', expInfo['frameRate'])
    dlg.addText('#'*40)
    dlg.addText('This may be caused by background processes or')
    dlg.addText('incorrect screen settings.')
    dlg.addText('')
    dlg.addText('If this is the first time seeing this message, you should:')
    dlg.addText('1. Cancel this run')
    dlg.addText('2. Close anything running in the background.')
    dlg.addText('3. Check that the screen refresh rate is correct.')
    dlg.addText('4. Re-run the script and try again.')
    dlg.addText('')
    return dlg


def checkRates(frameRateRange, statusTextStim):
    statusTextStim.text = 'Checking Screen Refresh - Please wait.'
    statusTextStim.draw()
    statusTextStim.win.flip()
    
    rates = [expInfo['frameRate']]
    for i in range(10):
        rates.append(win.getActualFrameRate())
    medianRate = np.median(rates)
    logging.exp('Measured Framerates: %s' % rates)
    if not (frameRateRange[0] < medianRate < frameRateRange[1]):
        logging.exp('Median Rate (%.03f) still out of range' % medianRate)
        dlg = frameRateWarning()
        dlg.show()
        if not dlg.OK:
            logging.exp('User Cancelled')
            core.quit()  # user pressed cancel
        else:
            logging.exp('User continued anyway; assuming 60Hz')
            # expInfo['frameRate'] = medianRate
            expInfo['frameRate'] = 60.0  # Assume a frame rate of 60
            expInfo['originalMedianFrameRate'] = medianRate
    else:
        logging.exp('Median Rate (%.03f) in acceptable range; continuing' % medianRate)
        expInfo['frameRate'] = medianRate

# Check for sane frame rate
if not (frameRateRange[0] < expInfo['frameRate'] < frameRateRange[1]):
    logging.exp('Unexpected frame rate was measured on first try: %.03f' %
                expInfo['frameRate'])
    checkRates(frameRateRange=frameRateRange, statusTextStim=raText)

frameDur = 1.0/round(expInfo['frameRate'])
outerFrame = visual.Rect(win=win, lineWidth=1, lineColor='white',
                         width=16.4, height=12.4, units='deg', name='outerFrame')

# outerFrame.autoDraw = True
# Record Git Status in Subject Logs
from subprocess import check_output, check_call, CalledProcessError
try:
    revision = check_output(['git','rev-parse', '--short', 'HEAD'])
    revision = revision.strip()
    gitInstalled = True
except:
    if os.path.exists('VERSION'):
        with open('VERSION', 'r') as f:
            revision = f.read().strip()
    else:
        revision = ''
    gitInstalled = False

if gitInstalled:
    try:
        check_call(['git', 'diff-files', '--quiet'])
    except CalledProcessError:
        revision = 'Task changes detected. Nearest head: %s' % revision
        status_msg = check_output(['git', 'status'])
        msg ="""%s

Warning: the experiment has unexpected changes.
""" % status_msg
        logging.critical(msg)
        # core.quit()

expInfo['git-revision'] = revision

movieClock = core.Clock()

record_example = False

winRecorder = hcpcommon.WinRecorder(win=win, 
                                    clock=movieClock, 
                                    record=record_example)

raWinRecorder = hcpcommon.WinRecorder(win=raWin, 
                                    clock=movieClock, 
                                    record=record_example)


# Initialize components for Routine "scanInstr1"
scanInstr1Clock = core.Clock()
scanInstr1Text = visual.TextStim(win=win, ori=0, name='scanInstr1Text',
    text='These are the shapes that you will see:',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
allShapes = visual.ImageStim(win=win, name='allShapes',
    image='images/allshapes.png', mask=None,
    ori=0, pos=[0, -2], size=[15, 9],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "scanInstr2"
scanInstr2Clock = core.Clock()
scanInstr2Text = visual.TextStim(win=win, ori=0, name='scanInstr2Text',
    text='Push the POINTER finger button as QUICKLY as possible!',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
goshapes = visual.ImageStim(win=win, name='goshapes',
    image='images/goshapes.bmp', mask=None,
    ori=0, pos=[0, -2.25], size=(12, 9),
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "scanInstr3"
scanInstr3Clock = core.Clock()
scanInstr3TextTop = visual.TextStim(win=win, ori=0, name='scanInstr3TextTop',
    text='For example, if you see:',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
goShape1 = visual.ImageStim(win=win, name='goShape1',
    image='images/small_oct.png', mask=None,
    ori=0, pos=[0,2], size=[6, 5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
pointerFinger1 = visual.ImageStim(win=win, name='pointerFinger1',
    image='images/pointerFingerButton.png', mask=None,
    ori=0, pos=[0, -2], size=[2.5, 3],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
scanInstr3TextBottom = visual.TextStim(win=win, ori=0, name='scanInstr3TextBottom',
    text='Push the POINTER button as quickly as possible!',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)


# Initialize components for Routine "scanInstr4"
scanInstr4Clock = core.Clock()
scanInstr4TextTop = visual.TextStim(win=win, ori=0, name='scanInstr4TextTop',
    text='Or if you see:',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
goShape2 = visual.ImageStim(win=win, name='goShape2',
    image='images/small_parall.png', mask=None,
    ori=0, pos=[0, 2], size=[6, 5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
pointerFinger2 = visual.ImageStim(win=win, name='pointerFinger2',
    image='images/pointerFingerButton.png', mask=None,
    ori=0, pos=[0,-2], size=[2.5, 3],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
scanInstr4TextBottom = visual.TextStim(win=win, ori=0, name='scanInstr4TextBottom',
    text='Push the POINTER button as quickly as possible!',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)


# Initialize components for Routine "scanInstr5"
scanInstr5Clock = core.Clock()
scanInstr5Text = visual.TextStim(win=win, ori=0, name='scanInstr5Text',
    text='Do NOT press any button!',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
squareInstructions = visual.ImageStim(win=win, name='squareInstructions',
    image='images/small_sq.png', mask=None,
    ori=0, pos=[-4, -1], size=[9, 7],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
circleInstructions = visual.ImageStim(win=win, name='circleInstructions',
    image='images/small_circle.png', mask=None,
    ori=0, pos=[4, -1], size=[9, 7],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "scanInstr6"
scanInstr6Clock = core.Clock()
scanInstr6TextTop = visual.TextStim(win=win, ori=0, name='scanInstr6TextTop',
    text='For example, if you see:',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
nogoShape1 = visual.ImageStim(win=win, name='nogoShape1',
    image='images/small_sq.png', mask=None,
    ori=0, pos=[0, 2], size=[6, 5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
noPress = visual.ImageStim(win=win, name='noPress',
    image='images/noPressButton.png', mask=None,
    ori=0, pos=[0, -2], size=[2.5, 3],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
scanInstr6TextBottom = visual.TextStim(win=win, ori=0, name='scanInstr6TextBottom',
    text='Do NOT press any button!',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)


# Initialize components for Routine "scanInstr7"
scanInstr7Clock = core.Clock()
scanInstr7TextTop = visual.TextStim(win=win, ori=0, name='scanInstr7TextTop',
    text='Or if you see:',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
nogoShape2 = visual.ImageStim(win=win, name='nogoShape2',
    image='images/small_circle.png', mask=None,
    ori=0, pos=[0, 2], size=[6, 5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
noPress2 = visual.ImageStim(win=win, name='noPress2',
    image='images/noPressButton.png', mask=None,
    ori=0, pos=[0,-2], size=[2.5, 3],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
scanInstr7TextBottom = visual.TextStim(win=win, ori=0, name='scanInstr7TextBottom',
    text='Do NOT press any button!',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)


# Initialize components for Routine "scanInstr8"
scanInstr8Clock = core.Clock()
recapShapes = visual.ImageStim(win=win, name='recapShapes',
    image='images/allshapesnogo.png', mask=None,
    ori=0, pos=[0, 0], size=[15, 9],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=0.0)


# Initialize components for Routine "scanInstr9"
scanInstr9Clock = core.Clock()
scanInstr9Text = visual.TextStim(win=win, ori=0, name='scanInstr9Text',
    text="Let's practice.",    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "exampleTrial"
exampleTrialClock = core.Clock()
preshapeFixExampleText = visual.TextStim(win=win, ori=0, name='preshapeFixExampleText',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
shapeExample = visual.ImageStim(win=win, name='shapeExample',
    image='sin', mask=None,
    ori=0, pos=[0, 0], size=[35,25],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
fixExampleText = visual.TextStim(win=win, ori=0, name='fixExampleText',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)



# Initialize components for Routine "feedback"
feedbackClock = core.Clock()
feedbackText = visual.TextStim(win=win, ori=0, name='feedbackText',
    text='default text',    font='Arial',
    pos=[0, 2], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
feedbackTextDetails = visual.TextStim(win=win, ori=0, name='feedbackTextDetails',
    text='default text',    font='Arial',
    pos=[0, -2], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "pracInstr10"
pracInstr10Clock = core.Clock()
pracInstr6Text = visual.TextStim(win=win, ori=0, name='pracInstr6Text',
    text='Do you have any questions?',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "pracInstr11"
pracInstr11Clock = core.Clock()
pracInstr11Text = visual.TextStim(win=win, ori=0, name='pracInstr11Text',
    text='Are you ready to begin the practice?',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "recapInstr"
recapInstrClock = core.Clock()
recapInstrText = visual.TextStim(win=win, ori=0, name='recapInstrText',
    text='As soon as you see any shape OTHER THAN the circle or the square, press the button under your POINTER finger as quickly as possible. If you see a circle or a square, DO NOT press any button!',    font='Arial',
    pos=[0, 4], height=textLetterSize * .7, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
recapShapes2 = visual.ImageStim(win=win, name='recapShapes2',
    image='images/allshapesnogo.png', mask=None,
    ori=0, pos=[0, -2], size=[15, 9],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "waitingScanner"
waitingScannerClock = core.Clock()
waitScannerText = visual.TextStim(win=win, ori=0, name='waitScannerText',
    text='Get ready!',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

events = [] # run, condition, onset, duration, value

nReps = expInfo['nRuns']

import datetime
fmriClock = core.Clock()
trigger = 'usb'
#trigger = 'parallel'
if trigger == 'parallel':
    from psychopy.contrib import parallel as winioport
    #from psychopy import parallel
elif trigger == 'usb':
    from psychopy.hardware.emulator import launchScan
    #
    # settings for launchScan:
    MR_settings = { 
        'TR': 2.000, # duration (sec) per volume
        'volumes': 210, # number of whole-brain 3D volumes / frames
        'sync': config['trigger'], # character to use as the sync timing event; assumed to come at start of a volume
        'skip': 0, # number of volumes lacking a sync pulse at start of scan (for T1 stabilization)
        }

# Initialize components for Routine "countdown"
countdownClock = core.Clock()

countdownText = visual.TextStim(win=win, ori=0, name='countdownText',
    text='default text',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)




# Initialize components for Routine "trial"
trialClock = core.Clock()


import pandas as pd

# Specify columns explicitly for correct order 
COLUMNS = [
    'trialNum', 'stim', 'corrAns', 'prepotency', 'ISI',
    'countdownStartTime', 'countdownLabel',
    'shapeStartTime', 'shapeEndTime',
    'fixStartTime', 'fixEndTime', 'runEndTime',
    'trialOutcome', 'nogoCondition',
    'resp', 'trialResp.keys', 'trialResp.rt', 'trialResp_rel.rt',
    'trialResp.firstKey', 'trialResp.firstRt',
    'corrRespMsg', 'corrRespCode', 'corrRespTrialType',
    'isiPress.keys', 'isiPress.rt', 'isiPress_rel.rt',
    'hitCount', 'missCount', 'falseAlarmCount', 'corrRejectCount',
    'totalAcc', 'goAcc', 'nogoAcc', 'goFiveBackAvg', 'nogoFiveBackAvg',
    'totalFiveBackAvg', 'fixResp.keys',
    'prevRewShape', 'counterbalance', 'condFile',
    'mriMode', 'frameRate', 'expName', 'triggerWallTime',
    'git-revision',
    'date', 'sessionID', 'nRuns', 'runStopTime',
]

def updateRunLog(entries):
    df = pd.DataFrame(entries)
    df = df.loc[df['runs.thisN'] == runs.thisN]  # Grab only rows from the current run
    df['run'] = runNum
    clean_df = df.filter(COLUMNS)
    clean_df['trialNum'] = clean_df['trialNum'].astype(int)

    runOutfile = '%s_wide.csv' % filename
    clean_df.to_csv(runOutfile, header=True, index=False, encoding='utf-8')

shapeStims = {}
shapesList = ['pentagon', 'hexagon', 'octagon', 'plaque',
              'circle', 'square', 'trapezoid', 'parallelogram']
for shapeLabel in shapesList:
    shapeStim = visual.ImageStim(win=win, name=shapeLabel,
        image=u'images/%s.png' % shapeLabel, mask=None,
        ori=0, pos=[0, 0], size=shapeSize,
        color=[1,1,1], colorSpace='rgb', opacity=1.0,
        flipHoriz=False, flipVert=False,
        texRes=128, interpolate=True)
    shapeStims[shapeLabel] = shapeStim

fixText = visual.TextStim(win=win, ori=0, name='fixText',
    text=u'+',    font=u'Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1)

shapeFrames = int(round(.6 * expInfo['frameRate']))
responseWindowFrames = int(round(.8 * expInfo['frameRate']))

logging.exp('Using measured frame rate of %.05f' % expInfo['frameRate'])
logging.exp('Displaying trial shape for %d frames' % shapeFrames)
logging.exp('Accepting responses for %d frames' % responseWindowFrames)

extraISIStim = visual.TextStim(win=win, text='', name='extraISIStim')

dummyISIText = visual.TextStim(win=win, ori=0, name='dummyISIText',
    text=None,    font='Arial',
    pos=[0, 0], height=0.1, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-5.0)

# Initialize components for Routine "endRun"
endRunClock = core.Clock()
endRunFixation = visual.TextStim(win=win, ori=0, name='endRunFixation',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
endRunText = visual.TextStim(win=win, ori=0, name='endRunText',
    text='You finished this round.',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
# 8 sec countdown + 3min 51.6sec + .4s TR adjustment 
# = 240s, 4:00min
runDuration = 240


# Initialize components for Routine "end"
endClock = core.Clock()
endText = visual.TextStim(win=win, ori=0, name='endText',
    text='Congratulations! You finished!',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Create some handy timers
globalClock = core.Clock()  # to track the time since experiment started
routineTimer = core.CountdownTimer()  # to track time remaining of each (non-slip) routine 

#------Prepare to start Routine "intro"-------
t = 0
introClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat

introResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
introResp.status = NOT_STARTED
if mode == 'practice':
    raVerbalText.text = '"Welcome to the Shapes Game. In this game, you are going to see shapes quickly appearing on the screen, one after the other."'
else:
    raVerbalText.text = '"Welcome back to the Shapes Game!"'

raText.text = 'Press <space> to continue.'
raVerbalText.draw()
raText.draw()
raWinRecorder.keyframe()
raWin.flip()



movieClock.reset()
# keep track of which components have finished
introComponents = []
introComponents.append(introText)
introComponents.append(introText2)
introComponents.append(introResp)
for thisComponent in introComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "intro"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = introClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    
    # *introText* updates
    if t >= 0.0 and introText.status == NOT_STARTED:
        # keep track of start time/frame for later
        introText.tStart = t  # underestimates by a little under one frame
        introText.frameNStart = frameN  # exact frame index
        introText.setAutoDraw(True)
    
    # *introText2* updates
    if t >= 0.0 and introText2.status == NOT_STARTED:
        # keep track of start time/frame for later
        introText2.tStart = t  # underestimates by a little under one frame
        introText2.frameNStart = frameN  # exact frame index
        introText2.setAutoDraw(True)
    
    # *introResp* updates
    if t >= 0.0 and introResp.status == NOT_STARTED:
        # keep track of start time/frame for later
        introResp.tStart = t  # underestimates by a little under one frame
        introResp.frameNStart = frameN  # exact frame index
        introResp.status = STARTED
        # keyboard checking is just starting
        win.callOnFlip(introResp.clock.reset)  # t=0 on next screen flip
        event.clearEvents(eventType='keyboard')
    if introResp.status == STARTED:
        theseKeys = event.getKeys(keyList=['space'])
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            introResp.keys = theseKeys[-1]  # just the last key pressed
            introResp.rt = introResp.clock.getTime()
            # a response ends the routine
            continueRoutine = False
    
    
    
    
    if frameN == 0:
        winRecorder.keyframe()
        raWinRecorder.keyframe()
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in introComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "intro"-------
for thisComponent in introComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)

# check responses
if introResp.keys in ['', [], None]:  # No response was made
   introResp.keys=None
# store data for thisExp (ExperimentHandler)
thisExp.addData('introResp.keys',introResp.keys)
if introResp.keys != None:  # we had a response
    thisExp.addData('introResp.rt', introResp.rt)
thisExp.nextEntry()





# the Routine "intro" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()

# set up handler to look after randomisation of conditions etc
pracDirections = data.TrialHandler(nReps=mode == 'practice', method='sequential', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='pracDirections')
thisExp.addLoop(pracDirections)  # add the loop to the experiment
thisPracDirection = pracDirections.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisPracDirection.rgb)
if thisPracDirection != None:
    for paramName in thisPracDirection.keys():
        exec(paramName + '= thisPracDirection.' + paramName)

for thisPracDirection in pracDirections:
    currentLoop = pracDirections
    # abbreviate parameter names if possible (e.g. rgb = thisPracDirection.rgb)
    if thisPracDirection != None:
        for paramName in thisPracDirection.keys():
            exec(paramName + '= thisPracDirection.' + paramName)
    
    #------Prepare to start Routine "scanInstr1"-------
    t = 0
    scanInstr1Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr1Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr1Resp.status = NOT_STARTED
    raVerbalText.text = '"Here are the eight shapes that you will see in this game."'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    scanInstr1Components = []
    scanInstr1Components.append(scanInstr1Text)
    scanInstr1Components.append(scanInstr1Resp)
    scanInstr1Components.append(allShapes)
    for thisComponent in scanInstr1Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr1"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr1Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr1Text* updates
        if t >= 0.0 and scanInstr1Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr1Text.tStart = t  # underestimates by a little under one frame
            scanInstr1Text.frameNStart = frameN  # exact frame index
            scanInstr1Text.setAutoDraw(True)
        
        # *scanInstr1Resp* updates
        if t >= 0.0 and scanInstr1Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr1Resp.tStart = t  # underestimates by a little under one frame
            scanInstr1Resp.frameNStart = frameN  # exact frame index
            scanInstr1Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr1Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr1Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr1Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr1Resp.rt = scanInstr1Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        
        # *allShapes* updates
        if t >= 0.0 and allShapes.status == NOT_STARTED:
            # keep track of start time/frame for later
            allShapes.tStart = t  # underestimates by a little under one frame
            allShapes.frameNStart = frameN  # exact frame index
            allShapes.setAutoDraw(True)
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr1Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr1"-------
    for thisComponent in scanInstr1Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr1Resp.keys in ['', [], None]:  # No response was made
       scanInstr1Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr1Resp.keys',scanInstr1Resp.keys)
    if scanInstr1Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr1Resp.rt', scanInstr1Resp.rt)
    
    # the Routine "scanInstr1" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr2"-------
    t = 0
    scanInstr2Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr2Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr2Resp.status = NOT_STARTED
    raVerbalText.text = '"For all the shapes you see here, you will have to push the button under your POINTER finger as quickly as possible whenever they appear on the screen. As it will become clear in a second, you do not have to worry about memorizing these shapes."'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    scanInstr2Components = []
    scanInstr2Components.append(scanInstr2Text)
    scanInstr2Components.append(scanInstr2Resp)
    scanInstr2Components.append(goshapes)
    for thisComponent in scanInstr2Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr2"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr2Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr2Text* updates
        if t >= 0.0 and scanInstr2Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr2Text.tStart = t  # underestimates by a little under one frame
            scanInstr2Text.frameNStart = frameN  # exact frame index
            scanInstr2Text.setAutoDraw(True)
        
        # *scanInstr2Resp* updates
        if t >= 0.0 and scanInstr2Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr2Resp.tStart = t  # underestimates by a little under one frame
            scanInstr2Resp.frameNStart = frameN  # exact frame index
            scanInstr2Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr2Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr2Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr2Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr2Resp.rt = scanInstr2Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        
        # *goshapes* updates
        if t >= 0.0 and goshapes.status == NOT_STARTED:
            # keep track of start time/frame for later
            goshapes.tStart = t  # underestimates by a little under one frame
            goshapes.frameNStart = frameN  # exact frame index
            goshapes.setAutoDraw(True)
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr2Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr2"-------
    for thisComponent in scanInstr2Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr2Resp.keys in ['', [], None]:  # No response was made
       scanInstr2Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr2Resp.keys',scanInstr2Resp.keys)
    if scanInstr2Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr2Resp.rt', scanInstr2Resp.rt)
    
    # the Routine "scanInstr2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr3"-------
    t = 0
    scanInstr3Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr3Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr3Resp.status = NOT_STARTED
    raVerbalText.text = '"So, for example, if you see this shape appear on the screen, press the button under your pointer finger as quickly as possible!"'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    scanInstr3Components = []
    scanInstr3Components.append(scanInstr3TextTop)
    scanInstr3Components.append(goShape1)
    scanInstr3Components.append(pointerFinger1)
    scanInstr3Components.append(scanInstr3TextBottom)
    scanInstr3Components.append(scanInstr3Resp)
    for thisComponent in scanInstr3Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr3"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr3Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr3TextTop* updates
        if t >= 0.0 and scanInstr3TextTop.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr3TextTop.tStart = t  # underestimates by a little under one frame
            scanInstr3TextTop.frameNStart = frameN  # exact frame index
            scanInstr3TextTop.setAutoDraw(True)
        
        # *goShape1* updates
        if t >= 0.0 and goShape1.status == NOT_STARTED:
            # keep track of start time/frame for later
            goShape1.tStart = t  # underestimates by a little under one frame
            goShape1.frameNStart = frameN  # exact frame index
            goShape1.setAutoDraw(True)
        
        # *pointerFinger1* updates
        if t >= 0.0 and pointerFinger1.status == NOT_STARTED:
            # keep track of start time/frame for later
            pointerFinger1.tStart = t  # underestimates by a little under one frame
            pointerFinger1.frameNStart = frameN  # exact frame index
            pointerFinger1.setAutoDraw(True)
        
        # *scanInstr3TextBottom* updates
        if t >= 0.0 and scanInstr3TextBottom.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr3TextBottom.tStart = t  # underestimates by a little under one frame
            scanInstr3TextBottom.frameNStart = frameN  # exact frame index
            scanInstr3TextBottom.setAutoDraw(True)
        
        # *scanInstr3Resp* updates
        if t >= 0.0 and scanInstr3Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr3Resp.tStart = t  # underestimates by a little under one frame
            scanInstr3Resp.frameNStart = frameN  # exact frame index
            scanInstr3Resp.status = STARTED
            # keyboard checking is just starting
            event.clearEvents(eventType='keyboard')
        if scanInstr3Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr3Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr3"-------
    for thisComponent in scanInstr3Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # the Routine "scanInstr3" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr4"-------
    t = 0
    scanInstr4Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr4Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr4Resp.status = NOT_STARTED
    raVerbalText.text = '"Or if you see this shape appear on the screen, press the button under your pointer finger as quickly as possible!"'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    scanInstr4Components = []
    scanInstr4Components.append(scanInstr4TextTop)
    scanInstr4Components.append(goShape2)
    scanInstr4Components.append(pointerFinger2)
    scanInstr4Components.append(scanInstr4TextBottom)
    scanInstr4Components.append(scanInstr4Resp)
    for thisComponent in scanInstr4Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr4"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr4Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr4TextTop* updates
        if t >= 0.0 and scanInstr4TextTop.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr4TextTop.tStart = t  # underestimates by a little under one frame
            scanInstr4TextTop.frameNStart = frameN  # exact frame index
            scanInstr4TextTop.setAutoDraw(True)
        
        # *goShape2* updates
        if t >= 0.0 and goShape2.status == NOT_STARTED:
            # keep track of start time/frame for later
            goShape2.tStart = t  # underestimates by a little under one frame
            goShape2.frameNStart = frameN  # exact frame index
            goShape2.setAutoDraw(True)
        
        # *pointerFinger2* updates
        if t >= 0.0 and pointerFinger2.status == NOT_STARTED:
            # keep track of start time/frame for later
            pointerFinger2.tStart = t  # underestimates by a little under one frame
            pointerFinger2.frameNStart = frameN  # exact frame index
            pointerFinger2.setAutoDraw(True)
        
        # *scanInstr4TextBottom* updates
        if t >= 0.0 and scanInstr4TextBottom.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr4TextBottom.tStart = t  # underestimates by a little under one frame
            scanInstr4TextBottom.frameNStart = frameN  # exact frame index
            scanInstr4TextBottom.setAutoDraw(True)
        
        # *scanInstr4Resp* updates
        if t >= 0.0 and scanInstr4Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr4Resp.tStart = t  # underestimates by a little under one frame
            scanInstr4Resp.frameNStart = frameN  # exact frame index
            scanInstr4Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr4Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr4Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr4Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr4Resp.rt = scanInstr4Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr4Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr4"-------
    for thisComponent in scanInstr4Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr4Resp.keys in ['', [], None]:  # No response was made
       scanInstr4Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr4Resp.keys',scanInstr4Resp.keys)
    if scanInstr4Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr4Resp.rt', scanInstr4Resp.rt)
    
    # the Routine "scanInstr4" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr5"-------
    t = 0
    scanInstr5Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr5Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr5Resp.status = NOT_STARTED
    raVerbalText.text = '"However, when you see the circle or the square appear on the screen, you should NOT press any button."'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    scanInstr5Components = []
    scanInstr5Components.append(scanInstr5Text)
    scanInstr5Components.append(squareInstructions)
    scanInstr5Components.append(circleInstructions)
    scanInstr5Components.append(scanInstr5Resp)
    for thisComponent in scanInstr5Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr5"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr5Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr5Text* updates
        if t >= 0.0 and scanInstr5Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr5Text.tStart = t  # underestimates by a little under one frame
            scanInstr5Text.frameNStart = frameN  # exact frame index
            scanInstr5Text.setAutoDraw(True)
        
        # *squareInstructions* updates
        if t >= 0.0 and squareInstructions.status == NOT_STARTED:
            # keep track of start time/frame for later
            squareInstructions.tStart = t  # underestimates by a little under one frame
            squareInstructions.frameNStart = frameN  # exact frame index
            squareInstructions.setAutoDraw(True)
        
        # *circleInstructions* updates
        if t >= 0.0 and circleInstructions.status == NOT_STARTED:
            # keep track of start time/frame for later
            circleInstructions.tStart = t  # underestimates by a little under one frame
            circleInstructions.frameNStart = frameN  # exact frame index
            circleInstructions.setAutoDraw(True)
        
        # *scanInstr5Resp* updates
        if t >= 0.0 and scanInstr5Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr5Resp.tStart = t  # underestimates by a little under one frame
            scanInstr5Resp.frameNStart = frameN  # exact frame index
            scanInstr5Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr5Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr5Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr5Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr5Resp.rt = scanInstr5Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr5Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr5"-------
    for thisComponent in scanInstr5Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr5Resp.keys in ['', [], None]:  # No response was made
       scanInstr5Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr5Resp.keys',scanInstr5Resp.keys)
    if scanInstr5Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr5Resp.rt', scanInstr5Resp.rt)
    
    # the Routine "scanInstr5" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr6"-------
    t = 0
    scanInstr6Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr6Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr6Resp.status = NOT_STARTED
    raVerbalText.text = '"So, for example, if you see the square appear on the screen, do not press any button."'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    scanInstr6Components = []
    scanInstr6Components.append(scanInstr6TextTop)
    scanInstr6Components.append(nogoShape1)
    scanInstr6Components.append(noPress)
    scanInstr6Components.append(scanInstr6TextBottom)
    scanInstr6Components.append(scanInstr6Resp)
    for thisComponent in scanInstr6Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr6"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr6Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr6TextTop* updates
        if t >= 0.0 and scanInstr6TextTop.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr6TextTop.tStart = t  # underestimates by a little under one frame
            scanInstr6TextTop.frameNStart = frameN  # exact frame index
            scanInstr6TextTop.setAutoDraw(True)
        
        # *nogoShape1* updates
        if t >= 0.0 and nogoShape1.status == NOT_STARTED:
            # keep track of start time/frame for later
            nogoShape1.tStart = t  # underestimates by a little under one frame
            nogoShape1.frameNStart = frameN  # exact frame index
            nogoShape1.setAutoDraw(True)
        
        # *noPress* updates
        if t >= 0.0 and noPress.status == NOT_STARTED:
            # keep track of start time/frame for later
            noPress.tStart = t  # underestimates by a little under one frame
            noPress.frameNStart = frameN  # exact frame index
            noPress.setAutoDraw(True)
        
        # *scanInstr6TextBottom* updates
        if t >= 0.0 and scanInstr6TextBottom.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr6TextBottom.tStart = t  # underestimates by a little under one frame
            scanInstr6TextBottom.frameNStart = frameN  # exact frame index
            scanInstr6TextBottom.setAutoDraw(True)
        
        # *scanInstr6Resp* updates
        if t >= 0.0 and scanInstr6Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr6Resp.tStart = t  # underestimates by a little under one frame
            scanInstr6Resp.frameNStart = frameN  # exact frame index
            scanInstr6Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr6Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr6Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr6Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr6Resp.rt = scanInstr6Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr6Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr6"-------
    for thisComponent in scanInstr6Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr6Resp.keys in ['', [], None]:  # No response was made
       scanInstr6Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr6Resp.keys',scanInstr6Resp.keys)
    if scanInstr6Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr6Resp.rt', scanInstr6Resp.rt)
    
    # the Routine "scanInstr6" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr7"-------
    t = 0
    scanInstr7Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr7Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr7Resp.status = NOT_STARTED
    raVerbalText.text = '"Or if you see the circle appear on the screen, do not press any button."'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    scanInstr7Components = []
    scanInstr7Components.append(scanInstr7TextTop)
    scanInstr7Components.append(nogoShape2)
    scanInstr7Components.append(noPress2)
    scanInstr7Components.append(scanInstr7TextBottom)
    scanInstr7Components.append(scanInstr7Resp)
    for thisComponent in scanInstr7Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr7"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr7Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr7TextTop* updates
        if t >= 0.0 and scanInstr7TextTop.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr7TextTop.tStart = t  # underestimates by a little under one frame
            scanInstr7TextTop.frameNStart = frameN  # exact frame index
            scanInstr7TextTop.setAutoDraw(True)
        
        # *nogoShape2* updates
        if t >= 0.0 and nogoShape2.status == NOT_STARTED:
            # keep track of start time/frame for later
            nogoShape2.tStart = t  # underestimates by a little under one frame
            nogoShape2.frameNStart = frameN  # exact frame index
            nogoShape2.setAutoDraw(True)
        
        # *noPress2* updates
        if t >= 0.0 and noPress2.status == NOT_STARTED:
            # keep track of start time/frame for later
            noPress2.tStart = t  # underestimates by a little under one frame
            noPress2.frameNStart = frameN  # exact frame index
            noPress2.setAutoDraw(True)
        
        # *scanInstr7TextBottom* updates
        if t >= 0.0 and scanInstr7TextBottom.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr7TextBottom.tStart = t  # underestimates by a little under one frame
            scanInstr7TextBottom.frameNStart = frameN  # exact frame index
            scanInstr7TextBottom.setAutoDraw(True)
        
        # *scanInstr7Resp* updates
        if t >= 0.0 and scanInstr7Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr7Resp.tStart = t  # underestimates by a little under one frame
            scanInstr7Resp.frameNStart = frameN  # exact frame index
            scanInstr7Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr7Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr7Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr7Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr7Resp.rt = scanInstr7Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr7Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr7"-------
    for thisComponent in scanInstr7Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr7Resp.keys in ['', [], None]:  # No response was made
       scanInstr7Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr7Resp.keys',scanInstr7Resp.keys)
    if scanInstr7Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr7Resp.rt', scanInstr7Resp.rt)
    
    # the Routine "scanInstr7" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr8"-------
    t = 0
    scanInstr8Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    raVerbalText.text = (
        '"To recap: As soon as you see any shape OTHER THAN the circle '
        'or the square, press the button under your POINTER finger as '
        'quickly as possible. The shapes will disappear very quickly, '
        'so try your best to press on time!"')
    raText.text = 'Press <space> to continue.'
    
    raText.pos = [0, -6]
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    npresses = 0
    # keep track of which components have finished
    scanInstr8Components = []
    scanInstr8Components.append(recapShapes)
    for thisComponent in scanInstr8Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr8"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr8Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *recapShapes* updates
        if t >= 0.0 and recapShapes.status == NOT_STARTED:
            # keep track of start time/frame for later
            recapShapes.tStart = t  # underestimates by a little under one frame
            recapShapes.frameNStart = frameN  # exact frame index
            recapShapes.setAutoDraw(True)
        if event.getKeys(keyList = ['space']):
            npresses += 1
            if npresses == 1:
                raVerbalText.text = '"And if you see a circle or a square, DO NOT press any button!\n\nDo you have any questions?"'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            else:
                continueRoutine = False
        
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr8Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr8"-------
    for thisComponent in scanInstr8Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # the Routine "scanInstr8" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "scanInstr9"-------
    t = 0
    scanInstr9Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    scanInstr9Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    scanInstr9Resp.status = NOT_STARTED
    raVerbalText.text = '"Let\'s practice a few trials together."'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    misses = 0
    hits = 0
    false_alarms = 0
    correct_rejects = 0
    
    nogoAcc=0
    goAcc=0
    totalAcc=0
    
    totalAccList = []
    goAccList = []
    nogoAccList = []
    
    goFiveBackAvg = 0
    nogoFiveBackAvg = 0
    totalFiveBackAvg = 0
    # keep track of which components have finished
    scanInstr9Components = []
    scanInstr9Components.append(scanInstr9Text)
    scanInstr9Components.append(scanInstr9Resp)
    for thisComponent in scanInstr9Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "scanInstr9"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = scanInstr9Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *scanInstr9Text* updates
        if t >= 0.0 and scanInstr9Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr9Text.tStart = t  # underestimates by a little under one frame
            scanInstr9Text.frameNStart = frameN  # exact frame index
            scanInstr9Text.setAutoDraw(True)
        
        # *scanInstr9Resp* updates
        if t >= 0.0 and scanInstr9Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            scanInstr9Resp.tStart = t  # underestimates by a little under one frame
            scanInstr9Resp.frameNStart = frameN  # exact frame index
            scanInstr9Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(scanInstr9Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if scanInstr9Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                scanInstr9Resp.keys = theseKeys[-1]  # just the last key pressed
                scanInstr9Resp.rt = scanInstr9Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in scanInstr9Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "scanInstr9"-------
    for thisComponent in scanInstr9Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if scanInstr9Resp.keys in ['', [], None]:  # No response was made
       scanInstr9Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('scanInstr9Resp.keys',scanInstr9Resp.keys)
    if scanInstr9Resp.keys != None:  # we had a response
        pracDirections.addData('scanInstr9Resp.rt', scanInstr9Resp.rt)
    raVerbalText.text = ''
    raText.text = ''
    raVerbalText.draw()
    raText.draw()
    raWin.flip()
    
    # the Routine "scanInstr9" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    exampleTrials = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions(os.path.join('conditions', 'examples.csv')),
        seed=None, name='exampleTrials')
    thisExp.addLoop(exampleTrials)  # add the loop to the experiment
    thisExampleTrial = exampleTrials.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisExampleTrial.rgb)
    if thisExampleTrial != None:
        for paramName in thisExampleTrial.keys():
            exec(paramName + '= thisExampleTrial.' + paramName)
    
    for thisExampleTrial in exampleTrials:
        currentLoop = exampleTrials
        # abbreviate parameter names if possible (e.g. rgb = thisExampleTrial.rgb)
        if thisExampleTrial != None:
            for paramName in thisExampleTrial.keys():
                exec(paramName + '= thisExampleTrial.' + paramName)
        
        #------Prepare to start Routine "exampleTrial"-------
        t = 0
        exampleTrialClock.reset()  # clock 
        frameN = -1
        routineTimer.add(3.100000)
        # update component parameters for each repeat
        shapeExample.setImage(os.path.join('images', stim))
        exampleTrialResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
        exampleTrialResp.status = NOT_STARTED
        number_trials = exampleTrials.thisN + 1
        
        raText.pos = [0, 0]
        shapeExample.frameNStart = None
        fixExampleText.frameNStart = None
        # keep track of which components have finished
        exampleTrialComponents = []
        exampleTrialComponents.append(preshapeFixExampleText)
        exampleTrialComponents.append(shapeExample)
        exampleTrialComponents.append(exampleTrialResp)
        exampleTrialComponents.append(fixExampleText)
        for thisComponent in exampleTrialComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "exampleTrial"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = exampleTrialClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *preshapeFixExampleText* updates
            if t >= 0.0 and preshapeFixExampleText.status == NOT_STARTED:
                # keep track of start time/frame for later
                preshapeFixExampleText.tStart = t  # underestimates by a little under one frame
                preshapeFixExampleText.frameNStart = frameN  # exact frame index
                preshapeFixExampleText.setAutoDraw(True)
            if preshapeFixExampleText.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                preshapeFixExampleText.setAutoDraw(False)
            
            # *shapeExample* updates
            if t >= 1 and shapeExample.status == NOT_STARTED:
                # keep track of start time/frame for later
                shapeExample.tStart = t  # underestimates by a little under one frame
                shapeExample.frameNStart = frameN  # exact frame index
                shapeExample.setAutoDraw(True)
            if shapeExample.status == STARTED and t >= (1 + (.6-win.monitorFramePeriod*0.75)): #most of one frame period left
                shapeExample.setAutoDraw(False)
            
            # *exampleTrialResp* updates
            if t >= 1 and exampleTrialResp.status == NOT_STARTED:
                # keep track of start time/frame for later
                exampleTrialResp.tStart = t  # underestimates by a little under one frame
                exampleTrialResp.frameNStart = frameN  # exact frame index
                exampleTrialResp.status = STARTED
                # AllowedKeys looks like a variable named `allowed_keys`
                if not 'allowed_keys' in locals():
                    logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                    core.quit()
                if not type(allowed_keys) in [list, tuple, np.ndarray]:
                    if not isinstance(allowed_keys, basestring):
                        logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                        core.quit()
                    elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                    else:  allowed_keys = eval(allowed_keys)
                # keyboard checking is just starting
                win.callOnFlip(exampleTrialResp.clock.reset)  # t=0 on next screen flip
                event.clearEvents(eventType='keyboard')
            if exampleTrialResp.status == STARTED and t >= (1 + (0.8-win.monitorFramePeriod*0.75)): #most of one frame period left
                exampleTrialResp.status = STOPPED
            if exampleTrialResp.status == STARTED:
                theseKeys = event.getKeys(keyList=list(allowed_keys))
                
                # check for quit:
                if "escape" in theseKeys:
                    endExpNow = True
                if len(theseKeys) > 0:  # at least one key was pressed
                    exampleTrialResp.keys = theseKeys[-1]  # just the last key pressed
                    exampleTrialResp.rt = exampleTrialResp.clock.getTime()
            
            # *fixExampleText* updates
            if t >= 1.6 and fixExampleText.status == NOT_STARTED:
                # keep track of start time/frame for later
                fixExampleText.tStart = t  # underestimates by a little under one frame
                fixExampleText.frameNStart = frameN  # exact frame index
                fixExampleText.setAutoDraw(True)
            if fixExampleText.status == STARTED and t >= (1.6 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                fixExampleText.setAutoDraw(False)
            
            if ((frameN == 0) or 
                (frameN == shapeExample.frameNStart) or
                (frameN == fixExampleText.frameNStart)):
                winRecorder.keyframe()
            
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in exampleTrialComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "exampleTrial"-------
        for thisComponent in exampleTrialComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        # check responses
        if exampleTrialResp.keys in ['', [], None]:  # No response was made
           exampleTrialResp.keys=None
        # store data for exampleTrials (TrialHandler)
        exampleTrials.addData('exampleTrialResp.keys',exampleTrialResp.keys)
        if exampleTrialResp.keys != None:  # we had a response
            exampleTrials.addData('exampleTrialResp.rt', exampleTrialResp.rt)
        resp=None
        respLast=None
        
        if exampleTrialResp.keys:
            resp = 'go'
            respLast = 'go'
        else:
            resp = None
            respLast = None
        
        if (str(corrAns)==str('go') and resp == str('go')):
            msg="CORRECT!"
            msgDetails = ""
            corrRespMsg='correct'
            corrRespCode = 1
            corrRespTrialType='Hit'
            hits+=1
            goAcc = hits / (hits + misses)
            goAccList.append(corrRespCode)
            totalAccList.append(corrRespCode)
        
        elif (str(corrAns)==str('nogo') and resp == None):
            msg="CORRECT!"
            msgDetails = ""
            corrRespMsg='correct'
            corrRespCode = 1
            corrRespTrialType='corReject'
            correct_rejects+=1
            nogoAcc = correct_rejects / (correct_rejects + false_alarms)
            nogoAccList.append(corrRespCode)
            totalAccList.append(corrRespCode)
        
        elif (str(corrAns)==str('go') and resp == None):
            msg="INCORRECT."
            msgDetails = "PRESS for all shapes except the circle and square."
            corrRespMsg='incorrect'
            corrRespCode = 0
            corrRespTrialType='Miss'
            misses+=1
            goAcc = hits / (hits + misses)
            goAccList.append(corrRespCode)
            totalAccList.append(corrRespCode)
        
        elif (str(corrAns)==str('nogo') and resp == 'go'):
            msg="INCORRECT."
            msgDetails = "Do NOT press for circle or square."
            corrRespMsg='incorrect'
            corrRespCode = 0
            corrRespTrialType='falseAlarm'
            false_alarms+=1
            nogoAcc = correct_rejects / (correct_rejects + false_alarms)
            nogoAccList.append(corrRespCode)
            totalAccList.append(corrRespCode)
        
        totalAcc = (hits + correct_rejects) / (number_trials)
        
        #Calculate 5back stats here
        
        #GoAcc5back
        if (hits + misses)>= 5:
            goFiveBackAvg = 0
            for i in range(-1, -6,-1):
                goFiveBackAvg+=goAccList[i]
            goFiveBackAvg=goFiveBackAvg/5.0
        
        #NoGoAcc5back
        if (correct_rejects + false_alarms) >= 5:
            nogoFiveBackAvg = 0
            for i in range(-1, -6,-1):
                nogoFiveBackAvg+=nogoAccList[i]
            nogoFiveBackAvg=nogoFiveBackAvg/5.0
        
        #TotalAcc5back
        if number_trials >= 5:
            totalFiveBackAvg = 0
            for i in range(-1, -6,-1):
                totalFiveBackAvg+=totalAccList[i]
            totalFiveBackAvg=totalFiveBackAvg/5.0
        
        currentLoop.addData('resp', resp)
        currentLoop.addData('msg', msg)
        currentLoop.addData('corrRespMsg', corrRespMsg)
        currentLoop.addData('corrRespCode', corrRespCode)
        currentLoop.addData('corrRespTrialType', corrRespTrialType)
        currentLoop.addData('hitCount', hits)
        currentLoop.addData('missCount', misses)
        currentLoop.addData('falseAlarmCount', false_alarms)
        currentLoop.addData('corrRejectCount', correct_rejects)
        
        currentLoop.addData('totalAcc', totalAcc)
        currentLoop.addData('goAcc', goAcc)
        currentLoop.addData('nogoAcc', nogoAcc)
        
        currentLoop.addData('goAccList', goAccList)
        currentLoop.addData('nogoAccList', nogoAccList)
        currentLoop.addData('totalAccList', totalAccList)
        
        currentLoop.addData('goFiveBackAvg', goFiveBackAvg)
        currentLoop.addData('nogoFiveBackAvg', nogoFiveBackAvg)
        currentLoop.addData('totalFiveBackAvg', totalFiveBackAvg)
        raTemplate = 'Response: %s.\n\nTotal accuracy: %04.2f %% \nGo accuracy: %04.2f %% \nNoGo accuracy: %04.2f %%'
        raArgs = (corrRespTrialType, totalAcc*100, goAcc*100, nogoAcc*100)
        raText.text = raTemplate % raArgs
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        
        #------Prepare to start Routine "feedback"-------
        t = 0
        feedbackClock.reset()  # clock 
        frameN = -1
        routineTimer.add(2.000000)
        # update component parameters for each repeat
        feedbackText.setText(msg)
        feedbackTextDetails.setText(msgDetails)
        
        # keep track of which components have finished
        feedbackComponents = []
        feedbackComponents.append(feedbackText)
        feedbackComponents.append(feedbackTextDetails)
        for thisComponent in feedbackComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "feedback"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = feedbackClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *feedbackText* updates
            if t >= 0.0 and feedbackText.status == NOT_STARTED:
                # keep track of start time/frame for later
                feedbackText.tStart = t  # underestimates by a little under one frame
                feedbackText.frameNStart = frameN  # exact frame index
                feedbackText.setAutoDraw(True)
            if feedbackText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                feedbackText.setAutoDraw(False)
            
            # *feedbackTextDetails* updates
            if t >= 0.0 and feedbackTextDetails.status == NOT_STARTED:
                # keep track of start time/frame for later
                feedbackTextDetails.tStart = t  # underestimates by a little under one frame
                feedbackTextDetails.frameNStart = frameN  # exact frame index
                feedbackTextDetails.setAutoDraw(True)
            if feedbackTextDetails.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                feedbackTextDetails.setAutoDraw(False)
            if frameN == 0:
                winRecorder.keyframe()
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "feedback"-------
        for thisComponent in feedbackComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        
        thisExp.nextEntry()
        
    # completed 1 repeats of 'exampleTrials'
    
    
    #------Prepare to start Routine "pracInstr10"-------
    t = 0
    pracInstr10Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    pracInstr6Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    pracInstr6Resp.status = NOT_STARTED
    raVerbalText.text = '"Do you have any questions?\n\nNow you will complete a short practice that will look exactly like the game you will play in the scanner.\n\nIn this practice, we will not tell you if you are right or wrong. Just try your best to follow the instructions and play the game."'
    raText.text = 'Press <space> to continue.'
    
    raText.pos = [0, -6]
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    pracInstr10Components = []
    pracInstr10Components.append(pracInstr6Text)
    pracInstr10Components.append(pracInstr6Resp)
    for thisComponent in pracInstr10Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "pracInstr10"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = pracInstr10Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *pracInstr6Text* updates
        if t >= 0.0 and pracInstr6Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            pracInstr6Text.tStart = t  # underestimates by a little under one frame
            pracInstr6Text.frameNStart = frameN  # exact frame index
            pracInstr6Text.setAutoDraw(True)
        
        # *pracInstr6Resp* updates
        if t >= 0.0 and pracInstr6Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            pracInstr6Resp.tStart = t  # underestimates by a little under one frame
            pracInstr6Resp.frameNStart = frameN  # exact frame index
            pracInstr6Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(pracInstr6Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if pracInstr6Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                pracInstr6Resp.keys = theseKeys[-1]  # just the last key pressed
                pracInstr6Resp.rt = pracInstr6Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in pracInstr10Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "pracInstr10"-------
    for thisComponent in pracInstr10Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if pracInstr6Resp.keys in ['', [], None]:  # No response was made
       pracInstr6Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('pracInstr6Resp.keys',pracInstr6Resp.keys)
    if pracInstr6Resp.keys != None:  # we had a response
        pracDirections.addData('pracInstr6Resp.rt', pracInstr6Resp.rt)
    
    # the Routine "pracInstr10" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "pracInstr11"-------
    t = 0
    pracInstr11Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    pracInstr11Resp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    pracInstr11Resp.status = NOT_STARTED
    raVerbalText.text = '"Are you ready to begin the practice?"'
    raText.text = 'Press <space> to continue.\n\nGive feedback as needed. Make sure the participant understands the task.'
    raText.pos = [0, -3]
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    pracInstr11Components = []
    pracInstr11Components.append(pracInstr11Text)
    pracInstr11Components.append(pracInstr11Resp)
    for thisComponent in pracInstr11Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "pracInstr11"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = pracInstr11Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *pracInstr11Text* updates
        if t >= 0.0 and pracInstr11Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            pracInstr11Text.tStart = t  # underestimates by a little under one frame
            pracInstr11Text.frameNStart = frameN  # exact frame index
            pracInstr11Text.setAutoDraw(True)
        
        # *pracInstr11Resp* updates
        if t >= 0.0 and pracInstr11Resp.status == NOT_STARTED:
            # keep track of start time/frame for later
            pracInstr11Resp.tStart = t  # underestimates by a little under one frame
            pracInstr11Resp.frameNStart = frameN  # exact frame index
            pracInstr11Resp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(pracInstr11Resp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if pracInstr11Resp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                pracInstr11Resp.keys = theseKeys[-1]  # just the last key pressed
                pracInstr11Resp.rt = pracInstr11Resp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in pracInstr11Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "pracInstr11"-------
    for thisComponent in pracInstr11Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if pracInstr11Resp.keys in ['', [], None]:  # No response was made
       pracInstr11Resp.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('pracInstr11Resp.keys',pracInstr11Resp.keys)
    if pracInstr11Resp.keys != None:  # we had a response
        pracDirections.addData('pracInstr11Resp.rt', pracInstr11Resp.rt)
    
    # the Routine "pracInstr11" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed mode == 'practice' repeats of 'pracDirections'


# set up handler to look after randomisation of conditions etc
scanDirections = data.TrialHandler(nReps=mode != 'practice', method='sequential', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='scanDirections')
thisExp.addLoop(scanDirections)  # add the loop to the experiment
thisScanDirection = scanDirections.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisScanDirection.rgb)
if thisScanDirection != None:
    for paramName in thisScanDirection.keys():
        exec(paramName + '= thisScanDirection.' + paramName)

for thisScanDirection in scanDirections:
    currentLoop = scanDirections
    # abbreviate parameter names if possible (e.g. rgb = thisScanDirection.rgb)
    if thisScanDirection != None:
        for paramName in thisScanDirection.keys():
            exec(paramName + '= thisScanDirection.' + paramName)
    
    #------Prepare to start Routine "recapInstr"-------
    t = 0
    recapInstrClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    recapInstrResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    recapInstrResp.status = NOT_STARTED
    raVerbalText.text = '"As soon as you see any shape OTHER THAN the circle or the square, press the button under your POINTER finger as quickly as possible. If you see a circle or a square, DO NOT press any button!"'
    raText.text = 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    recapInstrComponents = []
    recapInstrComponents.append(recapInstrText)
    recapInstrComponents.append(recapInstrResp)
    recapInstrComponents.append(recapShapes2)
    for thisComponent in recapInstrComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "recapInstr"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = recapInstrClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *recapInstrText* updates
        if t >= 0.0 and recapInstrText.status == NOT_STARTED:
            # keep track of start time/frame for later
            recapInstrText.tStart = t  # underestimates by a little under one frame
            recapInstrText.frameNStart = frameN  # exact frame index
            recapInstrText.setAutoDraw(True)
        
        # *recapInstrResp* updates
        if t >= 0.0 and recapInstrResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            recapInstrResp.tStart = t  # underestimates by a little under one frame
            recapInstrResp.frameNStart = frameN  # exact frame index
            recapInstrResp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(recapInstrResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if recapInstrResp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                recapInstrResp.keys = theseKeys[-1]  # just the last key pressed
                recapInstrResp.rt = recapInstrResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        
        # *recapShapes2* updates
        if t >= 0.0 and recapShapes2.status == NOT_STARTED:
            # keep track of start time/frame for later
            recapShapes2.tStart = t  # underestimates by a little under one frame
            recapShapes2.frameNStart = frameN  # exact frame index
            recapShapes2.setAutoDraw(True)
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in recapInstrComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "recapInstr"-------
    for thisComponent in recapInstrComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if recapInstrResp.keys in ['', [], None]:  # No response was made
       recapInstrResp.keys=None
    # store data for scanDirections (TrialHandler)
    scanDirections.addData('recapInstrResp.keys',recapInstrResp.keys)
    if recapInstrResp.keys != None:  # we had a response
        scanDirections.addData('recapInstrResp.rt', recapInstrResp.rt)
    raText.text =''
    raText.draw()
    raWin.flip()
    
    # the Routine "recapInstr" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed mode != 'practice' repeats of 'scanDirections'


# set up handler to look after randomisation of conditions etc
runs = data.TrialHandler(nReps=nRuns, method='random', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='runs')
thisExp.addLoop(runs)  # add the loop to the experiment
thisRun = runs.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisRun.rgb)
if thisRun != None:
    for paramName in thisRun.keys():
        exec(paramName + '= thisRun.' + paramName)

for thisRun in runs:
    currentLoop = runs
    # abbreviate parameter names if possible (e.g. rgb = thisRun.rgb)
    if thisRun != None:
        for paramName in thisRun.keys():
            exec(paramName + '= thisRun.' + paramName)
    
    #------Prepare to start Routine "waitingScanner"-------
    t = 0
    waitingScannerClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    runNum = runNumber + runs.thisN
    
    events = []
    
    # Reset Position of RA Text
    raVerbalText.text = ''
    raText.text = ''
    
    raText.pos = [0, 0]
    
    raText.draw()
    raVerbalText.draw()
    raWin.flip()
    raText.text = 'Start the scan.'
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    waitingScannerComponents = []
    waitingScannerComponents.append(waitScannerText)
    for thisComponent in waitingScannerComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "waitingScanner"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = waitingScannerClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *waitScannerText* updates
        if t >= 0.0 and waitScannerText.status == NOT_STARTED:
            # keep track of start time/frame for later
            waitScannerText.tStart = t  # underestimates by a little under one frame
            waitScannerText.frameNStart = frameN  # exact frame index
            waitScannerText.setAutoDraw(True)
        
        if expInfo['mriMode'] != 'Off':
            if trigger == 'usb':
                vol = launchScan(win, MR_settings, 
                      globalClock=fmriClock, 
                      mode=expInfo['mriMode'],
                      wait_msg = '')
                win.flip()
        
            elif trigger == 'parallel':
                address = 0x378
                #parallel.setPortAddress(0x378)
                wait_msg = "Get ready!"
                pinStatus = winioport.inp(address)
                waitMsgStim = visual.TextStim(win, color='white', text=wait_msg)
                waitMsgStim.draw()
                win.flip()
                while True:
                    if pinStatus != winioport.inp(address):
                       break
                       # start exp when pin values change
                fmriClock.reset()
                logging.exp('parallel trigger: start of scan')
                win.flip()  # blank the screen on first sync pulse received
        else:
            fmriClock.reset()
        
        triggerWallTime = datetime.datetime.today()
        expInfo['triggerWallTime'] = triggerWallTime.strftime('%a %b %d %H:%M:%S %Y')
        expInfo['triggerGlobalTime'] = globalClock.getTime()
        continueRoutine = False
        
        filename = _thisDir + os.sep + u'data/' + '_'.join([expName, expInfo['sessionID'], 'run%s'%runNum, triggerWallTime.strftime('%Y-%m-%d_%H%M%S')])
        # winRecorder.keyframe()
        continueRoutine = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in waitingScannerComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "waitingScanner"-------
    for thisComponent in waitingScannerComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    raText.text = ''
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # the Routine "waitingScanner" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    countdowns = data.TrialHandler(nReps=4, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='countdowns')
    thisExp.addLoop(countdowns)  # add the loop to the experiment
    thisCountdown = countdowns.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisCountdown.rgb)
    if thisCountdown != None:
        for paramName in thisCountdown.keys():
            exec(paramName + '= thisCountdown.' + paramName)
    
    for thisCountdown in countdowns:
        currentLoop = countdowns
        # abbreviate parameter names if possible (e.g. rgb = thisCountdown.rgb)
        if thisCountdown != None:
            for paramName in thisCountdown.keys():
                exec(paramName + '= thisCountdown.' + paramName)
        
        #------Prepare to start Routine "countdown"-------
        t = 0
        countdownClock.reset()  # clock 
        frameN = -1
        routineTimer.add(2.000000)
        # update component parameters for each repeat
        countdownTime = fmriClock.getTime()
        countdownMsg = currentLoop.nRemaining + 1
        
        countdownText.setText(countdownMsg)
        condFile = 'conditions/%s%s.csv' % (mode, runNum)
        currentLoop.addData('condFile', condFile)
        
        misses = 0
        hits = 0
        false_alarms = 0
        correct_rejects = 0
        
        nogoAcc=0
        goAcc=0
        totalAcc=0
        
        totalAccList = []
        goAccList = []
        nogoAccList = []
        
        goFiveBackAvg = 0
        nogoFiveBackAvg = 0
        totalFiveBackAvg = 0
        
        # keep track of which components have finished
        countdownComponents = []
        countdownComponents.append(countdownText)
        for thisComponent in countdownComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "countdown"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = countdownClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            
            
            # *countdownText* updates
            if t >= 0.0 and countdownText.status == NOT_STARTED:
                # keep track of start time/frame for later
                countdownText.tStart = t  # underestimates by a little under one frame
                countdownText.frameNStart = frameN  # exact frame index
                countdownText.setAutoDraw(True)
            if countdownText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                countdownText.setAutoDraw(False)
            
            if frameN == 0:
                winRecorder.keyframe()
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in countdownComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "countdown"-------
        for thisComponent in countdownComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        currentLoop.addData('countdownStartTime', countdownTime)
        currentLoop.addData('countdownLabel', countdownMsg)
        currentLoop.addData('trialNum', 0)
        
        
        thisExp.nextEntry()
        
    # completed 4 repeats of 'countdowns'
    
    
    # set up handler to look after randomisation of conditions etc
    trials = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions(condFile),
        seed=None, name='trials')
    thisExp.addLoop(trials)  # add the loop to the experiment
    thisTrial = trials.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisTrial.rgb)
    if thisTrial != None:
        for paramName in thisTrial.keys():
            exec(paramName + '= thisTrial.' + paramName)
    
    for thisTrial in trials:
        currentLoop = trials
        # abbreviate parameter names if possible (e.g. rgb = thisTrial.rgb)
        if thisTrial != None:
            for paramName in thisTrial.keys():
                exec(paramName + '= thisTrial.' + paramName)
        
        #------Prepare to start Routine "trial"-------
        t = 0
        trialClock.reset()  # clock 
        frameN = -1
        routineTimer.add(0.800000)
        # update component parameters for each repeat
        number_trials = trials.thisN + 1
        currentLoop.addData('trialNum', number_trials)
        
        raUpdated = False
        shouldWriteTrial = True
        
        currentLoop.addData('condFile', condFile)
        
        '''Nogo Shapes are listed in the conditions.csv using labels 
        (prevRew or neutral) while Go shapes are specified by shape 
        (e.g. hexagon.png). Substitue the correct Nogo shape here given 
        the previously rewarded shape.'''
        
        if corrAns == 'nogo':
            # Shape Lookup from reward counterbalance condition
            if expInfo['prevRewShape'] == 'na':
                nogoCondition = 'Nogo'
            else:
                nogoCondition = stim
            stim = rewShapes[stim]
            currentLoop.addData('stim', stim)
            thisTrial['stim'] = stim
        #    thisTrial['stim'] = '.png'
        else:
            nogoCondition = None  # undefined for go trials
            # stim = stim  # defined in conditions file
        
        shapeLabel = stim[:-4]  # strip '.png'
        trialShape = shapeStims[shapeLabel]
        trialShape.autoDraw = True
        
        # Force a non-slip builder routine of "unknown" duration
        # Base fixed duration is .2s (the extra allowable response window),
        # then add the remaining amount here.
        isiToAdd = ISI - 0.2
        routineTimer.add(isiToAdd)
        fixStartTime = None
        
        trialKeys = []
        trialRTs = []
        trialRTs_rel = []
        
        isiKeys = []
        isiRTs = []
        isiRTs_rel = []
        
        # keep track of which components have finished
        trialComponents = []
        trialComponents.append(dummyISIText)
        for thisComponent in trialComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "trial"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = trialClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            if frameN > responseWindowFrames and raUpdated == False:
                if mode == 'practice':
                    totalTrials = 16
                else:
                    totalTrials = 92
            
                resp=None
                respLast=None
            
                if len(trialKeys):
                    resp = 'go'
                    respFirstKey = trialKeys[0]
                    respFirstRt = trialRTs[0]
                    respLast = 'go'
                else: #no press
                    trialKeys = None
                    trialRTs = None
                    resp = None
                    respLast = None
                    respFirstKey = None
                    respFirstRt = None
            
                if (str(corrAns) == str('go') and resp == str('go')):
                    msg="Correct!"
                    corrRespMsg='correct'
                    corrRespCode = 1
                    corrRespTrialType='Hit'
                    hits+=1
                    goAcc = hits / (hits + misses)
                    goAccList.append(corrRespCode)
                    totalAccList.append(corrRespCode)
            
                elif (str(corrAns) == str('nogo') and resp == None):
                    msg="Correct!"
                    corrRespMsg='correct'
                    corrRespCode = 1
                    corrRespTrialType='corReject'
                    correct_rejects+=1
                    nogoAcc = correct_rejects / (correct_rejects + false_alarms)
                    nogoAccList.append(corrRespCode)
                    totalAccList.append(corrRespCode)
            
                elif (str(corrAns) == str('go') and resp == None):
                    msg="Incorrect. PRESS for all shapes other than circle and square."
                    corrRespMsg='incorrect'
                    corrRespCode = 0
                    corrRespTrialType='Miss'
                    misses+=1
                    goAcc = hits / (hits + misses)
                    goAccList.append(corrRespCode)
                    totalAccList.append(corrRespCode)
            
                elif (str(corrAns) == str('nogo') and resp == 'go'):
                    msg="Incorrect. Do NOT press for circle or square."
                    corrRespMsg='incorrect'
                    corrRespCode = 0
                    corrRespTrialType='falseAlarm'
                    false_alarms+=1
                    nogoAcc = correct_rejects / (correct_rejects + false_alarms)
                    nogoAccList.append(corrRespCode)
                    totalAccList.append(corrRespCode)
            
                totalAcc = (hits + correct_rejects) / (trials.thisN+1)
            
                if corrRespCode:
                    if corrAns == 'go':
                        trialOutcome = 'go'
                    else:
                        trialOutcome = nogoCondition
                else:
                    trialOutcome = 'errorOrMiss'
            
                #Calculate 5back stats here
            
                #GoAcc5back
                if (hits + misses)>= 5:
                    goFiveBackAvg = 0
                    for i in range(-1, -6,-1):
                        goFiveBackAvg+=goAccList[i]
                    goFiveBackAvg=goFiveBackAvg/5.0
            
                #NoGoAcc5back
                if (correct_rejects + false_alarms) >= 5:
                    nogoFiveBackAvg = 0
                    for i in range(-1, -6,-1):
                        nogoFiveBackAvg+=nogoAccList[i]
                    nogoFiveBackAvg=nogoFiveBackAvg/5.0
            
                #TotalAcc5back
                if number_trials >= 5:
                    totalFiveBackAvg = 0
                    for i in range(-1, -6,-1):
                        totalFiveBackAvg+=totalAccList[i]
                    totalFiveBackAvg=totalFiveBackAvg/5.0
            
            
                currentLoop.addData('trialOutcome', trialOutcome)
                currentLoop.addData('resp', resp)
                currentLoop.addData('respLast', respLast)
                currentLoop.addData('trialResp.keys', trialKeys)
                currentLoop.addData('trialResp.rt', trialRTs)
                currentLoop.addData('trialResp_rel.rt', trialRTs_rel)
                currentLoop.addData('trialResp.firstKey', respFirstKey)
                currentLoop.addData('trialResp.firstRt', respFirstRt)
                currentLoop.addData('msg', msg)
                currentLoop.addData('corrRespMsg', corrRespMsg)
                currentLoop.addData('corrRespCode', corrRespCode)
                currentLoop.addData('corrRespTrialType', corrRespTrialType)
                currentLoop.addData('hitCount', hits)
                currentLoop.addData('missCount', misses)
                currentLoop.addData('falseAlarmCount', false_alarms)
                currentLoop.addData('corrRejectCount', correct_rejects)
            
                currentLoop.addData('totalAcc', totalAcc)
                currentLoop.addData('goAcc', goAcc)
                currentLoop.addData('nogoAcc', nogoAcc)
            
                currentLoop.addData('goAccList', goAccList)
                currentLoop.addData('nogoAccList', nogoAccList)
                currentLoop.addData('totalAccList', totalAccList)
            
                currentLoop.addData('goFiveBackAvg', goFiveBackAvg)
                currentLoop.addData('nogoFiveBackAvg', nogoFiveBackAvg)
                currentLoop.addData('totalFiveBackAvg', totalFiveBackAvg)
            
                currentLoop.addData('nogoCondition', nogoCondition)
            
                events.append({'run': runNum,
                               'condition': trialOutcome,
                               'onset': shapeStartTime,
                               'duration': .6,})
            
            
                raText.text = 'Trial # %s of %d in this round.\n\nResponse: %s\n\nTotal accuracy: %04.2f %% \nGo accuracy: %04.2f %% \nNoGo accuracy: %04.2f %% \n\nTotal accuracy last 5 trials: %04.2f %% \nGo accuracy last 5 trials: %04.2f %% \nNoGo accuracy last 5 trials: %04.2f %%' % (
                    number_trials, totalTrials, corrRespTrialType, totalAcc*100, goAcc*100, nogoAcc*100, totalFiveBackAvg*100, goFiveBackAvg*100, nogoFiveBackAvg*100)
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
                raUpdated = True
            
            # Update trial log during ISI
            if corrAns == 'nogo' and shouldWriteTrial and t > 1:
                updateRunLog(thisExp.entries)
                shouldWriteTrial = False
            if frameN == 0:
                shapeStartTime = fmriClock.getTime()
                winRecorder.keyframe()
            
            if frameN >= shapeFrames and trialShape.autoDraw == True:  # (36 * 16 2/3 == 600ms) 
                trialShape.autoDraw = False
                shapeEndTime = fmriClock.getTime()
                winRecorder.keyframe()
            
            if frameN == 0:
                trialComponents.append(extraISIStim)
                extraISIStim.status = STARTED
            
            if frameN >= shapeFrames and not fixStartTime:
                fixText.autoDraw = True
                fixStartTime = fmriClock.getTime()
                winRecorder.keyframe()
            
            framePresses = event.getKeys(keyList=allowed_keys,
                                         timeStamped=fmriClock)
            if framePresses:
                if frameN < responseWindowFrames:  # 800ms
                    trialKeys.extend([l[0] for l in framePresses])
                    trialRTs.extend([l[1] for l in framePresses])
            
                    # RT for CARIT (only) was initially distributed relative to 
                    # task start time. Add a time relative to the start of
                    # the _trial_ as well.
                    trialRTs_rel.extend([l[1] - shapeStartTime for l in framePresses])
                else:
                    isiKeys.extend([l[0] for l in framePresses])
                    isiRTs.extend([l[1] - fixStartTime for l in framePresses])
                    isiRTs_rel.extend([l[1] - shapeStartTime for l in framePresses])
            
            
            # *dummyISIText* updates
            if t >= 0.0 and dummyISIText.status == NOT_STARTED:
                # keep track of start time/frame for later
                dummyISIText.tStart = t  # underestimates by a little under one frame
                dummyISIText.frameNStart = frameN  # exact frame index
                dummyISIText.setAutoDraw(True)
            if dummyISIText.status == STARTED and t >= (0.0 + (.8-win.monitorFramePeriod*0.75)): #most of one frame period left
                dummyISIText.setAutoDraw(False)
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in trialComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "trial"-------
        for thisComponent in trialComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        
        
        
        currentLoop.addData('shapeStartTime', shapeStartTime)
        currentLoop.addData('shapeEndTime', shapeEndTime)
        
        fixText.autoDraw = False
        fixEndTime = fmriClock.getTime()
        
        currentLoop.addData('isiPress.keys', isiKeys)
        currentLoop.addData('isiPress.rt', isiRTs)
        currentLoop.addData('isiPress_rel.rt', isiRTs_rel)
        currentLoop.addData('fixStartTime', fixStartTime)
        currentLoop.addData('fixEndTime', fixEndTime)
        
        thisExp.nextEntry()
        
    # completed 1 repeats of 'trials'
    
    
    #------Prepare to start Routine "endRun"-------
    t = 0
    endRunClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    endRunResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    endRunResp.status = NOT_STARTED
    if mode == 'practice':
        shouldContinue = False
    else:
        shouldContinue = True
    runEndTime = None
    
    
    # keep track of which components have finished
    endRunComponents = []
    endRunComponents.append(endRunFixation)
    endRunComponents.append(endRunText)
    endRunComponents.append(endRunResp)
    for thisComponent in endRunComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "endRun"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = endRunClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *endRunFixation* updates
        if t >= 0.0 and endRunFixation.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunFixation.tStart = t  # underestimates by a little under one frame
            endRunFixation.frameNStart = frameN  # exact frame index
            endRunFixation.setAutoDraw(True)
        if endRunFixation.status == STARTED and (mode == 'practice' or fmriClock.getTime() >= runDuration):
            endRunFixation.setAutoDraw(False)
        
        # *endRunText* updates
        if (mode == 'practice' or fmriClock.getTime() >= runDuration) and endRunText.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunText.tStart = t  # underestimates by a little under one frame
            endRunText.frameNStart = frameN  # exact frame index
            endRunText.setAutoDraw(True)
        
        # *endRunResp* updates
        if (mode == 'practice' or fmriClock.getTime() >= runDuration) and endRunResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunResp.tStart = t  # underestimates by a little under one frame
            endRunResp.frameNStart = frameN  # exact frame index
            endRunResp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(endRunResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if endRunResp.status == STARTED:
            theseKeys = event.getKeys(keyList=['y', 'n', 'left', 'right', 'space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                endRunResp.keys = theseKeys[-1]  # just the last key pressed
                endRunResp.rt = endRunResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            continueRoutine = shouldContinue
        
        if not runEndTime:
            if mode == 'practice' or fmriClock.getTime() >= runDuration:
                runEndTime = fmriClock.getTime()
                currentLoop.addData('runEndTime', runEndTime)
        
                events.append({'run': runNum,
                               'condition': 'runEnd',
                               'onset': runEndTime,
                               'duration': 0,})
                events_df = pd.DataFrame(events)
                designOutfile = '%s_design.csv' % filename
                events_df.to_csv(designOutfile, index=False)
        
                entries = thisExp.entries
                entries[-1]['runEndTime'] = runEndTime
                updateRunLog(entries)
        
        if mode == 'practice' or fmriClock.getTime() >= runDuration:
            if raText.text != 'Press <space> to continue.':
                raText.text = 'Press <space> to continue.'
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in endRunComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "endRun"-------
    for thisComponent in endRunComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if endRunResp.keys in ['', [], None]:  # No response was made
       endRunResp.keys=None
    # store data for runs (TrialHandler)
    runs.addData('endRunResp.keys',endRunResp.keys)
    if endRunResp.keys != None:  # we had a response
        runs.addData('endRunResp.rt', endRunResp.rt)
    
    raText.text=''
    raText.draw()
    raWin.flip()
    # the Routine "endRun" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed nRuns repeats of 'runs'


#------Prepare to start Routine "end"-------
t = 0
endClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat
endResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
endResp.status = NOT_STARTED
raText.text='Press <space> to continue.'
raText.draw()
raWinRecorder.keyframe()
raWin.flip()

# keep track of which components have finished
endComponents = []
endComponents.append(endText)
endComponents.append(endResp)
for thisComponent in endComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "end"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = endClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    # *endText* updates
    if t >= 0.0 and endText.status == NOT_STARTED:
        # keep track of start time/frame for later
        endText.tStart = t  # underestimates by a little under one frame
        endText.frameNStart = frameN  # exact frame index
        endText.setAutoDraw(True)
    
    # *endResp* updates
    if t >= 0.0 and endResp.status == NOT_STARTED:
        # keep track of start time/frame for later
        endResp.tStart = t  # underestimates by a little under one frame
        endResp.frameNStart = frameN  # exact frame index
        endResp.status = STARTED
        # keyboard checking is just starting
        win.callOnFlip(endResp.clock.reset)  # t=0 on next screen flip
        event.clearEvents(eventType='keyboard')
    if endResp.status == STARTED:
        theseKeys = event.getKeys(keyList=['y', 'n', 'left', 'right', 'space'])
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            endResp.keys = theseKeys[-1]  # just the last key pressed
            endResp.rt = endResp.clock.getTime()
            # a response ends the routine
            continueRoutine = False
    if frameN == 0:
        winRecorder.keyframe()
    
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in endComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "end"-------
for thisComponent in endComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)
# check responses
if endResp.keys in ['', [], None]:  # No response was made
   endResp.keys=None
# store data for thisExp (ExperimentHandler)
thisExp.addData('endResp.keys',endResp.keys)
if endResp.keys != None:  # we had a response
    thisExp.addData('endResp.rt', endResp.rt)
thisExp.nextEntry()

# the Routine "end" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()





now = datetime.datetime.now().strftime('%Y_%m_%d_%H%M')
win_fname = 'win-movie-{}.png'.format(now)
rawin_fname = 'rawin-movie-{}.png'.format(now)

# Save stills and convert to .mov
winRecorder.save(win_fname)
raWinRecorder.save(rawin_fname)
hcpcommon.WinRecorder.save_sidebyside([winRecorder, raWinRecorder], 
                                      'EXAMPLE_{}.mov'.format(now))





























# these shouldn't be strictly necessary (should auto-save)
thisExp.saveAsPickle(filename)
logging.flush()
# make sure everything is closed down
thisExp.abort() # or data files will save again on exit
win.close()
core.quit()
