#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Part of the Human Connectome - Lifespan Project Task fMRI Battery
Psychopy implementation by Erik K Kastman, Constanza M Vidal Bustamente, and Leah H Somerville
**********************************************************************************************
Tested with Psychopy 1.83.04; Psyexp Experiment Builder XML available upon request
  Peirce, JW (2007) PsychoPy - Psychophysics software in Python. Journal of Neuroscience Methods, 162(1-2), 8-13.
  Peirce, JW (2009) Generating stimuli for neuroscience using PsychoPy. Frontiers in Neuroinformatics, 2:10. doi: 10.3389/neuro.11.010.2008
"""

from __future__ import division  # so that 1/3=0.333 instead of 1/3=0
import datetime
from psychopy import locale_setup, visual, core, data, event, logging, sound, gui
from psychopy.constants import *  # things like STARTED, FINISHED
import numpy as np  # whole numpy lib is available, prepend 'np.'
from numpy import sin, cos, tan, log, log10, pi, average, sqrt, std, deg2rad, rad2deg, linspace, asarray
from numpy.random import random, randint, normal, shuffle
import os  # handy system and path functions
import sys # to get file system encoding

# Common HCP Tools
import hcpcommon
hcpcommon.checkScreenCount()
# Ensure that relative paths start from the same directory as this script
_thisDir = os.path.dirname(os.path.abspath(__file__)).decode(sys.getfilesystemencoding())
os.chdir(_thisDir)

# Store info about the experiment session
expName = 'GUESSING'  # from the Builder filename that created this script
expInfo = {u'sessionID': u'', u'runNumber': u'1', u'mriMode': u'scan', u'nRuns': u'1', u'shape': u'circle'}
dlg = gui.DlgFromDict(dictionary=expInfo, title=expName)
if dlg.OK == False: core.quit()  # user pressed cancel
expInfo['date'] = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')  # add a simple timestamp
expInfo['expName'] = expName

# Data file name stem = absolute path + name; later add .psyexp, .csv, .log, etc
filename = _thisDir + os.sep + 'data' + os.path.sep + '_'.join([expName, expInfo['sessionID'], 'run'+expInfo['runNumber'], expInfo['date']])

# An ExperimentHandler isn't essential but helps with data saving
thisExp = data.ExperimentHandler(name=expName, version='',
    extraInfo=expInfo, runtimeInfo=None,
    originPath=None,
    savePickle=True, saveWideText=False,
    dataFileName=filename)
#save a log file for detail verbose info
logFile = logging.LogFile(filename+'.log', level=logging.EXP)
logging.console.setLevel(logging.WARNING)  # this outputs to the screen, not a file

endExpNow = False  # flag for 'escape' or other condition => quit the exp

# Start Code - component code to be run before the window creation
## EKK Screen Edit of compiled script
import pyglet
display = pyglet.window.get_platform().get_default_display()
screens = display.get_screens()
resolution = [screens[-1].width, screens[-1].height]
import yaml
from psychopy import monitors
if not os.path.exists('siteConfig.yaml'): raise IOError('Missing siteConfig.yaml - Please copy configuration text file')
with open('siteConfig.yaml') as f:
    config = yaml.safe_load(f)
mon = monitors.Monitor('newMonitor')
mon.setWidth(config['monitor']['width'])
mon.setDistance(config['monitor']['distance'])
mon.setSizePix(resolution)

# Setup the Window
win = visual.Window(size=resolution, fullscr=False, screen=config['monitor']['screen'], allowGUI=False, allowStencil=False,
    monitor=mon, color=[-1,-1,-1], colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg')
# store frame rate of monitor if we can measure it successfully
expInfo['frameRate']=win.getActualFrameRate()
if expInfo['frameRate']!=None:
    frameDur = 1.0/round(expInfo['frameRate'])
else:
    frameDur = 1.0/60.0 # couldn't get a reliable measure so guess

# Initialize components for Routine "intro"
introClock = core.Clock()
import yaml
import itertools
import datetime

if os.path.exists('siteConfig.yaml'):
    with open('siteConfig.yaml') as f:
        config = yaml.safe_load(f)
else:
    copyMsg = ('Please copy configuration text file '
               '"siteConfig.yaml.example" to "siteConfig.yaml" '
               'and edit it with your trigger and buttons.')
    raise IOError(copyMsg)

logging.exp('Using Site Configuration: %s' % config)
allowed_keys = list(itertools.chain.from_iterable(
    config['keys'].values()))
all_allowed_keys = allowed_keys + ['space']

# Invert Key dictionary for "meaning" of keypress
guessLookup = dict()
for guess, keys in config['keys'].iteritems():
    for key in keys:
        guessLookup[key] = guess

# n.b. Shape is specified without extension in Guessing
# and with extension in CARIT.
if expInfo['shape'] in ['circle', 'c']:
    expInfo['rewShape'] = 'circle'
elif expInfo['shape'] in ['square', 's']:
    expInfo['rewShape'] = 'square'
else:
    raise ValueError('counterbalance must be either "circle" or "square"')
del(expInfo['shape']) # Cleanup logfile

runNumber = expInfo['runNumber']

if runNumber == 'practice':
    nRuns = 99
    runNumber = 0
else:
    nRuns = expInfo['nRuns']
    try:
        runNumber = int(runNumber)
    except ValueError:
        raise ValueError('runNumber must be either "practice" or a whole number, but it was "%s"\n' % runNumber)

titleLetterSize = config['style']['titleLetterSize']  # 3
subtitleLetterSize = config['style']['subtitleLetterSize']  # .7
textLetterSize = config['style']['textLetterSize']  # 1
fixLetterSize = config['style']['fixLetterSize']  # 2.5
wrapWidth = config['style']['wrapWidth']  # 30
verbalColor = config['style']['verbalColor']  # '#3EB4F0'

# Other Custom Sizes
qmarkSize = 5

try:
    age = float(expInfo['age'])
except ValueError:
    raise ValueError("Please enter age in years")
except KeyError:
    age = np.nan

# Initialize no responses and practice trial dataframe 
# (resets at start of run, but declare initially for practice)
nNoResp = 0

# Initialize a cumulative reward for all runs
totalCumulativeReward = 0
introText = visual.TextStim(win=win, ori=0, name='introText',
    text='GUESSING',    font='Arial',
    pos=[0, 2], height=titleLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
introText2 = visual.TextStim(win=win, ori=0, name='introText2',
    text='GAME',    font='Arial',
    pos=[0, -2], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
# Setup the RA Experimenter Window
raWin = visual.Window(size=[1280,800], fullscr=False, allowGUI=True, allowStencil=False,
    monitor=u'testMonitor', color=u'black', colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg', name='raWin',
    screen=0)

raText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[0, -3], height=textLetterSize * 1.5, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raVerbalText = visual.TextStim(win=raWin, ori=0, name='raVerbalText',
    text='', font='Arial',
    pos=[0,3], height=textLetterSize * 1.5, wrapWidth=30,
    color=verbalColor, colorSpace='rgb', opacity=1,
    depth=0.0, italic=True)

# Record Git Status in Subject Logs
from subprocess import check_output, check_call, CalledProcessError
try:
    revision = check_output(['git','rev-parse', '--short', 'HEAD'])
    revision = revision.strip()
    gitInstalled = True
except:
    if os.path.exists('VERSION'):
        with open('VERSION', 'r') as f:
            revision = f.read()
    else:
        revision = ''
    gitInstalled = False

if gitInstalled:
    try:
        check_call(['git', 'diff-files', '--quiet'])
    except CalledProcessError:
        revision = 'Task changes detected. Nearest head: %s' % revision
        status_msg = check_output(['git', 'status'])
        msg ="""%s

Warning: the experiment has unexpected changes.
""" % status_msg
        logging.critical(msg)
        # core.quit()

expInfo['git-revision'] = revision

outerFrame = visual.Rect(win=win, lineWidth=1, lineColor='white',
                         width=16.4, height=12.4, units='deg')

pointerXoffset = 5
pointerYoffset = -4.5
babyPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='Baby', pos=(-pointerXoffset, pointerYoffset))
pointerPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='POINTER FINGER', pos=(-pointerXoffset, pointerYoffset-1))
adultPrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='Adult', pos=(pointerXoffset, pointerYoffset))
middlePrompt = visual.TextStim(win=win, height=subtitleLetterSize,
                             text='MIDDLE FINGER', pos=(pointerXoffset, pointerYoffset-1))

buttonPrompts = [babyPrompt, pointerPrompt, adultPrompt, middlePrompt]

#outerFrame.autoDraw = True

import hcpcommon
movieClock = core.Clock()

record_example, cleanup = False, True

winRecorder = hcpcommon.WinRecorder(win=win, 
                                    clock=movieClock, 
                                    record=record_example,
                                    cleanup=cleanup)

raWinRecorder = hcpcommon.WinRecorder(win=raWin, 
                                      clock=movieClock, 
                                      record=record_example,
                                      cleanup=cleanup)

def save_movie(filename, recorder1, recorder2):
    win_fname = filename + '_win-movie.png'
    rawin_fname = filename + '_rawin-movie.png'

    # Save stills and convert to .mov
    recorder1.save(win_fname)
    recorder2.save(rawin_fname)
    movie_outfile = filename + '_EXAMPLE.mov'
    hcpcommon.WinRecorder.save_sidebyside([recorder1, recorder2], 
                                          movie_outfile)
    print('Saved %s' % movie_outfile)
    return movie_outfile


# Initialize components for Routine "directionsAdult"
directionsAdultClock = core.Clock()
textAdult = visual.TextStim(win=win, ori=0, name='textAdult',
    text='These are adults',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
imageAdult1 = visual.ImageStim(win=win, name='imageAdult1',
    image='images/Adult1.jpg', mask=None,
    ori=0, pos=[-3, 0], size=[2.56, 8],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
imageAdult2 = visual.ImageStim(win=win, name='imageAdult2',
    image='images/Adult2.jpg', mask=None,
    ori=0, pos=[3, 0], size=[2.56, 8],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)


# Initialize components for Routine "directionsBaby"
directionsBabyClock = core.Clock()
imageBaby = visual.ImageStim(win=win, name='imageBaby',
    image='images/Baby1.jpg', mask=None,
    ori=0, pos=[0, 0], size=[4.223684, 6],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=64, interpolate=True, depth=0.0)
textBaby = visual.TextStim(win=win, ori=0, name='textBaby',
    text='This is a baby',    font='Arial',
    pos=[0, -4], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "directionsTrialBuild"
directionsTrialBuildClock = core.Clock()

directionsTrialBuildText2 = visual.TextStim(win=win, ori=0, name='directionsTrialBuildText2',
    text='+',    font='Arial',
    pos=[0, 3], height=titleLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
directionsTrialBuildText4 = visual.TextStim(win=win, ori=0, name='directionsTrialBuildText4',
    text='?',    font='Arial',
    pos=[0, -1.5], height=titleLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
directionsBuildText5 = visual.TextStim(win=win, ori=0, name='directionsBuildText5',
    text='Make a guess!',    font='Arial',
    pos=[0, -4], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)


# Initialize components for Routine "directionsFeedback"
directionsFeedbackClock = core.Clock()

directionsFeedbackText1 = visual.TextStim(win=win, ori=0, name='directionsFeedbackText1',
    text='If you guessed right, you will win money!',    font='Arial',
    pos=[0, 3], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
directionsFeedbackText2 = visual.TextStim(win=win, ori=0, name='directionsFeedbackText2',
    text='If you guess wrong, you will lose money.',    font='Arial',
    pos=[0, -3], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)


# Initialize components for Routine "directionsInstructions1"
directionsInstructions1Clock = core.Clock()

imgBaby2 = visual.ImageStim(win=win, name='imgBaby2',
    image='images/Baby1.jpg', mask=None,
    ori=0, pos=[-5, 3], size=[(107./152)*4.5, 4.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
imgPointer = visual.ImageStim(win=win, name='imgPointer',
    image='images/pointerFingerButton.png', mask=None,
    ori=0, pos=[-5, -3], size=[5, 6],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
imgAdult1b = visual.ImageStim(win=win, name='imgAdult1b',
    image='images/Adult1.jpg', mask=None,
    ori=0, pos=[4, 3], size=[160./520 * 4.5, 4.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-3.0)
imgAdult2b = visual.ImageStim(win=win, name='imgAdult2b',
    image='images/Adult2.jpg', mask=None,
    ori=0, pos=[6, 3], size=[160/500*4.5, 4.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-4.0)
imgMiddle = visual.ImageStim(win=win, name='imgMiddle',
    image='images/middleFingerButton.png', mask=None,
    ori=0, pos=[5, -3], size=[5, 6],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)


# Initialize components for Routine "directions2"
directions2Clock = core.Clock()

pracDirections2Header = visual.TextStim(win=win, ori=0, name='pracDirections2Header',
    text='How much you can win or lose will change during the game.',    font='Arial',
    pos=[0, 4.5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
exampleHighStim = visual.ImageStim(win=win, name='exampleHighStim',
    image='images/exampleHigh.png', mask=None,
    ori=0, pos=[-4, -2], size=[8.5, 8.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
exampleLowStim = visual.ImageStim(win=win, name='exampleLowStim',
    image='images/exampleLow.png', mask=None,
    ori=0, pos=[4, -2], size=[8.5, 8.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-3.0)


# Initialize components for Routine "kidPracTrial"
kidPracTrialClock = core.Clock()

kidPracTrialText = visual.TextStim(win=win, ori=0, name='kidPracTrialText',
    text='?',    font='Arial',
    pos=[0, 0], height=qmarkSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "feedback"
feedbackClock = core.Clock()
rewardAmount=None
feedbackName=None

imageDir = 'images'
DIME = os.path.join(imageDir, 'dime.png')
QUARTER = os.path.join(imageDir, 'quarter.png')

# Same yoffset for all coins
xoffset, yoffset = 0, -2.5
highXScale = 1.2

coinSize = [2.6, 2.6]
quarterSize = coinSize
#dimeSize = [coinSize[0] * .8, coinSize[1] * .8]
dimeSize = coinSize
xscale = 1.8
usedStims = set([])

# extrafeedbackStim is a visual stimulus for the purpose of
# extending a builder trial. It doesn't show anything, but prevents
# the routine from ending for feedback trials during practice with
# a longer display time. 
extraFeedbackStim = visual.TextStim(win=win, text='', name='extraFeedbackStim')

circleFrame = visual.Polygon(win=win, name='circleFrame',
    edges = 90, size=[11.5, 11.5],
    ori=0, pos=[0, 0],
    lineWidth=8, lineColor='white', lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=1,depth=-1.0, 
interpolate=True)
outerSquareFrame = visual.Rect(win=win, name='outerSquareFrame',
    width=[11.5, 11.5][0], height=[11.5, 11.5][1],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor='black', lineColorSpace='rgb',
    fillColor='white', fillColorSpace='rgb',
    opacity=1,depth=-2.0, 
interpolate=True)
innerSquareFrame = visual.Rect(win=win, name='innerSquareFrame',
    width=[11, 11][0], height=[11, 11][1],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor='black', lineColorSpace='rgb',
    fillColor='black', fillColorSpace='rgb',
    opacity=1,depth=-3.0, 
interpolate=True)
fdback = visual.TextStim(win=win, ori=0, name='fdback',
    text='default text',    font='Arial',
    pos=[0,0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
quarter1_fbk = visual.ImageStim(win=win, name='quarter1_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)
quarter2_fbk = visual.ImageStim(win=win, name='quarter2_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-6.0)
quarter3_fbk = visual.ImageStim(win=win, name='quarter3_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
quarter4_fbk = visual.ImageStim(win=win, name='quarter4_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=coinSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-8.0)
dime1_fbk = visual.ImageStim(win=win, name='dime1_fbk',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
dime2_fbk = visual.ImageStim(win=win, name='dime2_fbk',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-10.0)
xStim1 = visual.ImageStim(win=win, name='xStim1',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-11.0)
xStim2 = visual.ImageStim(win=win, name='xStim2',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-12.0)

if expInfo['rewShape'] == 'circle':
    rewFrames = [circleFrame]
    unrewFrames = [outerSquareFrame, innerSquareFrame]
elif expInfo['rewShape'] == 'square':
    rewFrames = [outerSquareFrame, innerSquareFrame]
    unrewFrames = [circleFrame]
else:
    raise StandardError('rewShape must be either "circle" or "square"')


# Initialize components for Routine "directions3"
directions3Clock = core.Clock()
getReady1Text = visual.TextStim(win=win, ori=0, name='getReady1Text',
    text='Press a button every time you see a "?" or the computer will guess for you!',    font='Arial',
    pos=[0, 2], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
getReady2Text = visual.TextStim(win=win, ori=0, name='getReady2Text',
    text='Do you have any questions?',    font='Arial',
    pos=[0, -2], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "trial_cue"
trial_cueClock = core.Clock()
import pandas as pd

# set win/loss cues
winAmount=None
lossAmount=None
winCueMsg=None
lossCueMsg=None

circle = visual.Polygon(win=win, name='circle',
    edges = 90, size=[15, 15],
    ori=0, pos=[0, 0],
    lineWidth=2, lineColor=[1,1,1], lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=0,depth=-1.0, 
interpolate=True)
line = visual.Line(win=win, name='line',
    start=(-[12, 0][0]/2.0, 0), end=(+[12, 0][0]/2.0, 0),
    ori=0, pos=[0, 0],
    lineWidth=4, lineColor=[1,1,1], lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=1,depth=-2.0, 
interpolate=True)
StakesWin = visual.TextStim(win=win, ori=0, name='StakesWin',
    text='default text',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)
StakesLoss = visual.TextStim(win=win, ori=0, name='StakesLoss',
    text='default text',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
dime1_cue = visual.ImageStim(win=win, name='dime1_cue',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)
dime2_cue = visual.ImageStim(win=win, name='dime2_cue',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-6.0)
dime3_cue = visual.ImageStim(win=win, name='dime3_cue',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
quarter1_cue = visual.ImageStim(win=win, name='quarter1_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-8.0)
quarter2_cue = visual.ImageStim(win=win, name='quarter2_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
quarter3_cue = visual.ImageStim(win=win, name='quarter3_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-10.0)
quarter4_cue = visual.ImageStim(win=win, name='quarter4_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-11.0)
quarter5_cue = visual.ImageStim(win=win, name='quarter5_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-12.0)
quarter6_cue = visual.ImageStim(win=win, name='quarter6_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-13.0)
cueStims = set([dime1_cue, dime2_cue, dime3_cue,
                quarter1_cue, quarter2_cue, quarter3_cue, 
                quarter4_cue, quarter5_cue, quarter6_cue])

xStim1_cue = visual.ImageStim(win=win, name='xStim1_cue',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-16.0)
xStim2_cue = visual.ImageStim(win=win, name='xStim2_cue',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-17.0)


# Initialize components for Routine "pracJitter"
pracJitterClock = core.Clock()
pracJitter1 = visual.TextStim(win=win, ori=0, name='pracJitter1',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "trial"
trialClock = core.Clock()
guess = visual.TextStim(win=win, ori=0, name='guess',
    text='?',    font='Arial',
    pos=[0, 0], height=qmarkSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raResponseTopText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[0, 5], height=textLetterSize * 1.2, wrapWidth=20,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raLast5Text= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[-4, -2], height=textLetterSize, wrapWidth=10,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raTotalText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[4, -2], height=textLetterSize, wrapWidth=10,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

def responseText(currentTrialN, respKey, totalTrials):
    if respKey == '':
        respLabel = ''  # No response yet (but not a miss)
    elif respKey in config['keys']['baby']:
        respLabel = '(baby)'
    elif respKey in config['keys']['adult']:
        respLabel = '(adult)'
    elif not respKey:  # respNum == None
        respLabel = '(no_response)'
    else:
        raise StandardError("Got unexpected response button: %s" % respKey)

    currentTempl = 'Current Trial: %d / %d\nResponse: %s %s'
    currentPat = (currentTrialN + 1, totalTrials, respKey, respLabel)
    # Add 1 for display of current trial
    return currentTempl % currentPat


def responseReport(trialsDf, prefix):
    counts = responseCounts(trialsDf)
    counts['prefix'] = prefix
    countTempl = '%(prefix)s:\nBaby: %(baby)d\nAdult: %(adult)d\nNo responses: %(no_response)d'
    msg = countTempl % counts
    del(counts['prefix'])
    return msg, counts


def responseCounts(trialsDf):
    counts = dict(
        baby=(trialsDf['guessResp.firstKey'].isin(config['keys']['baby'])).sum(),
        adult=(trialsDf['guessResp.firstKey'].isin(config['keys']['adult'])).sum(),
        no_response=(trialsDf['guessResp.firstKey'].isnull()).sum()
    )
    return counts


def updateTrialWin(currentLoopName, runDf, currentN,
                   currentText, respKeys, last5Text=None, 
                   totalText=None):
    # Endpoints included in slice - so subtract 4 from index.
    if currentN < 4:
        start = 0
    else:
        start = currentN - 4
    stop = currentN

    if last5Text:
        last5Msg, last5Counts = responseReport(runDf.loc[start:stop],
                                               'Last 5 Trials')
        last5Text.text = last5Msg
        last5Text.draw()

    if totalText:
        totalMsg, totalCounts = responseReport(runDf, 'Total')
        totalText.text = totalMsg
        totalText.draw()

    currentText.text = responseText(currentN, respKeys, runDf.shape[0])
    currentText.draw()
    raWinRecorder.keyframe()
    raWin.flip()


def updateRunLog(entries):
    df = pd.DataFrame(entries)

    # Filter rows for this run and selected columns
    run_df = df.loc[df['runs.thisN'] == runs.thisN].filter(COLUMNS)
    run_df['run'] = runNum  # 1-based index for "run"

    run_df.dropna(axis='index', subset=['trialNum'], inplace=True)
#    print clean_df['letterTime']
#    print clean_df['letterTime'].isnull()
#    clean_df = clean_df.iloc[1:]
#    clean_df['trialNum'] = clean_df['trialNum'].astype(int)

    runOutfile = '%s_wide.csv' % filename
    run_df.to_csv(runOutfile, encoding='utf-8',
                  header=True, index=False, )

# Initialize components for Routine "pracJitter"
pracJitterClock = core.Clock()
pracJitter1 = visual.TextStim(win=win, ori=0, name='pracJitter1',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "feedback"
feedbackClock = core.Clock()
rewardAmount=None
feedbackName=None

imageDir = 'images'
DIME = os.path.join(imageDir, 'dime.png')
QUARTER = os.path.join(imageDir, 'quarter.png')

# Same yoffset for all coins
xoffset, yoffset = 0, -2.5
highXScale = 1.2

coinSize = [2.6, 2.6]
quarterSize = coinSize
#dimeSize = [coinSize[0] * .8, coinSize[1] * .8]
dimeSize = coinSize
xscale = 1.8
usedStims = set([])

# extrafeedbackStim is a visual stimulus for the purpose of
# extending a builder trial. It doesn't show anything, but prevents
# the routine from ending for feedback trials during practice with
# a longer display time. 
extraFeedbackStim = visual.TextStim(win=win, text='', name='extraFeedbackStim')

circleFrame = visual.Polygon(win=win, name='circleFrame',
    edges = 90, size=[11.5, 11.5],
    ori=0, pos=[0, 0],
    lineWidth=8, lineColor='white', lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=1,depth=-1.0, 
interpolate=True)
outerSquareFrame = visual.Rect(win=win, name='outerSquareFrame',
    width=[11.5, 11.5][0], height=[11.5, 11.5][1],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor='black', lineColorSpace='rgb',
    fillColor='white', fillColorSpace='rgb',
    opacity=1,depth=-2.0, 
interpolate=True)
innerSquareFrame = visual.Rect(win=win, name='innerSquareFrame',
    width=[11, 11][0], height=[11, 11][1],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor='black', lineColorSpace='rgb',
    fillColor='black', fillColorSpace='rgb',
    opacity=1,depth=-3.0, 
interpolate=True)
fdback = visual.TextStim(win=win, ori=0, name='fdback',
    text='default text',    font='Arial',
    pos=[0,0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
quarter1_fbk = visual.ImageStim(win=win, name='quarter1_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)
quarter2_fbk = visual.ImageStim(win=win, name='quarter2_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-6.0)
quarter3_fbk = visual.ImageStim(win=win, name='quarter3_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
quarter4_fbk = visual.ImageStim(win=win, name='quarter4_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=coinSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-8.0)
dime1_fbk = visual.ImageStim(win=win, name='dime1_fbk',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
dime2_fbk = visual.ImageStim(win=win, name='dime2_fbk',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-10.0)
xStim1 = visual.ImageStim(win=win, name='xStim1',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-11.0)
xStim2 = visual.ImageStim(win=win, name='xStim2',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-12.0)

if expInfo['rewShape'] == 'circle':
    rewFrames = [circleFrame]
    unrewFrames = [outerSquareFrame, innerSquareFrame]
elif expInfo['rewShape'] == 'square':
    rewFrames = [outerSquareFrame, innerSquareFrame]
    unrewFrames = [circleFrame]
else:
    raise StandardError('rewShape must be either "circle" or "square"')


# Initialize components for Routine "pracCheckResponse"
pracCheckResponseClock = core.Clock()
pracFailMessage = '''You didn't press in time.\n\nRemember to press when you see the "?"\n\nLet's try again.'''
pracCheckReponseText = visual.TextStim(win=win, ori=0, name='pracCheckReponseText',
    text=pracFailMessage,    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "qMark"
qMarkClock = core.Clock()
qmark = visual.TextStim(win=win, ori=0, name='qmark',
    text='?',    font='Arial',
    pos=[0, 0], height=qmarkSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
qmarkText = visual.TextStim(win=win, ori=0, name='qmarkText',
    text='Make a guess!',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "directionsInstructions1"
directionsInstructions1Clock = core.Clock()

imgBaby2 = visual.ImageStim(win=win, name='imgBaby2',
    image='images/Baby1.jpg', mask=None,
    ori=0, pos=[-5, 3], size=[(107./152)*4.5, 4.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
imgPointer = visual.ImageStim(win=win, name='imgPointer',
    image='images/pointerFingerButton.png', mask=None,
    ori=0, pos=[-5, -3], size=[5, 6],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-2.0)
imgAdult1b = visual.ImageStim(win=win, name='imgAdult1b',
    image='images/Adult1.jpg', mask=None,
    ori=0, pos=[4, 3], size=[160./520 * 4.5, 4.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-3.0)
imgAdult2b = visual.ImageStim(win=win, name='imgAdult2b',
    image='images/Adult2.jpg', mask=None,
    ori=0, pos=[6, 3], size=[160/500*4.5, 4.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-4.0)
imgMiddle = visual.ImageStim(win=win, name='imgMiddle',
    image='images/middleFingerButton.png', mask=None,
    ori=0, pos=[5, -3], size=[5, 6],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)


# Initialize components for Routine "getReady_2"
getReady_2Clock = core.Clock()
# Initialize Reward for practice trials
# And also for the start of each run.
cumulativeReward = 0.0  # For this run

COLUMNS = [
    'trialNum', 
    'countdownStartTime', 'countdownLabel',
    'cueStartTime', 'cueEndTime',
    'isi1StartTime', 'isi1EndTime', 'ISI1',
    'guessStartTime', 'guessEndTime',
    'isi2StartTime', 'isi2EndTime', 'ISI2',
    'feedbackStartTime', 'feedbackEndTime',
    'runEndTime',
    'trialCondition', 'feedbackName', 'Feedback', 'valueCondition',
    'guessResp.firstKey', 'guessResp.firstRt', 'guessResp.firstGuess',
    'guessResp.keys', 'guessResp.rt',
    'isiPress1.keys', 'isiPress1.rt',
    'isiPress2.keys', 'isiPress2.rt', 
    'cumulativeNoResp', 'cumulativeReward',
    'rewardAmount', 'rewardAmountNum',
    # expInfo
    'rewShape',
    'sessionID', 'mriMode', 'nRuns', 
    'rewShape', 'run', 'expName', 'date', 
    'triggerGlobalTime', 'triggerWallTime',
    'frameRate', 'git-revision',]

def newTrialFrame(nTrials):
    df = pd.DataFrame(columns=COLUMNS, index=range(nTrials))
    df['guessResp.keys'] = 0  # Set responses to 0 (not nan) to count no responses.
    df['guessResp.firstKey'] = 0  
    # Update with Experiment Info Details
    for k, v in expInfo.iteritems():
        df[k] = v
    return df

getReadyText = visual.TextStim(win=win, ori=0, name='getReadyText',
    text='default text',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
startText = visual.TextStim(win=win, ori=0, name='startText',
    text=None,    font='Arial',
    pos=[0, -3], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)


# Initialize components for Routine "wait_for_scanner"
wait_for_scannerClock = core.Clock()
waitForScannerText = visual.TextStim(win=win, ori=0, name='waitForScannerText',
    text='default text',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
import time
fmriClock = core.Clock()
trigger = 'usb'
#trigger = 'parallel'
if trigger == 'parallel':
    from psychopy.contrib import parallel as winioport
    #from psychopy import parallel
elif trigger == 'usb':
    from psychopy.hardware.emulator import launchScan
    #
    # settings for launchScan:
    MR_settings = { 
        'TR': 2.000, # duration (sec) per volume
        'volumes': 210, # number of whole-brain 3D volumes / frames
        'sync': config['trigger'], # character to use as the sync timing event; assumed to come at start of a volume
        'skip': 0, # number of volumes lacking a sync pulse at start of scan (for T1 stabilization)
        }

# Don't display waiting message (even blink on for a second) when Off
if expInfo['mriMode'] == 'Off':
    waitForScannerMsg = ''
else:
    waitForScannerMsg = 'Get ready!'


# Initialize components for Routine "countdown"
countdownClock = core.Clock()

countdownText = visual.TextStim(win=win, ori=0, name='countdownText',
    text='default text',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "trial_cue"
trial_cueClock = core.Clock()
import pandas as pd

# set win/loss cues
winAmount=None
lossAmount=None
winCueMsg=None
lossCueMsg=None

circle = visual.Polygon(win=win, name='circle',
    edges = 90, size=[15, 15],
    ori=0, pos=[0, 0],
    lineWidth=2, lineColor=[1,1,1], lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=0,depth=-1.0, 
interpolate=True)
line = visual.Line(win=win, name='line',
    start=(-[12, 0][0]/2.0, 0), end=(+[12, 0][0]/2.0, 0),
    ori=0, pos=[0, 0],
    lineWidth=4, lineColor=[1,1,1], lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=1,depth=-2.0, 
interpolate=True)
StakesWin = visual.TextStim(win=win, ori=0, name='StakesWin',
    text='default text',    font='Arial',
    pos=[0, 5], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)
StakesLoss = visual.TextStim(win=win, ori=0, name='StakesLoss',
    text='default text',    font='Arial',
    pos=[0, -5], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
dime1_cue = visual.ImageStim(win=win, name='dime1_cue',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)
dime2_cue = visual.ImageStim(win=win, name='dime2_cue',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-6.0)
dime3_cue = visual.ImageStim(win=win, name='dime3_cue',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
quarter1_cue = visual.ImageStim(win=win, name='quarter1_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-8.0)
quarter2_cue = visual.ImageStim(win=win, name='quarter2_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
quarter3_cue = visual.ImageStim(win=win, name='quarter3_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-10.0)
quarter4_cue = visual.ImageStim(win=win, name='quarter4_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-11.0)
quarter5_cue = visual.ImageStim(win=win, name='quarter5_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-12.0)
quarter6_cue = visual.ImageStim(win=win, name='quarter6_cue',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-13.0)
cueStims = set([dime1_cue, dime2_cue, dime3_cue,
                quarter1_cue, quarter2_cue, quarter3_cue, 
                quarter4_cue, quarter5_cue, quarter6_cue])

xStim1_cue = visual.ImageStim(win=win, name='xStim1_cue',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-16.0)
xStim2_cue = visual.ImageStim(win=win, name='xStim2_cue',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-17.0)


# Initialize components for Routine "jitter"
jitterClock = core.Clock()
jitteredISI = visual.TextStim(win=win, ori=0, name='jitteredISI',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
extraISIStim = visual.TextStim(win=win, text='', name='extraISIStim')


# Initialize components for Routine "trial"
trialClock = core.Clock()
guess = visual.TextStim(win=win, ori=0, name='guess',
    text='?',    font='Arial',
    pos=[0, 0], height=qmarkSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raResponseTopText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[0, 5], height=textLetterSize * 1.2, wrapWidth=20,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raLast5Text= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[-4, -2], height=textLetterSize, wrapWidth=10,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raTotalText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='',    font='Arial',
    pos=[4, -2], height=textLetterSize, wrapWidth=10,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

def responseText(currentTrialN, respKey, totalTrials):
    if respKey == '':
        respLabel = ''  # No response yet (but not a miss)
    elif respKey in config['keys']['baby']:
        respLabel = '(baby)'
    elif respKey in config['keys']['adult']:
        respLabel = '(adult)'
    elif not respKey:  # respNum == None
        respLabel = '(no_response)'
    else:
        raise StandardError("Got unexpected response button: %s" % respKey)

    currentTempl = 'Current Trial: %d / %d\nResponse: %s %s'
    currentPat = (currentTrialN + 1, totalTrials, respKey, respLabel)
    # Add 1 for display of current trial
    return currentTempl % currentPat


def responseReport(trialsDf, prefix):
    counts = responseCounts(trialsDf)
    counts['prefix'] = prefix
    countTempl = '%(prefix)s:\nBaby: %(baby)d\nAdult: %(adult)d\nNo responses: %(no_response)d'
    msg = countTempl % counts
    del(counts['prefix'])
    return msg, counts


def responseCounts(trialsDf):
    counts = dict(
        baby=(trialsDf['guessResp.firstKey'].isin(config['keys']['baby'])).sum(),
        adult=(trialsDf['guessResp.firstKey'].isin(config['keys']['adult'])).sum(),
        no_response=(trialsDf['guessResp.firstKey'].isnull()).sum()
    )
    return counts


def updateTrialWin(currentLoopName, runDf, currentN,
                   currentText, respKeys, last5Text=None, 
                   totalText=None):
    # Endpoints included in slice - so subtract 4 from index.
    if currentN < 4:
        start = 0
    else:
        start = currentN - 4
    stop = currentN

    if last5Text:
        last5Msg, last5Counts = responseReport(runDf.loc[start:stop],
                                               'Last 5 Trials')
        last5Text.text = last5Msg
        last5Text.draw()

    if totalText:
        totalMsg, totalCounts = responseReport(runDf, 'Total')
        totalText.text = totalMsg
        totalText.draw()

    currentText.text = responseText(currentN, respKeys, runDf.shape[0])
    currentText.draw()
    raWinRecorder.keyframe()
    raWin.flip()


def updateRunLog(entries):
    df = pd.DataFrame(entries)

    # Filter rows for this run and selected columns
    run_df = df.loc[df['runs.thisN'] == runs.thisN].filter(COLUMNS)
    run_df['run'] = runNum  # 1-based index for "run"

    run_df.dropna(axis='index', subset=['trialNum'], inplace=True)
#    print clean_df['letterTime']
#    print clean_df['letterTime'].isnull()
#    clean_df = clean_df.iloc[1:]
#    clean_df['trialNum'] = clean_df['trialNum'].astype(int)

    runOutfile = '%s_wide.csv' % filename
    run_df.to_csv(runOutfile, encoding='utf-8',
                  header=True, index=False, )

# Initialize components for Routine "jitter2"
jitter2Clock = core.Clock()
jitteredISI_2 = visual.TextStim(win=win, ori=0, name='jitteredISI_2',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "feedback"
feedbackClock = core.Clock()
rewardAmount=None
feedbackName=None

imageDir = 'images'
DIME = os.path.join(imageDir, 'dime.png')
QUARTER = os.path.join(imageDir, 'quarter.png')

# Same yoffset for all coins
xoffset, yoffset = 0, -2.5
highXScale = 1.2

coinSize = [2.6, 2.6]
quarterSize = coinSize
#dimeSize = [coinSize[0] * .8, coinSize[1] * .8]
dimeSize = coinSize
xscale = 1.8
usedStims = set([])

# extrafeedbackStim is a visual stimulus for the purpose of
# extending a builder trial. It doesn't show anything, but prevents
# the routine from ending for feedback trials during practice with
# a longer display time. 
extraFeedbackStim = visual.TextStim(win=win, text='', name='extraFeedbackStim')

circleFrame = visual.Polygon(win=win, name='circleFrame',
    edges = 90, size=[11.5, 11.5],
    ori=0, pos=[0, 0],
    lineWidth=8, lineColor='white', lineColorSpace='rgb',
    fillColor=None, fillColorSpace='rgb',
    opacity=1,depth=-1.0, 
interpolate=True)
outerSquareFrame = visual.Rect(win=win, name='outerSquareFrame',
    width=[11.5, 11.5][0], height=[11.5, 11.5][1],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor='black', lineColorSpace='rgb',
    fillColor='white', fillColorSpace='rgb',
    opacity=1,depth=-2.0, 
interpolate=True)
innerSquareFrame = visual.Rect(win=win, name='innerSquareFrame',
    width=[11, 11][0], height=[11, 11][1],
    ori=0, pos=[0, 0],
    lineWidth=0, lineColor='black', lineColorSpace='rgb',
    fillColor='black', fillColorSpace='rgb',
    opacity=1,depth=-3.0, 
interpolate=True)
fdback = visual.TextStim(win=win, ori=0, name='fdback',
    text='default text',    font='Arial',
    pos=[0,0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
quarter1_fbk = visual.ImageStim(win=win, name='quarter1_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-5.0)
quarter2_fbk = visual.ImageStim(win=win, name='quarter2_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-6.0)
quarter3_fbk = visual.ImageStim(win=win, name='quarter3_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=quarterSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
quarter4_fbk = visual.ImageStim(win=win, name='quarter4_fbk',
    image=QUARTER, mask=None,
    ori=0, pos=[0, 0], size=coinSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-8.0)
dime1_fbk = visual.ImageStim(win=win, name='dime1_fbk',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
dime2_fbk = visual.ImageStim(win=win, name='dime2_fbk',
    image=DIME, mask=None,
    ori=0, pos=[0, 0], size=dimeSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-10.0)
xStim1 = visual.ImageStim(win=win, name='xStim1',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-11.0)
xStim2 = visual.ImageStim(win=win, name='xStim2',
    image='images/X.png', mask=None,
    ori=0, pos=[0, 0], size=[coinSize[0] * 1.1, coinSize[1] * 1.1],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-12.0)

if expInfo['rewShape'] == 'circle':
    rewFrames = [circleFrame]
    unrewFrames = [outerSquareFrame, innerSquareFrame]
elif expInfo['rewShape'] == 'square':
    rewFrames = [outerSquareFrame, innerSquareFrame]
    unrewFrames = [circleFrame]
else:
    raise StandardError('rewShape must be either "circle" or "square"')


# Initialize components for Routine "Break"
BreakClock = core.Clock()
breakJitter = visual.TextStim(win=win, ori=0, name='breakJitter',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "end_run"
end_runClock = core.Clock()

endRunText1 = visual.TextStim(win=win, ori=0, name='endRunText1',
    text=None,    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
endRunText2 = visual.TextStim(win=win, ori=0, name='endRunText2',
    text='default text',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)


# Initialize components for Routine "End"
EndClock = core.Clock()

endText = visual.TextStim(win=win, ori=0, name='endText',
    text='default text',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Create some handy timers
globalClock = core.Clock()  # to track the time since experiment started
routineTimer = core.CountdownTimer()  # to track time remaining of each (non-slip) routine 

#------Prepare to start Routine "intro"-------
t = 0
introClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat
# Init for practice (after newTrialFrame has been defined).
runDataframe = newTrialFrame(2)

introResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
introResp.status = NOT_STARTED
raVerbalText.text = '"Welcome to the Guessing Game. In this game, your job will be to guess whether the person hiding behind a question mark is a baby or an adult."'
raText.text = 'Press <space> to continue.'
raVerbalText.draw()
raText.draw()
raWinRecorder.keyframe()
raWin.flip()

#outerFrame.autoDraw = True
movieClock.reset()
# keep track of which components have finished
introComponents = []
introComponents.append(introText)
introComponents.append(introText2)
introComponents.append(introResp)
for thisComponent in introComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "intro"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = introClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    
    # *introText* updates
    if t >= 0.0 and introText.status == NOT_STARTED:
        # keep track of start time/frame for later
        introText.tStart = t  # underestimates by a little under one frame
        introText.frameNStart = frameN  # exact frame index
        introText.setAutoDraw(True)
    
    # *introText2* updates
    if t >= 0.0 and introText2.status == NOT_STARTED:
        # keep track of start time/frame for later
        introText2.tStart = t  # underestimates by a little under one frame
        introText2.frameNStart = frameN  # exact frame index
        introText2.setAutoDraw(True)
    
    # *introResp* updates
    if t >= 0.0 and introResp.status == NOT_STARTED:
        # keep track of start time/frame for later
        introResp.tStart = t  # underestimates by a little under one frame
        introResp.frameNStart = frameN  # exact frame index
        introResp.status = STARTED
        # AllowedKeys looks like a variable named `all_allowed_keys`
        if not 'all_allowed_keys' in locals():
            logging.error('AllowedKeys variable `all_allowed_keys` is not defined.')
            core.quit()
        if not type(all_allowed_keys) in [list, tuple, np.ndarray]:
            if not isinstance(all_allowed_keys, basestring):
                logging.error('AllowedKeys variable `all_allowed_keys` is not string- or list-like.')
                core.quit()
            elif not ',' in all_allowed_keys: all_allowed_keys = (all_allowed_keys,)
            else:  all_allowed_keys = eval(all_allowed_keys)
        # keyboard checking is just starting
        event.clearEvents(eventType='keyboard')
    if introResp.status == STARTED:
        theseKeys = event.getKeys(keyList=list(all_allowed_keys))
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            # a response ends the routine
            continueRoutine = False
    
    
    
    if frameN == 0:
        winRecorder.keyframe()
    
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in introComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "intro"-------
for thisComponent in introComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)





# the Routine "intro" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()

# set up handler to look after randomisation of conditions etc
pracDirections = data.TrialHandler(nReps=expInfo['runNumber'] == 'practice', method='random', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='pracDirections')
thisExp.addLoop(pracDirections)  # add the loop to the experiment
thisPracDirection = pracDirections.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisPracDirection.rgb)
if thisPracDirection != None:
    for paramName in thisPracDirection.keys():
        exec(paramName + '= thisPracDirection.' + paramName)

for thisPracDirection in pracDirections:
    currentLoop = pracDirections
    # abbreviate parameter names if possible (e.g. rgb = thisPracDirection.rgb)
    if thisPracDirection != None:
        for paramName in thisPracDirection.keys():
            exec(paramName + '= thisPracDirection.' + paramName)
    
    #------Prepare to start Routine "directionsAdult"-------
    t = 0
    directionsAdultClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    respAdult = event.BuilderKeyResponse()  # create an object of type KeyResponse
    respAdult.status = NOT_STARTED
    raVerbalText.text = '"These are adults, just like your mom or dad."'
    raText.text = 'Press <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directionsAdultComponents = []
    directionsAdultComponents.append(textAdult)
    directionsAdultComponents.append(imageAdult1)
    directionsAdultComponents.append(imageAdult2)
    directionsAdultComponents.append(respAdult)
    for thisComponent in directionsAdultComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directionsAdult"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directionsAdultClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *textAdult* updates
        if t >= 0.0 and textAdult.status == NOT_STARTED:
            # keep track of start time/frame for later
            textAdult.tStart = t  # underestimates by a little under one frame
            textAdult.frameNStart = frameN  # exact frame index
            textAdult.setAutoDraw(True)
        
        # *imageAdult1* updates
        if t >= 0.0 and imageAdult1.status == NOT_STARTED:
            # keep track of start time/frame for later
            imageAdult1.tStart = t  # underestimates by a little under one frame
            imageAdult1.frameNStart = frameN  # exact frame index
            imageAdult1.setAutoDraw(True)
        
        # *imageAdult2* updates
        if t >= 0.0 and imageAdult2.status == NOT_STARTED:
            # keep track of start time/frame for later
            imageAdult2.tStart = t  # underestimates by a little under one frame
            imageAdult2.frameNStart = frameN  # exact frame index
            imageAdult2.setAutoDraw(True)
        
        # *respAdult* updates
        if t >= 0.0 and respAdult.status == NOT_STARTED:
            # keep track of start time/frame for later
            respAdult.tStart = t  # underestimates by a little under one frame
            respAdult.frameNStart = frameN  # exact frame index
            respAdult.status = STARTED
            # keyboard checking is just starting
            event.clearEvents(eventType='keyboard')
        if respAdult.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directionsAdultComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directionsAdult"-------
    for thisComponent in directionsAdultComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # the Routine "directionsAdult" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directionsBaby"-------
    t = 0
    directionsBabyClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    respBaby = event.BuilderKeyResponse()  # create an object of type KeyResponse
    respBaby.status = NOT_STARTED
    raVerbalText.text = '"And this is a baby."'
    raText.text = 'Press <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directionsBabyComponents = []
    directionsBabyComponents.append(imageBaby)
    directionsBabyComponents.append(textBaby)
    directionsBabyComponents.append(respBaby)
    for thisComponent in directionsBabyComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directionsBaby"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directionsBabyClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *imageBaby* updates
        if t >= 0.0 and imageBaby.status == NOT_STARTED:
            # keep track of start time/frame for later
            imageBaby.tStart = t  # underestimates by a little under one frame
            imageBaby.frameNStart = frameN  # exact frame index
            imageBaby.setAutoDraw(True)
        
        # *textBaby* updates
        if t >= 0.0 and textBaby.status == NOT_STARTED:
            # keep track of start time/frame for later
            textBaby.tStart = t  # underestimates by a little under one frame
            textBaby.frameNStart = frameN  # exact frame index
            textBaby.setAutoDraw(True)
        
        # *respBaby* updates
        if t >= 0.0 and respBaby.status == NOT_STARTED:
            # keep track of start time/frame for later
            respBaby.tStart = t  # underestimates by a little under one frame
            respBaby.frameNStart = frameN  # exact frame index
            respBaby.status = STARTED
            # keyboard checking is just starting
            event.clearEvents(eventType='keyboard')
        if respBaby.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directionsBabyComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directionsBaby"-------
    for thisComponent in directionsBabyComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # the Routine "directionsBaby" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directionsTrialBuild"-------
    t = 0
    directionsTrialBuildClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    npresses=0
    record_expt_win = True
    raVerbalText.text = '"When you play this game you\'ll see a cross that looks like this before you are supposed to guess."'
    raText.text = 'Press <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # n.b. More RA instructions in Each Frame Build Code
    
    # keep track of which components have finished
    directionsTrialBuildComponents = []
    directionsTrialBuildComponents.append(directionsTrialBuildText2)
    directionsTrialBuildComponents.append(directionsTrialBuildText4)
    directionsTrialBuildComponents.append(directionsBuildText5)
    for thisComponent in directionsTrialBuildComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directionsTrialBuild"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directionsTrialBuildClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if event.getKeys(keyList = ['space']):
            npresses += 1
            record_expt_win = True
        
            if npresses == 1:
                raVerbalText.text = '"After the cross, you\'ll see a question mark. This is when you should guess if the person hiding behind the question mark is a baby or an adult."'
                raText.text = 'Press <space> to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            elif npresses == 2:
                raVerbalText.text = '"After you guess, you will find out if you guessed right or wrong."'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            else:
                continueRoutine = False
                record_expt_win = False
        
        
        # *directionsTrialBuildText2* updates
        if (npresses==0) and directionsTrialBuildText2.status == NOT_STARTED:
            # keep track of start time/frame for later
            directionsTrialBuildText2.tStart = t  # underestimates by a little under one frame
            directionsTrialBuildText2.frameNStart = frameN  # exact frame index
            directionsTrialBuildText2.setAutoDraw(True)
        
        # *directionsTrialBuildText4* updates
        if (npresses==1) and directionsTrialBuildText4.status == NOT_STARTED:
            # keep track of start time/frame for later
            directionsTrialBuildText4.tStart = t  # underestimates by a little under one frame
            directionsTrialBuildText4.frameNStart = frameN  # exact frame index
            directionsTrialBuildText4.setAutoDraw(True)
        
        # *directionsBuildText5* updates
        if (npresses==1) and directionsBuildText5.status == NOT_STARTED:
            # keep track of start time/frame for later
            directionsBuildText5.tStart = t  # underestimates by a little under one frame
            directionsBuildText5.frameNStart = frameN  # exact frame index
            directionsBuildText5.setAutoDraw(True)
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directionsTrialBuildComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directionsTrialBuild"-------
    for thisComponent in directionsTrialBuildComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # the Routine "directionsTrialBuild" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directionsFeedback"-------
    t = 0
    directionsFeedbackClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    npresses = 0
    record_expt_win = True
    raVerbalText.text = '"If you guessed right you will win money!"'
    raText.text = 'Press <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # n.b. More verbal text on build
    
    # keep track of which components have finished
    directionsFeedbackComponents = []
    directionsFeedbackComponents.append(directionsFeedbackText1)
    directionsFeedbackComponents.append(directionsFeedbackText2)
    for thisComponent in directionsFeedbackComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directionsFeedback"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directionsFeedbackClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if event.getKeys(keyList=['space']):
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = '"If you guessed wrong you will lose money."'
                raText.text = 'Press <space> to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            else:
                continueRoutine = False
                record_expt_win = False
        
        
        # *directionsFeedbackText1* updates
        if t >= 0 and directionsFeedbackText1.status == NOT_STARTED:
            # keep track of start time/frame for later
            directionsFeedbackText1.tStart = t  # underestimates by a little under one frame
            directionsFeedbackText1.frameNStart = frameN  # exact frame index
            directionsFeedbackText1.setAutoDraw(True)
        
        # *directionsFeedbackText2* updates
        if (npresses == 1) and directionsFeedbackText2.status == NOT_STARTED:
            # keep track of start time/frame for later
            directionsFeedbackText2.tStart = t  # underestimates by a little under one frame
            directionsFeedbackText2.frameNStart = frameN  # exact frame index
            directionsFeedbackText2.setAutoDraw(True)
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directionsFeedbackComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directionsFeedback"-------
    for thisComponent in directionsFeedbackComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # the Routine "directionsFeedback" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directionsInstructions1"-------
    t = 0
    directionsInstructions1Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    npresses=0
    raVerbalText.text = '"If you think it is a baby, you should press your POINTER finger."'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    if expInfo['runNumber']=='practice':
        raText.draw()
        raWin.flip()
        raVerbalText.text='"When you see the question mark, you should press a button to make your guess about whether a baby or an adult is hiding behind the question mark.\n\nIf you think it is a baby, you should press the button under your POINTER finger."'
        raText.pos = [0, -4]
        raText.draw()
        raVerbalText.draw()
        raWinRecorder.keyframe()
        record_expt_window = True
        raWin.flip()
    
    record_expt_window = True
    # keep track of which components have finished
    directionsInstructions1Components = []
    directionsInstructions1Components.append(imgBaby2)
    directionsInstructions1Components.append(imgPointer)
    directionsInstructions1Components.append(imgAdult1b)
    directionsInstructions1Components.append(imgAdult2b)
    directionsInstructions1Components.append(imgMiddle)
    for thisComponent in directionsInstructions1Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directionsInstructions1"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directionsInstructions1Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if event.getKeys(keyList=['space']):
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = '"If you think it is an adult, you should press the button under your MIDDLE finger."'
                raText.text = 'Press <space> to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            #elif npresses == 2:
            else:
                continueRoutine = False
                record_expt_win = False
        
        # *imgBaby2* updates
        if t >= 0 and imgBaby2.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgBaby2.tStart = t  # underestimates by a little under one frame
            imgBaby2.frameNStart = frameN  # exact frame index
            imgBaby2.setAutoDraw(True)
        
        # *imgPointer* updates
        if t >= 0 and imgPointer.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgPointer.tStart = t  # underestimates by a little under one frame
            imgPointer.frameNStart = frameN  # exact frame index
            imgPointer.setAutoDraw(True)
        
        # *imgAdult1b* updates
        if (npresses==1) and imgAdult1b.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgAdult1b.tStart = t  # underestimates by a little under one frame
            imgAdult1b.frameNStart = frameN  # exact frame index
            imgAdult1b.setAutoDraw(True)
        
        # *imgAdult2b* updates
        if (npresses == 1) and imgAdult2b.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgAdult2b.tStart = t  # underestimates by a little under one frame
            imgAdult2b.frameNStart = frameN  # exact frame index
            imgAdult2b.setAutoDraw(True)
        
        # *imgMiddle* updates
        if (npresses==1) and imgMiddle.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgMiddle.tStart = t  # underestimates by a little under one frame
            imgMiddle.frameNStart = frameN  # exact frame index
            imgMiddle.setAutoDraw(True)
        if record_expt_window:
            winRecorder.keyframe()
            record_expt_window = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directionsInstructions1Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directionsInstructions1"-------
    for thisComponent in directionsInstructions1Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    raText.text = ''
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # the Routine "directionsInstructions1" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directions2"-------
    t = 0
    directions2Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    npresses = 0
    record_expt_win = True
    
    raVerbalText.text ='"How much you can win or lose will change during the game. You will know how much you can win or lose during the game by seeing one of these pictures appear on the screen."'
    raText.text = 'Press <space> to continue.'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    
    # keep track of which components have finished
    directions2Components = []
    directions2Components.append(pracDirections2Header)
    directions2Components.append(exampleHighStim)
    directions2Components.append(exampleLowStim)
    for thisComponent in directions2Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions2"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions2Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if event.getKeys(keyList=['space']):
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = u'"When you see this picture, it means you can win or lose more — you can win $1 if you guess right or lose 50¢ if you guess wrong."'
                raText.text = 'Press <space> to continue.'
            elif npresses == 2:
                raVerbalText.text = u'"And when you see this picture, it means you can win or lose less — you can win 20¢ if you guess right or lose 10¢ if you guess wrong."'
                raText.text = 'Press <space> to continue.'
            else:
                continueRoutine = False
        
            raVerbalText.draw()
            raText.draw()
            raWinRecorder.keyframe()
            raWin.flip()
        
        
        # *pracDirections2Header* updates
        if t >= 0.0 and pracDirections2Header.status == NOT_STARTED:
            # keep track of start time/frame for later
            pracDirections2Header.tStart = t  # underestimates by a little under one frame
            pracDirections2Header.frameNStart = frameN  # exact frame index
            pracDirections2Header.setAutoDraw(True)
        
        # *exampleHighStim* updates
        if (npresses == 1) and exampleHighStim.status == NOT_STARTED:
            # keep track of start time/frame for later
            exampleHighStim.tStart = t  # underestimates by a little under one frame
            exampleHighStim.frameNStart = frameN  # exact frame index
            exampleHighStim.setAutoDraw(True)
        
        # *exampleLowStim* updates
        if (npresses == 2) and exampleLowStim.status == NOT_STARTED:
            # keep track of start time/frame for later
            exampleLowStim.tStart = t  # underestimates by a little under one frame
            exampleLowStim.frameNStart = frameN  # exact frame index
            exampleLowStim.setAutoDraw(True)
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions2Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions2"-------
    for thisComponent in directions2Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # the Routine "directions2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    kidPracControlLoop = data.TrialHandler(nReps=age < 8, method='random', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='kidPracControlLoop')
    thisExp.addLoop(kidPracControlLoop)  # add the loop to the experiment
    thisKidPracControlLoop = kidPracControlLoop.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisKidPracControlLoop.rgb)
    if thisKidPracControlLoop != None:
        for paramName in thisKidPracControlLoop.keys():
            exec(paramName + '= thisKidPracControlLoop.' + paramName)
    
    for thisKidPracControlLoop in kidPracControlLoop:
        currentLoop = kidPracControlLoop
        # abbreviate parameter names if possible (e.g. rgb = thisKidPracControlLoop.rgb)
        if thisKidPracControlLoop != None:
            for paramName in thisKidPracControlLoop.keys():
                exec(paramName + '= thisKidPracControlLoop.' + paramName)
        
        # set up handler to look after randomisation of conditions etc
        kidPracLoop = data.TrialHandler(nReps=1, method='sequential', 
            extraInfo=expInfo, originPath=-1,
            trialList=data.importConditions('conditions/kidPrac.csv'),
            seed=None, name='kidPracLoop')
        thisExp.addLoop(kidPracLoop)  # add the loop to the experiment
        thisKidPracLoop = kidPracLoop.trialList[0]  # so we can initialise stimuli with some values
        # abbreviate parameter names if possible (e.g. rgb=thisKidPracLoop.rgb)
        if thisKidPracLoop != None:
            for paramName in thisKidPracLoop.keys():
                exec(paramName + '= thisKidPracLoop.' + paramName)
        
        for thisKidPracLoop in kidPracLoop:
            currentLoop = kidPracLoop
            # abbreviate parameter names if possible (e.g. rgb = thisKidPracLoop.rgb)
            if thisKidPracLoop != None:
                for paramName in thisKidPracLoop.keys():
                    exec(paramName + '= thisKidPracLoop.' + paramName)
            
            #------Prepare to start Routine "kidPracTrial"-------
            t = 0
            kidPracTrialClock.reset()  # clock 
            frameN = -1
            # update component parameters for each repeat
            record_expt_win = True
            if currentLoop.thisN == 0:
                verbalMsg = '"Okay, let\'s do a couple of examples together.\n\nNow I am going to guess whether it is going to be a baby or an adult.\n\nI think it is going to be a baby, so I am going to push the button with my pointer finger."'
                raMsg = 'Push POINTER finger button' 
            elif currentLoop.thisN == 1:
                verbalMsg = '"Okay, now I am going to guess again. This time I think it is going to be an adult, so I am going to push the button with my middle finger."'
                raMsg = 'Push MIDDLE finger button'
            elif currentLoop.thisN == 2:
                verbalMsg = '"Okay, now I am going to guess again. This time I think it is going to be an adult, so I am going to push the button with my middle finger."'
                raMsg = 'Push MIDDLE finger button'
            
            raVerbalText.text = verbalMsg
            raText.text = raMsg
            raText.pos = [0, -5]
            
            raVerbalText.draw()
            raText.draw()
            raWinRecorder.keyframe()
            raWin.flip()
            kidPracTrialResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
            kidPracTrialResp.status = NOT_STARTED
            
            # keep track of which components have finished
            kidPracTrialComponents = []
            kidPracTrialComponents.append(kidPracTrialText)
            kidPracTrialComponents.append(kidPracTrialResp)
            for thisComponent in kidPracTrialComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "kidPracTrial"-------
            continueRoutine = True
            while continueRoutine:
                # get current time
                t = kidPracTrialClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                for prompt in buttonPrompts:
                    prompt.draw()
                
                
                # *kidPracTrialText* updates
                if t >= 0.0 and kidPracTrialText.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    kidPracTrialText.tStart = t  # underestimates by a little under one frame
                    kidPracTrialText.frameNStart = frameN  # exact frame index
                    kidPracTrialText.setAutoDraw(True)
                
                # *kidPracTrialResp* updates
                if t >= 0.0 and kidPracTrialResp.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    kidPracTrialResp.tStart = t  # underestimates by a little under one frame
                    kidPracTrialResp.frameNStart = frameN  # exact frame index
                    kidPracTrialResp.status = STARTED
                    # AllowedKeys looks like a variable named `allowed_keys`
                    if not 'allowed_keys' in locals():
                        logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                        core.quit()
                    if not type(allowed_keys) in [list, tuple, np.ndarray]:
                        if not isinstance(allowed_keys, basestring):
                            logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                            core.quit()
                        elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                        else:  allowed_keys = eval(allowed_keys)
                    # keyboard checking is just starting
                    event.clearEvents(eventType='keyboard')
                if kidPracTrialResp.status == STARTED:
                    theseKeys = event.getKeys(keyList=list(allowed_keys))
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        # a response ends the routine
                        continueRoutine = False
                if record_expt_win:
                    winRecorder.keyframe()
                    record_expt_win = False
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in kidPracTrialComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "kidPracTrial"-------
            for thisComponent in kidPracTrialComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            guessed_keys = '1'
            
            feedback1Msg = '"Yea, I was right."'
            feedback2Msg = '"Oh no, I was not right this time and I lost money."'
            feedback3Msg = '"Yea, I was right this time."'
            
            if currentLoop.thisN == 0:
                msg = feedback1Msg
            elif currentLoop.thisN == 1:
                msg = feedback2Msg
            elif currentLoop.thisN == 2:
                msg = feedback3Msg
            
            raVerbalText.text = msg
            raText.text = 'Wait 3 seconds'
            raVerbalText.draw()
            raText.draw()
            raWin.flip()
            
            
            # the Routine "kidPracTrial" was not non-slip safe, so reset the non-slip timer
            routineTimer.reset()
            
            #------Prepare to start Routine "feedback"-------
            t = 0
            feedbackClock.reset()  # clock 
            frameN = -1
            routineTimer.add(1.000000)
            # update component parameters for each repeat
            # calculate feedback and reward and account for missed responses
            fbkStims = set([dime1_fbk, dime2_fbk, quarter1_fbk, quarter2_fbk,
                            quarter3_fbk, quarter4_fbk])
            feedbackPos = [0, 2]
            
            usedStims.clear()
            
            if valueCondition == "low":  # DIME
              if Feedback == 1:  # Win
                feedbackName = "Win"
                rewardAmount = u"20¢"
                rewardAmountNum = .20
                dime1_fbk.pos = [-1 * xscale, yoffset]
                dime2_fbk.pos = [1 * xscale, yoffset]
                usedStims.update([dime1_fbk, dime2_fbk])
                xStim1.opacity, xStim2.opacity = 0, 0
            
              else:  # Lose
                feedbackName = "Lose"
                rewardAmount = u"10¢"
                rewardAmountNum = -.1
                dime1_fbk.pos = [0, yoffset]
                
                xStim1.pos = [0, yoffset]
                xStim1.opacity, xStim2.opacity = 1, 0
            
                usedStims.update([dime1_fbk])
            
            elif valueCondition == "high":  # QUARTER
              if Feedback == 1:  # Win
                feedbackName="Win"
                rewardAmount = "$1.00"
                rewardAmountNum = 1
                quarter1_fbk.pos = [-3 * highXScale, yoffset]
                quarter2_fbk.pos = [-1 * highXScale, yoffset]
                quarter3_fbk.pos = [1 * highXScale, yoffset]
                quarter4_fbk.pos = [3 * highXScale, yoffset]
                xStim1.opacity, xStim2.opacity = 0, 0
                usedStims.update([quarter1_fbk, quarter2_fbk, 
                                  quarter3_fbk, quarter4_fbk])
            
              else: # Lose
                rewardAmount = u"50¢"
                rewardAmountNum = -.5
                feedbackName="Lose"
                quarter1_fbk.pos = [-1.5 * highXScale, yoffset]
                quarter2_fbk.pos = [1.5 * highXScale, yoffset]
                xStim1.pos = [-1.5 * highXScale, yoffset]
                xStim2.pos = [1.5 * highXScale, yoffset]
                xStim1.opacity = 1
                xStim2.opacity = 1
                usedStims.update([quarter1_fbk, quarter2_fbk])
                xStim1.opacity, xStim2.opacity = 1, 1
            
            feedbackMsg = "%s %s" %(feedbackName, rewardAmount)
            RewardMsg="%s" %(rewardAmount)
            feedbackDisplayComponents = []
            
            # Allow for Too Slow on practice trials
            if currentLoop.name == 'practiceTrial' and not guessed_keys:
                feedbackName = 'Too slow'
                feedbackMsg = 'Too slow!'
                rewardAmount = 0
                usedStims.clear()
            
            for stim in usedStims:
              stim.opacity = 1
            
            for stim in fbkStims - usedStims:
              stim.opacity = 0
            
            if feedbackName == 'Win':
                for frame in rewFrames:
                    frame.opacity = 1
                for frame in unrewFrames:
                    frame.opacity = 0
            elif feedbackName == 'Lose':
                for frame in rewFrames:
                    frame.opacity = 0
                for frame in unrewFrames:
                    frame.opacity = 1
            else:  # Don't draw frame on practice no response trial
                for frame in rewFrames + unrewFrames + [xStim1, xStim2]:
                    frame.opacity = 0
            
            if not guessed_keys:
                nNoResp += 1
            
            record_expt_window = True
            fdback.setText(feedbackMsg)
            fdback.setPos(feedbackPos)
            
            
            
            # keep track of which components have finished
            feedbackComponents = []
            feedbackComponents.append(circleFrame)
            feedbackComponents.append(outerSquareFrame)
            feedbackComponents.append(innerSquareFrame)
            feedbackComponents.append(fdback)
            feedbackComponents.append(quarter1_fbk)
            feedbackComponents.append(quarter2_fbk)
            feedbackComponents.append(quarter3_fbk)
            feedbackComponents.append(quarter4_fbk)
            feedbackComponents.append(dime1_fbk)
            feedbackComponents.append(dime2_fbk)
            feedbackComponents.append(xStim1)
            feedbackComponents.append(xStim2)
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "feedback"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = feedbackClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                    if frameN == 0:
                        routineTimer.add(3.0)  # add more time for practice feedback (4s total)
                        feedbackComponents.append(extraFeedbackStim)
                        extraFeedbackStim.status = STARTED  
                        # Draw this forever (so components don't shut off), but maintain the non-slip timing
                
                    # n.b. Some of these stims (e.g. xStim1) may have opacity == 0
                    # and won't be visible, but "draw" them as active stims anyway
                    # for consistency with the rest of the routine. 
                    # Continue drawing stims even if the Builder response thinks
                    # they're done.
                    # Also, must be split up for order (frame in the back, then
                    # feedback, followed by X's on top.
                    for stim in [circleFrame, outerSquareFrame, innerSquareFrame]:
                #        if stim.status == FINISHED:
                        stim.draw()
                    for stim in set([fdback]).union(fbkStims):
                        stim.draw()
                    for stim in [xStim1, xStim2]:
                        stim.draw()
                
                
                # *circleFrame* updates
                if t >= 0.0 and circleFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    circleFrame.tStart = t  # underestimates by a little under one frame
                    circleFrame.frameNStart = frameN  # exact frame index
                    circleFrame.setAutoDraw(True)
                if circleFrame.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    circleFrame.setAutoDraw(False)
                
                # *outerSquareFrame* updates
                if t >= 0.0 and outerSquareFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    outerSquareFrame.tStart = t  # underestimates by a little under one frame
                    outerSquareFrame.frameNStart = frameN  # exact frame index
                    outerSquareFrame.setAutoDraw(True)
                if outerSquareFrame.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    outerSquareFrame.setAutoDraw(False)
                
                # *innerSquareFrame* updates
                if t >= 0.0 and innerSquareFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    innerSquareFrame.tStart = t  # underestimates by a little under one frame
                    innerSquareFrame.frameNStart = frameN  # exact frame index
                    innerSquareFrame.setAutoDraw(True)
                if innerSquareFrame.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    innerSquareFrame.setAutoDraw(False)
                
                # *fdback* updates
                if t >= 0.0 and fdback.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    fdback.tStart = t  # underestimates by a little under one frame
                    fdback.frameNStart = frameN  # exact frame index
                    fdback.setAutoDraw(True)
                if fdback.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    fdback.setAutoDraw(False)
                
                # *quarter1_fbk* updates
                if t >= 0.0 and quarter1_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter1_fbk.tStart = t  # underestimates by a little under one frame
                    quarter1_fbk.frameNStart = frameN  # exact frame index
                    quarter1_fbk.setAutoDraw(True)
                if quarter1_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter1_fbk.setAutoDraw(False)
                
                # *quarter2_fbk* updates
                if t >= 0.0 and quarter2_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter2_fbk.tStart = t  # underestimates by a little under one frame
                    quarter2_fbk.frameNStart = frameN  # exact frame index
                    quarter2_fbk.setAutoDraw(True)
                if quarter2_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter2_fbk.setAutoDraw(False)
                
                # *quarter3_fbk* updates
                if t >= 0.0 and quarter3_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter3_fbk.tStart = t  # underestimates by a little under one frame
                    quarter3_fbk.frameNStart = frameN  # exact frame index
                    quarter3_fbk.setAutoDraw(True)
                if quarter3_fbk.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter3_fbk.setAutoDraw(False)
                
                # *quarter4_fbk* updates
                if t >= 0.0 and quarter4_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter4_fbk.tStart = t  # underestimates by a little under one frame
                    quarter4_fbk.frameNStart = frameN  # exact frame index
                    quarter4_fbk.setAutoDraw(True)
                if quarter4_fbk.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter4_fbk.setAutoDraw(False)
                
                # *dime1_fbk* updates
                if t >= 0.0 and dime1_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime1_fbk.tStart = t  # underestimates by a little under one frame
                    dime1_fbk.frameNStart = frameN  # exact frame index
                    dime1_fbk.setAutoDraw(True)
                if dime1_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime1_fbk.setAutoDraw(False)
                
                # *dime2_fbk* updates
                if t >= 0.0 and dime2_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime2_fbk.tStart = t  # underestimates by a little under one frame
                    dime2_fbk.frameNStart = frameN  # exact frame index
                    dime2_fbk.setAutoDraw(True)
                if dime2_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime2_fbk.setAutoDraw(False)
                
                # *xStim1* updates
                if t >= 0.0 and xStim1.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim1.tStart = t  # underestimates by a little under one frame
                    xStim1.frameNStart = frameN  # exact frame index
                    xStim1.setAutoDraw(True)
                if xStim1.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim1.setAutoDraw(False)
                
                # *xStim2* updates
                if t >= 0.0 and xStim2.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim2.tStart = t  # underestimates by a little under one frame
                    xStim2.frameNStart = frameN  # exact frame index
                    xStim2.setAutoDraw(True)
                if xStim2.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim2.setAutoDraw(False)
                if frameN == 0:
                    feedbackStartTime = fmriClock.getTime()
                
                
                #for prompt in buttonPrompts:
                #    prompt.draw()
                
                if record_expt_window:
                    winRecorder.keyframe()
                    record_expt_window = False
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in feedbackComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "feedback"-------
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            if currentLoop.name != 'practiceTrial' and currentLoop.name != 'kidPracLoop':
                cumulativeReward += rewardAmountNum
            
                currentLoop.addData('guessResp.firstKey', guessRespFirstKey)
                currentLoop.addData('guessResp.firstRt', guessRespFirstRt)
            
                currentLoop.addData('rewardAmount', rewardAmountNum)
                currentLoop.addData('feedbackName', feedbackName)
                currentLoop.addData('cumulativeReward', cumulativeReward)
                currentLoop.addData('cumulativeNoResp', nNoResp)
            
                trialCondition = valueCondition + feedbackName
                currentLoop.addData('trialCondition', trialCondition)
            
                runDataframe.loc[currentN, 'rewardAmount'] = rewardAmountNum
                runDataframe.loc[currentN, 'feedbackName'] = feedbackName
                runDataframe.loc[currentN, 'cumulativeReward'] = cumulativeReward
                runDataframe.loc[currentN, 'cumulativeNoResp'] = nNoResp
                runDataframe.loc[currentN, 'trialCondition'] = trialCondition
            
            usedStims.clear()
            
            xStim1.setAutoDraw(False)
            xStim2.setAutoDraw(False)
            feedbackEndTime = fmriClock.getTime()
            
            if Feedback:
                feedback_str = 'correct'
            else:
                feedback_str = 'incorrect'
            
            if not guessed_keys:
                feedback_str += 'noresponse'
            
            if currentLoop.name == 'trials':
                feedbackOffsetTime = fmriClock.getTime()
                currentLoop.addData('feedbackStartTime', feedbackStartTime)
                currentLoop.addData('feedbackEndTime', feedbackEndTime)
                events.append({'run': runs.thisN + runNumber,
                               'condition': trialCondition + 'Feedback',
                               'onset': feedbackStartTime,
                               'duration': 1,})
                trialInfo = None
                currentLoop.addData('trialNum', trialNum)
                updateRunLog(thisExp.entries)
            
            
        # completed 1 repeats of 'kidPracLoop'
        
    # completed age < 8 repeats of 'kidPracControlLoop'
    
    
    #------Prepare to start Routine "directions3"-------
    t = 0
    directions3Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    startPracScreen = event.BuilderKeyResponse()  # create an object of type KeyResponse
    startPracScreen.status = NOT_STARTED
    nNoResp = 0  # For practice
    
    raVerbalText.text = '"Make a guess by pressing a button every time you see the "?". If you do not guess, the computer will guess for you, so do your best to press every time.\n\nDo you have any questions?\n\nOkay, now let\'s have you make a guess."'
    raText.text =  'Press <space> to continue.'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    directions3Components = []
    directions3Components.append(getReady1Text)
    directions3Components.append(getReady2Text)
    directions3Components.append(startPracScreen)
    for thisComponent in directions3Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directions3"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directions3Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *getReady1Text* updates
        if t >= 0.0 and getReady1Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            getReady1Text.tStart = t  # underestimates by a little under one frame
            getReady1Text.frameNStart = frameN  # exact frame index
            getReady1Text.setAutoDraw(True)
        
        # *getReady2Text* updates
        if t >= 0.0 and getReady2Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            getReady2Text.tStart = t  # underestimates by a little under one frame
            getReady2Text.frameNStart = frameN  # exact frame index
            getReady2Text.setAutoDraw(True)
        
        # *startPracScreen* updates
        if t >= 0.0 and startPracScreen.status == NOT_STARTED:
            # keep track of start time/frame for later
            startPracScreen.tStart = t  # underestimates by a little under one frame
            startPracScreen.frameNStart = frameN  # exact frame index
            startPracScreen.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(startPracScreen.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if startPracScreen.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                startPracScreen.keys = theseKeys[-1]  # just the last key pressed
                startPracScreen.rt = startPracScreen.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directions3Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directions3"-------
    for thisComponent in directions3Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if startPracScreen.keys in ['', [], None]:  # No response was made
       startPracScreen.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('startPracScreen.keys',startPracScreen.keys)
    if startPracScreen.keys != None:  # we had a response
        pracDirections.addData('startPracScreen.rt', startPracScreen.rt)
    raVerbalText.text = ''
    raText.text=''
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # the Routine "directions3" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    practiceTrialGroups = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions('conditions/allPrac.csv'),
        seed=None, name='practiceTrialGroups')
    thisExp.addLoop(practiceTrialGroups)  # add the loop to the experiment
    thisPracticeTrialGroup = practiceTrialGroups.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisPracticeTrialGroup.rgb)
    if thisPracticeTrialGroup != None:
        for paramName in thisPracticeTrialGroup.keys():
            exec(paramName + '= thisPracticeTrialGroup.' + paramName)
    
    for thisPracticeTrialGroup in practiceTrialGroups:
        currentLoop = practiceTrialGroups
        # abbreviate parameter names if possible (e.g. rgb = thisPracticeTrialGroup.rgb)
        if thisPracticeTrialGroup != None:
            for paramName in thisPracticeTrialGroup.keys():
                exec(paramName + '= thisPracticeTrialGroup.' + paramName)
        
        # set up handler to look after randomisation of conditions etc
        practiceTrial = data.TrialHandler(nReps=10, method='random', 
            extraInfo=expInfo, originPath=-1,
            trialList=[None],
            seed=None, name='practiceTrial')
        thisExp.addLoop(practiceTrial)  # add the loop to the experiment
        thisPracticeTrial = practiceTrial.trialList[0]  # so we can initialise stimuli with some values
        # abbreviate parameter names if possible (e.g. rgb=thisPracticeTrial.rgb)
        if thisPracticeTrial != None:
            for paramName in thisPracticeTrial.keys():
                exec(paramName + '= thisPracticeTrial.' + paramName)
        
        for thisPracticeTrial in practiceTrial:
            currentLoop = practiceTrial
            # abbreviate parameter names if possible (e.g. rgb = thisPracticeTrial.rgb)
            if thisPracticeTrial != None:
                for paramName in thisPracticeTrial.keys():
                    exec(paramName + '= thisPracticeTrial.' + paramName)
            
            #------Prepare to start Routine "trial_cue"-------
            t = 0
            trial_cueClock.reset()  # clock 
            frameN = -1
            routineTimer.add(1.500000)
            # update component parameters for each repeat
            # n.b.: Loading textures is expensive and may cause lost frames,
            # so keep dimes and quarters separate and simply set opacity to
            # hide 'invisible' coins.
            
            # Set number of blocks and trials / block by reading trials csv
            # Should be 6 blocks of 4 trials each
            
            # Full trials: set valueCondition from the first trial in the block.
            # Practice trials: set directly from the practice loop.
            if currentLoop.name != 'practiceTrial':
                # Indices for Trial Loop
                row_start = blocks.thisN * trialsPerBlock
                row_end = row_start + trialsPerBlock
                valueCondition = trialsDataframe.loc[row_start, 'valueCondition']
            
            # set win/loss cue displays
            if valueCondition == 'low':  # DIME
                winAmount=u'20¢'
                lossAmount=u'10¢'
                usedStims.update([dime1_cue, dime2_cue, dime3_cue])
                # Possible Win
                dime1_cue.pos = [-1.5, 2.5]
                dime2_cue.pos = [1.5, 2.5]
                # Possible Lose
                dime3_cue.pos = [0, -2.5]
                xStim1_cue.pos = dime3_cue.pos
                xStim1_cue.opacity, xStim2_cue.opacity = 1, 0
            
            else:  # QUARTER
                winAmount='$1.00'
                lossAmount=u'50¢'
                usedStims.update([quarter1_cue, quarter2_cue, quarter3_cue,
                                  quarter4_cue, quarter5_cue, quarter6_cue])
                # Possible Win
                quarter1_cue.pos = [-3 * xscale, 2]
                quarter2_cue.pos = [-1 * xscale, 2]
                quarter3_cue.pos = [1 * xscale, 2]
                quarter4_cue.pos = [3 * xscale, 2]
                # Possible Lose
                quarter5_cue.pos = [-1 * xscale, -2]
                quarter6_cue.pos = [1 * xscale, -2]
                xStim1_cue.pos = quarter5_cue.pos
                xStim2_cue.pos = quarter6_cue.pos
                xStim1_cue.opacity, xStim2_cue.opacity = 1, 1
            
            for coin in cueStims:
                coin.opacity = 1
            for coin in cueStims - usedStims:
                coin.opacity = 0
            
            winCueMsg = "Win %s" %(winAmount)
            lossCueMsg = "Lose %s" %(lossAmount)
            
            # When outside the trials loop, currentTrialN == blockN
            if currentLoop.name not in ['practiceTrial']:
                currentN = blocks.thisRepN * trialsPerBlock
            else:
                currentN = currentLoop.thisN
            trialNum = currentN + 1
            StakesWin.setText(winCueMsg)
            StakesLoss.setText(lossCueMsg)
            
            
            
            # keep track of which components have finished
            trial_cueComponents = []
            trial_cueComponents.append(circle)
            trial_cueComponents.append(line)
            trial_cueComponents.append(StakesWin)
            trial_cueComponents.append(StakesLoss)
            trial_cueComponents.append(dime1_cue)
            trial_cueComponents.append(dime2_cue)
            trial_cueComponents.append(dime3_cue)
            trial_cueComponents.append(quarter1_cue)
            trial_cueComponents.append(quarter2_cue)
            trial_cueComponents.append(quarter3_cue)
            trial_cueComponents.append(quarter4_cue)
            trial_cueComponents.append(quarter5_cue)
            trial_cueComponents.append(quarter6_cue)
            trial_cueComponents.append(xStim1_cue)
            trial_cueComponents.append(xStim2_cue)
            for thisComponent in trial_cueComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "trial_cue"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = trial_cueClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                #for prompt in buttonPrompts:
                #    prompt.draw()
                
                # *circle* updates
                if t >= 0.0 and circle.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    circle.tStart = t  # underestimates by a little under one frame
                    circle.frameNStart = frameN  # exact frame index
                    circle.setAutoDraw(True)
                if circle.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    circle.setAutoDraw(False)
                
                # *line* updates
                if t >= 0.0 and line.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    line.tStart = t  # underestimates by a little under one frame
                    line.frameNStart = frameN  # exact frame index
                    line.setAutoDraw(True)
                if line.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    line.setAutoDraw(False)
                
                # *StakesWin* updates
                if t >= 0.0 and StakesWin.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    StakesWin.tStart = t  # underestimates by a little under one frame
                    StakesWin.frameNStart = frameN  # exact frame index
                    StakesWin.setAutoDraw(True)
                if StakesWin.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    StakesWin.setAutoDraw(False)
                
                # *StakesLoss* updates
                if t >= 0.0 and StakesLoss.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    StakesLoss.tStart = t  # underestimates by a little under one frame
                    StakesLoss.frameNStart = frameN  # exact frame index
                    StakesLoss.setAutoDraw(True)
                if StakesLoss.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    StakesLoss.setAutoDraw(False)
                
                # *dime1_cue* updates
                if t >= 0.0 and dime1_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime1_cue.tStart = t  # underestimates by a little under one frame
                    dime1_cue.frameNStart = frameN  # exact frame index
                    dime1_cue.setAutoDraw(True)
                if dime1_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime1_cue.setAutoDraw(False)
                
                # *dime2_cue* updates
                if t >= 0.0 and dime2_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime2_cue.tStart = t  # underestimates by a little under one frame
                    dime2_cue.frameNStart = frameN  # exact frame index
                    dime2_cue.setAutoDraw(True)
                if dime2_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime2_cue.setAutoDraw(False)
                
                # *dime3_cue* updates
                if t >= 0.0 and dime3_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime3_cue.tStart = t  # underestimates by a little under one frame
                    dime3_cue.frameNStart = frameN  # exact frame index
                    dime3_cue.setAutoDraw(True)
                if dime3_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime3_cue.setAutoDraw(False)
                
                # *quarter1_cue* updates
                if t >= 0.0 and quarter1_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter1_cue.tStart = t  # underestimates by a little under one frame
                    quarter1_cue.frameNStart = frameN  # exact frame index
                    quarter1_cue.setAutoDraw(True)
                if quarter1_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter1_cue.setAutoDraw(False)
                
                # *quarter2_cue* updates
                if t >= 0.0 and quarter2_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter2_cue.tStart = t  # underestimates by a little under one frame
                    quarter2_cue.frameNStart = frameN  # exact frame index
                    quarter2_cue.setAutoDraw(True)
                if quarter2_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter2_cue.setAutoDraw(False)
                
                # *quarter3_cue* updates
                if t >= 0.0 and quarter3_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter3_cue.tStart = t  # underestimates by a little under one frame
                    quarter3_cue.frameNStart = frameN  # exact frame index
                    quarter3_cue.setAutoDraw(True)
                if quarter3_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter3_cue.setAutoDraw(False)
                
                # *quarter4_cue* updates
                if t >= 0.0 and quarter4_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter4_cue.tStart = t  # underestimates by a little under one frame
                    quarter4_cue.frameNStart = frameN  # exact frame index
                    quarter4_cue.setAutoDraw(True)
                if quarter4_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter4_cue.setAutoDraw(False)
                
                # *quarter5_cue* updates
                if t >= 0.0 and quarter5_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter5_cue.tStart = t  # underestimates by a little under one frame
                    quarter5_cue.frameNStart = frameN  # exact frame index
                    quarter5_cue.setAutoDraw(True)
                if quarter5_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter5_cue.setAutoDraw(False)
                
                # *quarter6_cue* updates
                if t >= 0.0 and quarter6_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter6_cue.tStart = t  # underestimates by a little under one frame
                    quarter6_cue.frameNStart = frameN  # exact frame index
                    quarter6_cue.setAutoDraw(True)
                if quarter6_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter6_cue.setAutoDraw(False)
                
                if frameN == 0:
                    cueStartTime = fmriClock.getTime()
                
                
                # *xStim1_cue* updates
                if t >= 0.0 and xStim1_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim1_cue.tStart = t  # underestimates by a little under one frame
                    xStim1_cue.frameNStart = frameN  # exact frame index
                    xStim1_cue.setAutoDraw(True)
                if xStim1_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim1_cue.setAutoDraw(False)
                
                # *xStim2_cue* updates
                if t >= 0.0 and xStim2_cue.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim2_cue.tStart = t  # underestimates by a little under one frame
                    xStim2_cue.frameNStart = frameN  # exact frame index
                    xStim2_cue.setAutoDraw(True)
                if xStim2_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim2_cue.setAutoDraw(False)
                if frameN == 0:
                    winRecorder.keyframe()
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in trial_cueComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "trial_cue"-------
            for thisComponent in trial_cueComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            
            
            if currentLoop.name == 'blocks':
                cueEndTime = fmriClock.getTime()
                currentLoop.addData('cueStartTime', cueStartTime)
                currentLoop.addData('cueEndTime', cueEndTime)
                events.append({'run': runNum,
                               'condition': valueCondition + 'Cue',
                               'onset': cueStartTime,
                               'duration': 1.5,})
            
                ## Create the first trial of a block and add cue time to it.
                #trialInfo = dict(cueOnset=cueOnsetTime,
                #                  cueOffset=cueOffsetTime,
                #                  run=runs.thisN + session)
                runDataframe.loc[currentN, 'cueStartTime'] = cueStartTime
                runDataframe.loc[currentN, 'cueEndTime'] = cueEndTime
                runDataframe.loc[currentN, 'run'] = runs.thisN + runNumber
                currentLoop.addData('trialNum', trialNum)
            
            
            #------Prepare to start Routine "pracJitter"-------
            t = 0
            pracJitterClock.reset()  # clock 
            frameN = -1
            routineTimer.add(2.000000)
            # update component parameters for each repeat
            
            # keep track of which components have finished
            pracJitterComponents = []
            pracJitterComponents.append(pracJitter1)
            for thisComponent in pracJitterComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "pracJitter"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = pracJitterClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                # *pracJitter1* updates
                if t >= 0.0 and pracJitter1.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    pracJitter1.tStart = t  # underestimates by a little under one frame
                    pracJitter1.frameNStart = frameN  # exact frame index
                    pracJitter1.setAutoDraw(True)
                if pracJitter1.status == STARTED and t >= (0.0 + (2.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    pracJitter1.setAutoDraw(False)
                for prompt in buttonPrompts:
                    prompt.draw()
                if frameN == 0:
                    winRecorder.keyframe()
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in pracJitterComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "pracJitter"-------
            for thisComponent in pracJitterComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            
            
            #------Prepare to start Routine "trial"-------
            t = 0
            trialClock.reset()  # clock 
            frameN = -1
            routineTimer.add(2.000000)
            # update component parameters for each repeat
            guessResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
            guessResp.status = NOT_STARTED
            
            raTextUpdated = False
            if currentLoop.name not in ['kidPracTrial', 'practiceTrial']:
                currentN = (blocks.thisN * trialsPerBlock) + trials.thisN
            else:
                currentN = 0
            
            trialNum = currentN + 1
            
            if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                last5=None
                total=None
            else:
                last5=raLast5Text
                total=raTotalText
            
            updateTrialWin(currentLoop.name, runDataframe, currentN,
                           raResponseTopText, '', 
                           last5Text=last5, totalText=total)
            
            
            
            # keep track of which components have finished
            trialComponents = []
            trialComponents.append(guess)
            trialComponents.append(guessResp)
            for thisComponent in trialComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "trial"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = trialClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                # *guess* updates
                if t >= 0 and guess.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    guess.tStart = t  # underestimates by a little under one frame
                    guess.frameNStart = frameN  # exact frame index
                    guess.setAutoDraw(True)
                if guess.status == STARTED and t >= (0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    guess.setAutoDraw(False)
                
                # *guessResp* updates
                if t >= 0.0 and guessResp.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    guessResp.tStart = t  # underestimates by a little under one frame
                    guessResp.frameNStart = frameN  # exact frame index
                    guessResp.status = STARTED
                    # AllowedKeys looks like a variable named `allowed_keys`
                    if not 'allowed_keys' in locals():
                        logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                        core.quit()
                    if not type(allowed_keys) in [list, tuple, np.ndarray]:
                        if not isinstance(allowed_keys, basestring):
                            logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                            core.quit()
                        elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                        else:  allowed_keys = eval(allowed_keys)
                    # keyboard checking is just starting
                    win.callOnFlip(guessResp.clock.reset)  # t=0 on next screen flip
                    event.clearEvents(eventType='keyboard')
                if guessResp.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    guessResp.status = STOPPED
                if guessResp.status == STARTED:
                    theseKeys = event.getKeys(keyList=list(allowed_keys))
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        guessResp.keys.extend(theseKeys)  # storing all keys
                        guessResp.rt.append(guessResp.clock.getTime())
                if frameN == 0:
                    guessStartTime = fmriClock.getTime()
                
                if not raTextUpdated and guessResp.keys:
                    runDataframe.loc[currentN, 'guessResp.keys'] = str(guessResp.keys)
                    runDataframe.loc[currentN, 'guessResp.rt'] = str(guessResp.rt)
                
                    guessRespFirstKey = guessResp.keys[0]
                    guessRespFirstRt = guessResp.rt[0]
                
                    if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                        last5=None
                        total=None
                    else:
                        last5=raLast5Text
                        total=raTotalText
                
                    # Don't update trials for practice.
                    updateTrialWin(currentLoop.name, runDataframe, currentN,
                                   raResponseTopText, guessRespFirstKey, 
                                   last5Text=last5, totalText=total)
                
                    raTextUpdated = True
                
                for prompt in buttonPrompts:
                    prompt.draw()
                
                if frameN == 0:
                    winRecorder.keyframe()
                
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in trialComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "trial"-------
            for thisComponent in trialComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            # check responses
            if guessResp.keys in ['', [], None]:  # No response was made
               guessResp.keys=None
            # store data for practiceTrial (TrialHandler)
            practiceTrial.addData('guessResp.keys',guessResp.keys)
            if guessResp.keys != None:  # we had a response
                practiceTrial.addData('guessResp.rt', guessResp.rt)
            # Also set in trial > raWinTrialCode > Each Frame for inter-trial presses
            if guessResp.keys:
                guessRespFirstKey = guessResp.keys[0]
                guessRespFirstRt = guessResp.rt[0]
                guessRespFirstGuess = guessLookup[guessRespFirstKey]
            else:
                guessRespFirstKey = None
                guessRespFirstRt = None
                guessRespFirstGuess = None
            
            if currentLoop.name == 'trials':
                guessEndTime = fmriClock.getTime()
                currentLoop.addData('guessStartTime', guessStartTime)
                currentLoop.addData('guessEndTime', guessEndTime)
                currentLoop.addData('guessResp.firstGuess', guessRespFirstGuess)
                events.append({'run': runNum,
                               'condition': 'guess',
                               'onset': guessStartTime,
                               'duration': 1.5})
                runDataframe.loc[currentN, 'trial'] = currentN + 1
                runDataframe.loc[currentN, 'run'] = runNum
                runDataframe.loc[currentN, 'guessStartTime'] = guessStartTime
                runDataframe.loc[currentN, 'guessEndTime'] = guessEndTime
                runDataframe.loc[currentN, 'guessResp.keys'] = str(guessResp.keys)
                runDataframe.loc[currentN, 'guessResp.firstKey'] = guessRespFirstKey
                runDataframe.loc[currentN, 'guessResp.firstRt'] = guessRespFirstRt
                runDataframe.loc[currentN, 'guessResp.firstGuess'] = guessRespFirstGuess
            
                if guessResp.keys:
                    runDataframe.loc[currentN, 'guessResp.rt'] = str(guessResp.rt)
                # else:  # Leave as nan
            
            guessed_keys = guessResp.keys
            if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                updateTrialWin(currentLoop.name, runDataframe, currentN,
                               raResponseTopText, guessRespFirstKey)
            else:
                updateTrialWin(currentLoop.name, runDataframe, currentN,
                               raResponseTopText, guessRespFirstKey, 
                               raLast5Text, raTotalText)
            
            
            
            
            #------Prepare to start Routine "pracJitter"-------
            t = 0
            pracJitterClock.reset()  # clock 
            frameN = -1
            routineTimer.add(2.000000)
            # update component parameters for each repeat
            
            # keep track of which components have finished
            pracJitterComponents = []
            pracJitterComponents.append(pracJitter1)
            for thisComponent in pracJitterComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "pracJitter"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = pracJitterClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                # *pracJitter1* updates
                if t >= 0.0 and pracJitter1.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    pracJitter1.tStart = t  # underestimates by a little under one frame
                    pracJitter1.frameNStart = frameN  # exact frame index
                    pracJitter1.setAutoDraw(True)
                if pracJitter1.status == STARTED and t >= (0.0 + (2.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    pracJitter1.setAutoDraw(False)
                for prompt in buttonPrompts:
                    prompt.draw()
                if frameN == 0:
                    winRecorder.keyframe()
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in pracJitterComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "pracJitter"-------
            for thisComponent in pracJitterComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            
            
            #------Prepare to start Routine "feedback"-------
            t = 0
            feedbackClock.reset()  # clock 
            frameN = -1
            routineTimer.add(1.000000)
            # update component parameters for each repeat
            # calculate feedback and reward and account for missed responses
            fbkStims = set([dime1_fbk, dime2_fbk, quarter1_fbk, quarter2_fbk,
                            quarter3_fbk, quarter4_fbk])
            feedbackPos = [0, 2]
            
            usedStims.clear()
            
            if valueCondition == "low":  # DIME
              if Feedback == 1:  # Win
                feedbackName = "Win"
                rewardAmount = u"20¢"
                rewardAmountNum = .20
                dime1_fbk.pos = [-1 * xscale, yoffset]
                dime2_fbk.pos = [1 * xscale, yoffset]
                usedStims.update([dime1_fbk, dime2_fbk])
                xStim1.opacity, xStim2.opacity = 0, 0
            
              else:  # Lose
                feedbackName = "Lose"
                rewardAmount = u"10¢"
                rewardAmountNum = -.1
                dime1_fbk.pos = [0, yoffset]
                
                xStim1.pos = [0, yoffset]
                xStim1.opacity, xStim2.opacity = 1, 0
            
                usedStims.update([dime1_fbk])
            
            elif valueCondition == "high":  # QUARTER
              if Feedback == 1:  # Win
                feedbackName="Win"
                rewardAmount = "$1.00"
                rewardAmountNum = 1
                quarter1_fbk.pos = [-3 * highXScale, yoffset]
                quarter2_fbk.pos = [-1 * highXScale, yoffset]
                quarter3_fbk.pos = [1 * highXScale, yoffset]
                quarter4_fbk.pos = [3 * highXScale, yoffset]
                xStim1.opacity, xStim2.opacity = 0, 0
                usedStims.update([quarter1_fbk, quarter2_fbk, 
                                  quarter3_fbk, quarter4_fbk])
            
              else: # Lose
                rewardAmount = u"50¢"
                rewardAmountNum = -.5
                feedbackName="Lose"
                quarter1_fbk.pos = [-1.5 * highXScale, yoffset]
                quarter2_fbk.pos = [1.5 * highXScale, yoffset]
                xStim1.pos = [-1.5 * highXScale, yoffset]
                xStim2.pos = [1.5 * highXScale, yoffset]
                xStim1.opacity = 1
                xStim2.opacity = 1
                usedStims.update([quarter1_fbk, quarter2_fbk])
                xStim1.opacity, xStim2.opacity = 1, 1
            
            feedbackMsg = "%s %s" %(feedbackName, rewardAmount)
            RewardMsg="%s" %(rewardAmount)
            feedbackDisplayComponents = []
            
            # Allow for Too Slow on practice trials
            if currentLoop.name == 'practiceTrial' and not guessed_keys:
                feedbackName = 'Too slow'
                feedbackMsg = 'Too slow!'
                rewardAmount = 0
                usedStims.clear()
            
            for stim in usedStims:
              stim.opacity = 1
            
            for stim in fbkStims - usedStims:
              stim.opacity = 0
            
            if feedbackName == 'Win':
                for frame in rewFrames:
                    frame.opacity = 1
                for frame in unrewFrames:
                    frame.opacity = 0
            elif feedbackName == 'Lose':
                for frame in rewFrames:
                    frame.opacity = 0
                for frame in unrewFrames:
                    frame.opacity = 1
            else:  # Don't draw frame on practice no response trial
                for frame in rewFrames + unrewFrames + [xStim1, xStim2]:
                    frame.opacity = 0
            
            if not guessed_keys:
                nNoResp += 1
            
            record_expt_window = True
            fdback.setText(feedbackMsg)
            fdback.setPos(feedbackPos)
            
            
            
            # keep track of which components have finished
            feedbackComponents = []
            feedbackComponents.append(circleFrame)
            feedbackComponents.append(outerSquareFrame)
            feedbackComponents.append(innerSquareFrame)
            feedbackComponents.append(fdback)
            feedbackComponents.append(quarter1_fbk)
            feedbackComponents.append(quarter2_fbk)
            feedbackComponents.append(quarter3_fbk)
            feedbackComponents.append(quarter4_fbk)
            feedbackComponents.append(dime1_fbk)
            feedbackComponents.append(dime2_fbk)
            feedbackComponents.append(xStim1)
            feedbackComponents.append(xStim2)
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "feedback"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = feedbackClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                    if frameN == 0:
                        routineTimer.add(3.0)  # add more time for practice feedback (4s total)
                        feedbackComponents.append(extraFeedbackStim)
                        extraFeedbackStim.status = STARTED  
                        # Draw this forever (so components don't shut off), but maintain the non-slip timing
                
                    # n.b. Some of these stims (e.g. xStim1) may have opacity == 0
                    # and won't be visible, but "draw" them as active stims anyway
                    # for consistency with the rest of the routine. 
                    # Continue drawing stims even if the Builder response thinks
                    # they're done.
                    # Also, must be split up for order (frame in the back, then
                    # feedback, followed by X's on top.
                    for stim in [circleFrame, outerSquareFrame, innerSquareFrame]:
                #        if stim.status == FINISHED:
                        stim.draw()
                    for stim in set([fdback]).union(fbkStims):
                        stim.draw()
                    for stim in [xStim1, xStim2]:
                        stim.draw()
                
                
                # *circleFrame* updates
                if t >= 0.0 and circleFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    circleFrame.tStart = t  # underestimates by a little under one frame
                    circleFrame.frameNStart = frameN  # exact frame index
                    circleFrame.setAutoDraw(True)
                if circleFrame.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    circleFrame.setAutoDraw(False)
                
                # *outerSquareFrame* updates
                if t >= 0.0 and outerSquareFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    outerSquareFrame.tStart = t  # underestimates by a little under one frame
                    outerSquareFrame.frameNStart = frameN  # exact frame index
                    outerSquareFrame.setAutoDraw(True)
                if outerSquareFrame.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    outerSquareFrame.setAutoDraw(False)
                
                # *innerSquareFrame* updates
                if t >= 0.0 and innerSquareFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    innerSquareFrame.tStart = t  # underestimates by a little under one frame
                    innerSquareFrame.frameNStart = frameN  # exact frame index
                    innerSquareFrame.setAutoDraw(True)
                if innerSquareFrame.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    innerSquareFrame.setAutoDraw(False)
                
                # *fdback* updates
                if t >= 0.0 and fdback.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    fdback.tStart = t  # underestimates by a little under one frame
                    fdback.frameNStart = frameN  # exact frame index
                    fdback.setAutoDraw(True)
                if fdback.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    fdback.setAutoDraw(False)
                
                # *quarter1_fbk* updates
                if t >= 0.0 and quarter1_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter1_fbk.tStart = t  # underestimates by a little under one frame
                    quarter1_fbk.frameNStart = frameN  # exact frame index
                    quarter1_fbk.setAutoDraw(True)
                if quarter1_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter1_fbk.setAutoDraw(False)
                
                # *quarter2_fbk* updates
                if t >= 0.0 and quarter2_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter2_fbk.tStart = t  # underestimates by a little under one frame
                    quarter2_fbk.frameNStart = frameN  # exact frame index
                    quarter2_fbk.setAutoDraw(True)
                if quarter2_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter2_fbk.setAutoDraw(False)
                
                # *quarter3_fbk* updates
                if t >= 0.0 and quarter3_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter3_fbk.tStart = t  # underestimates by a little under one frame
                    quarter3_fbk.frameNStart = frameN  # exact frame index
                    quarter3_fbk.setAutoDraw(True)
                if quarter3_fbk.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter3_fbk.setAutoDraw(False)
                
                # *quarter4_fbk* updates
                if t >= 0.0 and quarter4_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter4_fbk.tStart = t  # underestimates by a little under one frame
                    quarter4_fbk.frameNStart = frameN  # exact frame index
                    quarter4_fbk.setAutoDraw(True)
                if quarter4_fbk.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter4_fbk.setAutoDraw(False)
                
                # *dime1_fbk* updates
                if t >= 0.0 and dime1_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime1_fbk.tStart = t  # underestimates by a little under one frame
                    dime1_fbk.frameNStart = frameN  # exact frame index
                    dime1_fbk.setAutoDraw(True)
                if dime1_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime1_fbk.setAutoDraw(False)
                
                # *dime2_fbk* updates
                if t >= 0.0 and dime2_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime2_fbk.tStart = t  # underestimates by a little under one frame
                    dime2_fbk.frameNStart = frameN  # exact frame index
                    dime2_fbk.setAutoDraw(True)
                if dime2_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime2_fbk.setAutoDraw(False)
                
                # *xStim1* updates
                if t >= 0.0 and xStim1.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim1.tStart = t  # underestimates by a little under one frame
                    xStim1.frameNStart = frameN  # exact frame index
                    xStim1.setAutoDraw(True)
                if xStim1.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim1.setAutoDraw(False)
                
                # *xStim2* updates
                if t >= 0.0 and xStim2.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim2.tStart = t  # underestimates by a little under one frame
                    xStim2.frameNStart = frameN  # exact frame index
                    xStim2.setAutoDraw(True)
                if xStim2.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim2.setAutoDraw(False)
                if frameN == 0:
                    feedbackStartTime = fmriClock.getTime()
                
                
                #for prompt in buttonPrompts:
                #    prompt.draw()
                
                if record_expt_window:
                    winRecorder.keyframe()
                    record_expt_window = False
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in feedbackComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "feedback"-------
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            if currentLoop.name != 'practiceTrial' and currentLoop.name != 'kidPracLoop':
                cumulativeReward += rewardAmountNum
            
                currentLoop.addData('guessResp.firstKey', guessRespFirstKey)
                currentLoop.addData('guessResp.firstRt', guessRespFirstRt)
            
                currentLoop.addData('rewardAmount', rewardAmountNum)
                currentLoop.addData('feedbackName', feedbackName)
                currentLoop.addData('cumulativeReward', cumulativeReward)
                currentLoop.addData('cumulativeNoResp', nNoResp)
            
                trialCondition = valueCondition + feedbackName
                currentLoop.addData('trialCondition', trialCondition)
            
                runDataframe.loc[currentN, 'rewardAmount'] = rewardAmountNum
                runDataframe.loc[currentN, 'feedbackName'] = feedbackName
                runDataframe.loc[currentN, 'cumulativeReward'] = cumulativeReward
                runDataframe.loc[currentN, 'cumulativeNoResp'] = nNoResp
                runDataframe.loc[currentN, 'trialCondition'] = trialCondition
            
            usedStims.clear()
            
            xStim1.setAutoDraw(False)
            xStim2.setAutoDraw(False)
            feedbackEndTime = fmriClock.getTime()
            
            if Feedback:
                feedback_str = 'correct'
            else:
                feedback_str = 'incorrect'
            
            if not guessed_keys:
                feedback_str += 'noresponse'
            
            if currentLoop.name == 'trials':
                feedbackOffsetTime = fmriClock.getTime()
                currentLoop.addData('feedbackStartTime', feedbackStartTime)
                currentLoop.addData('feedbackEndTime', feedbackEndTime)
                events.append({'run': runs.thisN + runNumber,
                               'condition': trialCondition + 'Feedback',
                               'onset': feedbackStartTime,
                               'duration': 1,})
                trialInfo = None
                currentLoop.addData('trialNum', trialNum)
                updateRunLog(thisExp.entries)
            
            
            
            #------Prepare to start Routine "pracCheckResponse"-------
            t = 0
            pracCheckResponseClock.reset()  # clock 
            frameN = -1
            # update component parameters for each repeat
            
            pracCheckResponseResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
            pracCheckResponseResp.status = NOT_STARTED
            
            # keep track of which components have finished
            pracCheckResponseComponents = []
            pracCheckResponseComponents.append(pracCheckReponseText)
            pracCheckResponseComponents.append(pracCheckResponseResp)
            for thisComponent in pracCheckResponseComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "pracCheckResponse"-------
            continueRoutine = True
            while continueRoutine:
                # get current time
                t = pracCheckResponseClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                if frameN == 0:
                    if feedbackName == 'Too slow':
                        raVerbalText.text = '"%s"' % pracFailMessage
                        raText.text = 'Press <space> to continue'
                        raText.draw()
                        raVerbalText.draw
                        raWinRecorder.keyframe()
                        raWin.flip()
                        record_expt_window = True
                    else:
                        currentLoop.finished = True
                        continueRoutine = False
                    xStim1.opacity = 0
                    xStim2.opacity = 0
                
                
                
                # *pracCheckReponseText* updates
                if t >= 0.0 and pracCheckReponseText.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    pracCheckReponseText.tStart = t  # underestimates by a little under one frame
                    pracCheckReponseText.frameNStart = frameN  # exact frame index
                    pracCheckReponseText.setAutoDraw(True)
                
                # *pracCheckResponseResp* updates
                if t >= 0.0 and pracCheckResponseResp.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    pracCheckResponseResp.tStart = t  # underestimates by a little under one frame
                    pracCheckResponseResp.frameNStart = frameN  # exact frame index
                    pracCheckResponseResp.status = STARTED
                    # keyboard checking is just starting
                    event.clearEvents(eventType='keyboard')
                if pracCheckResponseResp.status == STARTED:
                    theseKeys = event.getKeys(keyList=['space'])
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        # a response ends the routine
                        continueRoutine = False
                if record_expt_window:
                    winRecorder.keyframe()
                    record_expt_window = False
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in pracCheckResponseComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "pracCheckResponse"-------
            for thisComponent in pracCheckResponseComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            raVerbalText.text = ''
            raText.text = ''
            raText.draw()
            raVerbalText.draw()
            raWin.flip()
            
            
            # the Routine "pracCheckResponse" was not non-slip safe, so reset the non-slip timer
            routineTimer.reset()
        # completed 10 repeats of 'practiceTrial'
        
    # completed 1 repeats of 'practiceTrialGroups'
    
# completed expInfo['runNumber'] == 'practice' repeats of 'pracDirections'


# set up handler to look after randomisation of conditions etc
scanDirections = data.TrialHandler(nReps=expInfo['runNumber'] != 'practice', method='random', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='scanDirections')
thisExp.addLoop(scanDirections)  # add the loop to the experiment
thisScanDirection = scanDirections.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisScanDirection.rgb)
if thisScanDirection != None:
    for paramName in thisScanDirection.keys():
        exec(paramName + '= thisScanDirection.' + paramName)

for thisScanDirection in scanDirections:
    currentLoop = scanDirections
    # abbreviate parameter names if possible (e.g. rgb = thisScanDirection.rgb)
    if thisScanDirection != None:
        for paramName in thisScanDirection.keys():
            exec(paramName + '= thisScanDirection.' + paramName)
    
    #------Prepare to start Routine "qMark"-------
    t = 0
    qMarkClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    qmarkResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    qmarkResp.status = NOT_STARTED
    raText.draw()
    raWin.flip()
    raVerbalText.text = '"When you see the question mark, you should press a button to make your guess about whether a baby or an adult is hiding behind the question mark."'
    raText.text = 'Press <space> to continue.'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    record_expt_window = True
    # keep track of which components have finished
    qMarkComponents = []
    qMarkComponents.append(qmark)
    qMarkComponents.append(qmarkText)
    qMarkComponents.append(qmarkResp)
    for thisComponent in qMarkComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "qMark"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = qMarkClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *qmark* updates
        if t >= 0.0 and qmark.status == NOT_STARTED:
            # keep track of start time/frame for later
            qmark.tStart = t  # underestimates by a little under one frame
            qmark.frameNStart = frameN  # exact frame index
            qmark.setAutoDraw(True)
        
        # *qmarkText* updates
        if t >= 0.0 and qmarkText.status == NOT_STARTED:
            # keep track of start time/frame for later
            qmarkText.tStart = t  # underestimates by a little under one frame
            qmarkText.frameNStart = frameN  # exact frame index
            qmarkText.setAutoDraw(True)
        
        # *qmarkResp* updates
        if t >= 0.0 and qmarkResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            qmarkResp.tStart = t  # underestimates by a little under one frame
            qmarkResp.frameNStart = frameN  # exact frame index
            qmarkResp.status = STARTED
            # AllowedKeys looks like a variable named `all_allowed_keys`
            if not 'all_allowed_keys' in locals():
                logging.error('AllowedKeys variable `all_allowed_keys` is not defined.')
                core.quit()
            if not type(all_allowed_keys) in [list, tuple, np.ndarray]:
                if not isinstance(all_allowed_keys, basestring):
                    logging.error('AllowedKeys variable `all_allowed_keys` is not string- or list-like.')
                    core.quit()
                elif not ',' in all_allowed_keys: all_allowed_keys = (all_allowed_keys,)
                else:  all_allowed_keys = eval(all_allowed_keys)
            # keyboard checking is just starting
            win.callOnFlip(qmarkResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if qmarkResp.status == STARTED:
            theseKeys = event.getKeys(keyList=list(all_allowed_keys))
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                qmarkResp.keys = theseKeys[-1]  # just the last key pressed
                qmarkResp.rt = qmarkResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if record_expt_window:
            winRecorder.keyframe()
            record_expt_window = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in qMarkComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "qMark"-------
    for thisComponent in qMarkComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if qmarkResp.keys in ['', [], None]:  # No response was made
       qmarkResp.keys=None
    # store data for scanDirections (TrialHandler)
    scanDirections.addData('qmarkResp.keys',qmarkResp.keys)
    if qmarkResp.keys != None:  # we had a response
        scanDirections.addData('qmarkResp.rt', qmarkResp.rt)
    
    # the Routine "qMark" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "directionsInstructions1"-------
    t = 0
    directionsInstructions1Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    npresses=0
    raVerbalText.text = '"If you think it is a baby, you should press your POINTER finger."'
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    if expInfo['runNumber']=='practice':
        raText.draw()
        raWin.flip()
        raVerbalText.text='"When you see the question mark, you should press a button to make your guess about whether a baby or an adult is hiding behind the question mark.\n\nIf you think it is a baby, you should press the button under your POINTER finger."'
        raText.pos = [0, -4]
        raText.draw()
        raVerbalText.draw()
        raWinRecorder.keyframe()
        record_expt_window = True
        raWin.flip()
    
    record_expt_window = True
    # keep track of which components have finished
    directionsInstructions1Components = []
    directionsInstructions1Components.append(imgBaby2)
    directionsInstructions1Components.append(imgPointer)
    directionsInstructions1Components.append(imgAdult1b)
    directionsInstructions1Components.append(imgAdult2b)
    directionsInstructions1Components.append(imgMiddle)
    for thisComponent in directionsInstructions1Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "directionsInstructions1"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = directionsInstructions1Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if event.getKeys(keyList=['space']):
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = '"If you think it is an adult, you should press the button under your MIDDLE finger."'
                raText.text = 'Press <space> to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            #elif npresses == 2:
            else:
                continueRoutine = False
                record_expt_win = False
        
        # *imgBaby2* updates
        if t >= 0 and imgBaby2.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgBaby2.tStart = t  # underestimates by a little under one frame
            imgBaby2.frameNStart = frameN  # exact frame index
            imgBaby2.setAutoDraw(True)
        
        # *imgPointer* updates
        if t >= 0 and imgPointer.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgPointer.tStart = t  # underestimates by a little under one frame
            imgPointer.frameNStart = frameN  # exact frame index
            imgPointer.setAutoDraw(True)
        
        # *imgAdult1b* updates
        if (npresses==1) and imgAdult1b.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgAdult1b.tStart = t  # underestimates by a little under one frame
            imgAdult1b.frameNStart = frameN  # exact frame index
            imgAdult1b.setAutoDraw(True)
        
        # *imgAdult2b* updates
        if (npresses == 1) and imgAdult2b.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgAdult2b.tStart = t  # underestimates by a little under one frame
            imgAdult2b.frameNStart = frameN  # exact frame index
            imgAdult2b.setAutoDraw(True)
        
        # *imgMiddle* updates
        if (npresses==1) and imgMiddle.status == NOT_STARTED:
            # keep track of start time/frame for later
            imgMiddle.tStart = t  # underestimates by a little under one frame
            imgMiddle.frameNStart = frameN  # exact frame index
            imgMiddle.setAutoDraw(True)
        if record_expt_window:
            winRecorder.keyframe()
            record_expt_window = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in directionsInstructions1Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "directionsInstructions1"-------
    for thisComponent in directionsInstructions1Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    raText.text = ''
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # the Routine "directionsInstructions1" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
# completed expInfo['runNumber'] != 'practice' repeats of 'scanDirections'


# set up handler to look after randomisation of conditions etc
runs = data.TrialHandler(nReps=nRuns, method='sequential', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='runs')
thisExp.addLoop(runs)  # add the loop to the experiment
thisRun = runs.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisRun.rgb)
if thisRun != None:
    for paramName in thisRun.keys():
        exec(paramName + '= thisRun.' + paramName)

for thisRun in runs:
    currentLoop = runs
    # abbreviate parameter names if possible (e.g. rgb = thisRun.rgb)
    if thisRun != None:
        for paramName in thisRun.keys():
            exec(paramName + '= thisRun.' + paramName)
    
    #------Prepare to start Routine "getReady_2"-------
    t = 0
    getReady_2Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    if expInfo['runNumber'] == 'practice':
        getReadyMsg = '''Do you have any questions? \n\n\nAre you ready to begin the practice?'''
    else:
        getReadyMsg = 'Be sure to press every time you see a "?"'
    
    events = []  # run, condition, onset, duration, value
    
    cumulativeReward = 0.0  # For this run
    nNoResp = 0  # For this run
    
    # Set number of blocks and trials / block by reading trials csv
    # Should be 6 blocks of 4 trials each
    
    runNum = runNumber + runs.thisN
    
    if runNumber == 0:
        trialsList = 'conditions/trialsPractice.csv'
    else:
        trialsList = 'conditions/trials_run%d.csv' % runNum
    logging.exp('Using conditions file %s' % trialsList)
    trialsDataframe = pd.read_csv(trialsList)
    
    blockCounts = trialsDataframe['blockN'].value_counts()
    if blockCounts.unique().size != 1:
        pat = 'All blocks in trials list %s must be the same length, got: %s'
        raise IOError(pat % (trialsList, blockCounts))
    else:
        nBlocks = blockCounts.size
        trialsPerBlock = blockCounts.unique()[0]
        totalTrials = nBlocks * trialsPerBlock
    
    totalTrialsN = trialsDataframe.shape[0]
    
    # Initialize a new df for real runs (vs. example trials)
    runDataframe = newTrialFrame(totalTrialsN)
    runOutfile = '%s_wide.csv' % filename
    getReadyText.setText(getReadyMsg)
    if expInfo['runNumber'] == 'practice':
        verbalMsg1 = (
            '"Now you will complete a short practice that will look '
            'exactly like the game you will play in the scanner.\n\n'
            'Just try your best to follow the instructions and '
            'play the game.')
        verbalMsg2 = (
            'During this practice you will not receive any real money, '
            'but when you play this game in the scanner you will be '
            'paid however much you make in real money! \n\nAre you '
            'ready to begin the practice?"')
        raVerbalText.text = verbalMsg1
        raText.text = 'Press <space> to continue.'
        raText.pos = [0, -5]
        raVerbalText.draw()
        raText.draw()
        raWinRecorder.keyframe()
        record_expt_window = True
        raWin.flip()
    else:
        verbalMsg1 = (
            '"Remember that the computer will choose for you if you '
            'don\'t respond, so be sure to press one of the buttons '
            'every time you see a \'?\'."')
        raText.text = 'Press <space> to continue.'
        raVerbalText.text = verbalMsg1
        raVerbalText.draw()
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
    npresses = 0
    record_expt_window = True
    # keep track of which components have finished
    getReady_2Components = []
    getReady_2Components.append(getReadyText)
    getReady_2Components.append(startText)
    for thisComponent in getReady_2Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "getReady_2"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = getReady_2Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        
        # *getReadyText* updates
        if t >= 0.0 and getReadyText.status == NOT_STARTED:
            # keep track of start time/frame for later
            getReadyText.tStart = t  # underestimates by a little under one frame
            getReadyText.frameNStart = frameN  # exact frame index
            getReadyText.setAutoDraw(True)
        
        # *startText* updates
        if t >= 0.0 and startText.status == NOT_STARTED:
            # keep track of start time/frame for later
            startText.tStart = t  # underestimates by a little under one frame
            startText.frameNStart = frameN  # exact frame index
            startText.setAutoDraw(True)
        if event.getKeys(keyList = ['space']):
            npresses += 1
            if expInfo['runNumber'] != 'practice':
                continueRoutine = False
            elif npresses == 1:
                raVerbalText.text = verbalMsg2
                raText.text = 'Press <space> to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                record_expt_window = True
                raWin.flip()
            else:
                continueRoutine = False
        if record_expt_window:
            winRecorder.keyframe()
            record_expt_window = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in getReady_2Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "getReady_2"-------
    for thisComponent in getReady_2Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    if expInfo['runNumber'] != 'practice':
        raText.text = 'Start the scan.'
    else:
        raText.text = ''
    raVerbalText.text = ''
    raText.pos = [0, 0]
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # the Routine "getReady_2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "wait_for_scanner"-------
    t = 0
    wait_for_scannerClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    waitForScannerText.setText(waitForScannerMsg)
    
    # keep track of which components have finished
    wait_for_scannerComponents = []
    wait_for_scannerComponents.append(waitForScannerText)
    for thisComponent in wait_for_scannerComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "wait_for_scanner"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = wait_for_scannerClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *waitForScannerText* updates
        if t >= 0.0 and waitForScannerText.status == NOT_STARTED:
            # keep track of start time/frame for later
            waitForScannerText.tStart = t  # underestimates by a little under one frame
            waitForScannerText.frameNStart = frameN  # exact frame index
            waitForScannerText.setAutoDraw(True)
        if frameN == 0:
            winRecorder.keyframe()
        
        if expInfo['mriMode'] != 'Off':
            if trigger == 'usb':
                vol = launchScan(win, MR_settings, 
                      globalClock=fmriClock, 
                      mode=expInfo['mriMode'],
                      wait_msg='')
        
            elif trigger == 'parallel':
                address = 0x378
                #parallel.setPortAddress(0x378)
                wait_msg = "Get ready!"
                pinStatus = winioport.inp(address)
                waitMsgStim = visual.TextStim(win, color='white', text=wait_msg)
                waitMsgStim.draw()
                win.flip()
                while True:
                    if pinStatus != winioport.inp(address):
                       break
                       # start exp when pin values change
                fmriClock.reset()
                logging.exp('parallel trigger: start of scan')
                win.flip()  # blank the screen on first sync pulse received
        else:
            fmriClock.reset()  # Practice
        
        win.flip()
        
        triggerWallTime = datetime.datetime.today()
        expInfo['triggerWallTime'] = triggerWallTime.strftime('%a %b %d %H:%M:%S %Y')
        expInfo['triggerGlobalTime'] = globalClock.getTime()
        continueRoutine = False
        
        filename = _thisDir + os.sep + u'data/' + '_'.join([expName, expInfo['sessionID'], 'run%s'%runNum, triggerWallTime.strftime('%Y-%m-%d_%H%M%S')])
        
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in wait_for_scannerComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "wait_for_scanner"-------
    for thisComponent in wait_for_scannerComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    raText.text = ''
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # the Routine "wait_for_scanner" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    countdowns = data.TrialHandler(nReps=4, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='countdowns')
    thisExp.addLoop(countdowns)  # add the loop to the experiment
    thisCountdown = countdowns.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisCountdown.rgb)
    if thisCountdown != None:
        for paramName in thisCountdown.keys():
            exec(paramName + '= thisCountdown.' + paramName)
    
    for thisCountdown in countdowns:
        currentLoop = countdowns
        # abbreviate parameter names if possible (e.g. rgb = thisCountdown.rgb)
        if thisCountdown != None:
            for paramName in thisCountdown.keys():
                exec(paramName + '= thisCountdown.' + paramName)
        
        #------Prepare to start Routine "countdown"-------
        t = 0
        countdownClock.reset()  # clock 
        frameN = -1
        routineTimer.add(2.000000)
        # update component parameters for each repeat
        countdownTime = fmriClock.getTime()
        countdownMsg = currentLoop.nRemaining + 1
        countdownText.setText(countdownMsg)
        
        # keep track of which components have finished
        countdownComponents = []
        countdownComponents.append(countdownText)
        for thisComponent in countdownComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "countdown"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = countdownClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            
            # *countdownText* updates
            if t >= 0.0 and countdownText.status == NOT_STARTED:
                # keep track of start time/frame for later
                countdownText.tStart = t  # underestimates by a little under one frame
                countdownText.frameNStart = frameN  # exact frame index
                countdownText.setAutoDraw(True)
            if countdownText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                countdownText.setAutoDraw(False)
            if frameN == 0:
                winRecorder.keyframe()
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in countdownComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "countdown"-------
        for thisComponent in countdownComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        currentLoop.addData('countdownStartTime', countdownTime)
        currentLoop.addData('countdownLabel', countdownMsg)
        currentLoop.addData('trialNum', 0)
        
        
        thisExp.nextEntry()
        
    # completed 4 repeats of 'countdowns'
    
    
    # set up handler to look after randomisation of conditions etc
    blocks = data.TrialHandler(nReps=nBlocks, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='blocks')
    thisExp.addLoop(blocks)  # add the loop to the experiment
    thisBlock = blocks.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisBlock.rgb)
    if thisBlock != None:
        for paramName in thisBlock.keys():
            exec(paramName + '= thisBlock.' + paramName)
    
    for thisBlock in blocks:
        currentLoop = blocks
        # abbreviate parameter names if possible (e.g. rgb = thisBlock.rgb)
        if thisBlock != None:
            for paramName in thisBlock.keys():
                exec(paramName + '= thisBlock.' + paramName)
        
        #------Prepare to start Routine "trial_cue"-------
        t = 0
        trial_cueClock.reset()  # clock 
        frameN = -1
        routineTimer.add(1.500000)
        # update component parameters for each repeat
        # n.b.: Loading textures is expensive and may cause lost frames,
        # so keep dimes and quarters separate and simply set opacity to
        # hide 'invisible' coins.
        
        # Set number of blocks and trials / block by reading trials csv
        # Should be 6 blocks of 4 trials each
        
        # Full trials: set valueCondition from the first trial in the block.
        # Practice trials: set directly from the practice loop.
        if currentLoop.name != 'practiceTrial':
            # Indices for Trial Loop
            row_start = blocks.thisN * trialsPerBlock
            row_end = row_start + trialsPerBlock
            valueCondition = trialsDataframe.loc[row_start, 'valueCondition']
        
        # set win/loss cue displays
        if valueCondition == 'low':  # DIME
            winAmount=u'20¢'
            lossAmount=u'10¢'
            usedStims.update([dime1_cue, dime2_cue, dime3_cue])
            # Possible Win
            dime1_cue.pos = [-1.5, 2.5]
            dime2_cue.pos = [1.5, 2.5]
            # Possible Lose
            dime3_cue.pos = [0, -2.5]
            xStim1_cue.pos = dime3_cue.pos
            xStim1_cue.opacity, xStim2_cue.opacity = 1, 0
        
        else:  # QUARTER
            winAmount='$1.00'
            lossAmount=u'50¢'
            usedStims.update([quarter1_cue, quarter2_cue, quarter3_cue,
                              quarter4_cue, quarter5_cue, quarter6_cue])
            # Possible Win
            quarter1_cue.pos = [-3 * xscale, 2]
            quarter2_cue.pos = [-1 * xscale, 2]
            quarter3_cue.pos = [1 * xscale, 2]
            quarter4_cue.pos = [3 * xscale, 2]
            # Possible Lose
            quarter5_cue.pos = [-1 * xscale, -2]
            quarter6_cue.pos = [1 * xscale, -2]
            xStim1_cue.pos = quarter5_cue.pos
            xStim2_cue.pos = quarter6_cue.pos
            xStim1_cue.opacity, xStim2_cue.opacity = 1, 1
        
        for coin in cueStims:
            coin.opacity = 1
        for coin in cueStims - usedStims:
            coin.opacity = 0
        
        winCueMsg = "Win %s" %(winAmount)
        lossCueMsg = "Lose %s" %(lossAmount)
        
        # When outside the trials loop, currentTrialN == blockN
        if currentLoop.name not in ['practiceTrial']:
            currentN = blocks.thisRepN * trialsPerBlock
        else:
            currentN = currentLoop.thisN
        trialNum = currentN + 1
        StakesWin.setText(winCueMsg)
        StakesLoss.setText(lossCueMsg)
        
        
        
        # keep track of which components have finished
        trial_cueComponents = []
        trial_cueComponents.append(circle)
        trial_cueComponents.append(line)
        trial_cueComponents.append(StakesWin)
        trial_cueComponents.append(StakesLoss)
        trial_cueComponents.append(dime1_cue)
        trial_cueComponents.append(dime2_cue)
        trial_cueComponents.append(dime3_cue)
        trial_cueComponents.append(quarter1_cue)
        trial_cueComponents.append(quarter2_cue)
        trial_cueComponents.append(quarter3_cue)
        trial_cueComponents.append(quarter4_cue)
        trial_cueComponents.append(quarter5_cue)
        trial_cueComponents.append(quarter6_cue)
        trial_cueComponents.append(xStim1_cue)
        trial_cueComponents.append(xStim2_cue)
        for thisComponent in trial_cueComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "trial_cue"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = trial_cueClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            #for prompt in buttonPrompts:
            #    prompt.draw()
            
            # *circle* updates
            if t >= 0.0 and circle.status == NOT_STARTED:
                # keep track of start time/frame for later
                circle.tStart = t  # underestimates by a little under one frame
                circle.frameNStart = frameN  # exact frame index
                circle.setAutoDraw(True)
            if circle.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                circle.setAutoDraw(False)
            
            # *line* updates
            if t >= 0.0 and line.status == NOT_STARTED:
                # keep track of start time/frame for later
                line.tStart = t  # underestimates by a little under one frame
                line.frameNStart = frameN  # exact frame index
                line.setAutoDraw(True)
            if line.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                line.setAutoDraw(False)
            
            # *StakesWin* updates
            if t >= 0.0 and StakesWin.status == NOT_STARTED:
                # keep track of start time/frame for later
                StakesWin.tStart = t  # underestimates by a little under one frame
                StakesWin.frameNStart = frameN  # exact frame index
                StakesWin.setAutoDraw(True)
            if StakesWin.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                StakesWin.setAutoDraw(False)
            
            # *StakesLoss* updates
            if t >= 0.0 and StakesLoss.status == NOT_STARTED:
                # keep track of start time/frame for later
                StakesLoss.tStart = t  # underestimates by a little under one frame
                StakesLoss.frameNStart = frameN  # exact frame index
                StakesLoss.setAutoDraw(True)
            if StakesLoss.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                StakesLoss.setAutoDraw(False)
            
            # *dime1_cue* updates
            if t >= 0.0 and dime1_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                dime1_cue.tStart = t  # underestimates by a little under one frame
                dime1_cue.frameNStart = frameN  # exact frame index
                dime1_cue.setAutoDraw(True)
            if dime1_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                dime1_cue.setAutoDraw(False)
            
            # *dime2_cue* updates
            if t >= 0.0 and dime2_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                dime2_cue.tStart = t  # underestimates by a little under one frame
                dime2_cue.frameNStart = frameN  # exact frame index
                dime2_cue.setAutoDraw(True)
            if dime2_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                dime2_cue.setAutoDraw(False)
            
            # *dime3_cue* updates
            if t >= 0.0 and dime3_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                dime3_cue.tStart = t  # underestimates by a little under one frame
                dime3_cue.frameNStart = frameN  # exact frame index
                dime3_cue.setAutoDraw(True)
            if dime3_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                dime3_cue.setAutoDraw(False)
            
            # *quarter1_cue* updates
            if t >= 0.0 and quarter1_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                quarter1_cue.tStart = t  # underestimates by a little under one frame
                quarter1_cue.frameNStart = frameN  # exact frame index
                quarter1_cue.setAutoDraw(True)
            if quarter1_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                quarter1_cue.setAutoDraw(False)
            
            # *quarter2_cue* updates
            if t >= 0.0 and quarter2_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                quarter2_cue.tStart = t  # underestimates by a little under one frame
                quarter2_cue.frameNStart = frameN  # exact frame index
                quarter2_cue.setAutoDraw(True)
            if quarter2_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                quarter2_cue.setAutoDraw(False)
            
            # *quarter3_cue* updates
            if t >= 0.0 and quarter3_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                quarter3_cue.tStart = t  # underestimates by a little under one frame
                quarter3_cue.frameNStart = frameN  # exact frame index
                quarter3_cue.setAutoDraw(True)
            if quarter3_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                quarter3_cue.setAutoDraw(False)
            
            # *quarter4_cue* updates
            if t >= 0.0 and quarter4_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                quarter4_cue.tStart = t  # underestimates by a little under one frame
                quarter4_cue.frameNStart = frameN  # exact frame index
                quarter4_cue.setAutoDraw(True)
            if quarter4_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                quarter4_cue.setAutoDraw(False)
            
            # *quarter5_cue* updates
            if t >= 0.0 and quarter5_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                quarter5_cue.tStart = t  # underestimates by a little under one frame
                quarter5_cue.frameNStart = frameN  # exact frame index
                quarter5_cue.setAutoDraw(True)
            if quarter5_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                quarter5_cue.setAutoDraw(False)
            
            # *quarter6_cue* updates
            if t >= 0.0 and quarter6_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                quarter6_cue.tStart = t  # underestimates by a little under one frame
                quarter6_cue.frameNStart = frameN  # exact frame index
                quarter6_cue.setAutoDraw(True)
            if quarter6_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                quarter6_cue.setAutoDraw(False)
            
            if frameN == 0:
                cueStartTime = fmriClock.getTime()
            
            
            # *xStim1_cue* updates
            if t >= 0.0 and xStim1_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                xStim1_cue.tStart = t  # underestimates by a little under one frame
                xStim1_cue.frameNStart = frameN  # exact frame index
                xStim1_cue.setAutoDraw(True)
            if xStim1_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                xStim1_cue.setAutoDraw(False)
            
            # *xStim2_cue* updates
            if t >= 0.0 and xStim2_cue.status == NOT_STARTED:
                # keep track of start time/frame for later
                xStim2_cue.tStart = t  # underestimates by a little under one frame
                xStim2_cue.frameNStart = frameN  # exact frame index
                xStim2_cue.setAutoDraw(True)
            if xStim2_cue.status == STARTED and t >= (0.0 + (1.5-win.monitorFramePeriod*0.75)): #most of one frame period left
                xStim2_cue.setAutoDraw(False)
            if frameN == 0:
                winRecorder.keyframe()
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in trial_cueComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "trial_cue"-------
        for thisComponent in trial_cueComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        
        
        if currentLoop.name == 'blocks':
            cueEndTime = fmriClock.getTime()
            currentLoop.addData('cueStartTime', cueStartTime)
            currentLoop.addData('cueEndTime', cueEndTime)
            events.append({'run': runNum,
                           'condition': valueCondition + 'Cue',
                           'onset': cueStartTime,
                           'duration': 1.5,})
        
            ## Create the first trial of a block and add cue time to it.
            #trialInfo = dict(cueOnset=cueOnsetTime,
            #                  cueOffset=cueOffsetTime,
            #                  run=runs.thisN + session)
            runDataframe.loc[currentN, 'cueStartTime'] = cueStartTime
            runDataframe.loc[currentN, 'cueEndTime'] = cueEndTime
            runDataframe.loc[currentN, 'run'] = runs.thisN + runNumber
            currentLoop.addData('trialNum', trialNum)
        
        
        # set up handler to look after randomisation of conditions etc
        trials = data.TrialHandler(nReps=1, method='sequential', 
            extraInfo=expInfo, originPath=-1,
            trialList=data.importConditions(trialsList, selection=slice(row_start, row_end)),
            seed=None, name='trials')
        thisExp.addLoop(trials)  # add the loop to the experiment
        thisTrial = trials.trialList[0]  # so we can initialise stimuli with some values
        # abbreviate parameter names if possible (e.g. rgb=thisTrial.rgb)
        if thisTrial != None:
            for paramName in thisTrial.keys():
                exec(paramName + '= thisTrial.' + paramName)
        
        for thisTrial in trials:
            currentLoop = trials
            # abbreviate parameter names if possible (e.g. rgb = thisTrial.rgb)
            if thisTrial != None:
                for paramName in thisTrial.keys():
                    exec(paramName + '= thisTrial.' + paramName)
            
            #------Prepare to start Routine "jitter"-------
            t = 0
            jitterClock.reset()  # clock 
            frameN = -1
            routineTimer.add(1.500000)
            # update component parameters for each repeat
            # Force a non-slip builder routine of "unknown" duration
            # Base fixed duration is .5s, then add the remaining amount here.
            routineTimer.add(ISI1 - 1.5)
            presses, rts = [], []
            
            # keep track of which components have finished
            jitterComponents = []
            jitterComponents.append(jitteredISI)
            for thisComponent in jitterComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "jitter"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = jitterClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                # *jitteredISI* updates
                if t >= 0.0 and jitteredISI.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    jitteredISI.tStart = t  # underestimates by a little under one frame
                    jitteredISI.frameNStart = frameN  # exact frame index
                    jitteredISI.setAutoDraw(True)
                if jitteredISI.status == STARTED and frameN >= 0:
                    jitteredISI.setAutoDraw(False)
                for prompt in buttonPrompts:
                    prompt.draw()
                jitteredISI.draw()
                
                if frameN == 0:
                    isi1StartTime = fmriClock.getTime()
                    jitterComponents.append(extraISIStim)
                    extraISIStim.status = STARTED
                    winRecorder.keyframe()
                
                
                isiPresses = event.getKeys(keyList=allowed_keys, timeStamped=fmriClock)
                if isiPresses:
                    presses.extend([p[0] for p in isiPresses])
                    rts.extend([p[1] - isi1StartTime for p in isiPresses])
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in jitterComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "jitter"-------
            for thisComponent in jitterComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            isi1EndTime = fmriClock.getTime()
            currentLoop.addData('isiPress1.keys', presses)
            currentLoop.addData('isiPress1.rt', rts)
            currentLoop.addData('isi1StartTime', isi1StartTime)
            currentLoop.addData('isi1EndTime', isi1EndTime)
            runDataframe.loc[currentN, 'isiPress1.keys'] = str(presses)
            runDataframe.loc[currentN, 'isiPress1.rt'] = str(rts)
            runDataframe.loc[currentN, 'isi1StartTime'] = isi1StartTime
            runDataframe.loc[currentN, 'isi1EndTime'] = isi1EndTime
            
            
            #------Prepare to start Routine "trial"-------
            t = 0
            trialClock.reset()  # clock 
            frameN = -1
            routineTimer.add(2.000000)
            # update component parameters for each repeat
            guessResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
            guessResp.status = NOT_STARTED
            
            raTextUpdated = False
            if currentLoop.name not in ['kidPracTrial', 'practiceTrial']:
                currentN = (blocks.thisN * trialsPerBlock) + trials.thisN
            else:
                currentN = 0
            
            trialNum = currentN + 1
            
            if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                last5=None
                total=None
            else:
                last5=raLast5Text
                total=raTotalText
            
            updateTrialWin(currentLoop.name, runDataframe, currentN,
                           raResponseTopText, '', 
                           last5Text=last5, totalText=total)
            
            
            
            # keep track of which components have finished
            trialComponents = []
            trialComponents.append(guess)
            trialComponents.append(guessResp)
            for thisComponent in trialComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "trial"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = trialClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                # *guess* updates
                if t >= 0 and guess.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    guess.tStart = t  # underestimates by a little under one frame
                    guess.frameNStart = frameN  # exact frame index
                    guess.setAutoDraw(True)
                if guess.status == STARTED and t >= (0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    guess.setAutoDraw(False)
                
                # *guessResp* updates
                if t >= 0.0 and guessResp.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    guessResp.tStart = t  # underestimates by a little under one frame
                    guessResp.frameNStart = frameN  # exact frame index
                    guessResp.status = STARTED
                    # AllowedKeys looks like a variable named `allowed_keys`
                    if not 'allowed_keys' in locals():
                        logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                        core.quit()
                    if not type(allowed_keys) in [list, tuple, np.ndarray]:
                        if not isinstance(allowed_keys, basestring):
                            logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                            core.quit()
                        elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                        else:  allowed_keys = eval(allowed_keys)
                    # keyboard checking is just starting
                    win.callOnFlip(guessResp.clock.reset)  # t=0 on next screen flip
                    event.clearEvents(eventType='keyboard')
                if guessResp.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    guessResp.status = STOPPED
                if guessResp.status == STARTED:
                    theseKeys = event.getKeys(keyList=list(allowed_keys))
                    
                    # check for quit:
                    if "escape" in theseKeys:
                        endExpNow = True
                    if len(theseKeys) > 0:  # at least one key was pressed
                        guessResp.keys.extend(theseKeys)  # storing all keys
                        guessResp.rt.append(guessResp.clock.getTime())
                if frameN == 0:
                    guessStartTime = fmriClock.getTime()
                
                if not raTextUpdated and guessResp.keys:
                    runDataframe.loc[currentN, 'guessResp.keys'] = str(guessResp.keys)
                    runDataframe.loc[currentN, 'guessResp.rt'] = str(guessResp.rt)
                
                    guessRespFirstKey = guessResp.keys[0]
                    guessRespFirstRt = guessResp.rt[0]
                
                    if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                        last5=None
                        total=None
                    else:
                        last5=raLast5Text
                        total=raTotalText
                
                    # Don't update trials for practice.
                    updateTrialWin(currentLoop.name, runDataframe, currentN,
                                   raResponseTopText, guessRespFirstKey, 
                                   last5Text=last5, totalText=total)
                
                    raTextUpdated = True
                
                for prompt in buttonPrompts:
                    prompt.draw()
                
                if frameN == 0:
                    winRecorder.keyframe()
                
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in trialComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "trial"-------
            for thisComponent in trialComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            # check responses
            if guessResp.keys in ['', [], None]:  # No response was made
               guessResp.keys=None
            # store data for trials (TrialHandler)
            trials.addData('guessResp.keys',guessResp.keys)
            if guessResp.keys != None:  # we had a response
                trials.addData('guessResp.rt', guessResp.rt)
            # Also set in trial > raWinTrialCode > Each Frame for inter-trial presses
            if guessResp.keys:
                guessRespFirstKey = guessResp.keys[0]
                guessRespFirstRt = guessResp.rt[0]
                guessRespFirstGuess = guessLookup[guessRespFirstKey]
            else:
                guessRespFirstKey = None
                guessRespFirstRt = None
                guessRespFirstGuess = None
            
            if currentLoop.name == 'trials':
                guessEndTime = fmriClock.getTime()
                currentLoop.addData('guessStartTime', guessStartTime)
                currentLoop.addData('guessEndTime', guessEndTime)
                currentLoop.addData('guessResp.firstGuess', guessRespFirstGuess)
                events.append({'run': runNum,
                               'condition': 'guess',
                               'onset': guessStartTime,
                               'duration': 1.5})
                runDataframe.loc[currentN, 'trial'] = currentN + 1
                runDataframe.loc[currentN, 'run'] = runNum
                runDataframe.loc[currentN, 'guessStartTime'] = guessStartTime
                runDataframe.loc[currentN, 'guessEndTime'] = guessEndTime
                runDataframe.loc[currentN, 'guessResp.keys'] = str(guessResp.keys)
                runDataframe.loc[currentN, 'guessResp.firstKey'] = guessRespFirstKey
                runDataframe.loc[currentN, 'guessResp.firstRt'] = guessRespFirstRt
                runDataframe.loc[currentN, 'guessResp.firstGuess'] = guessRespFirstGuess
            
                if guessResp.keys:
                    runDataframe.loc[currentN, 'guessResp.rt'] = str(guessResp.rt)
                # else:  # Leave as nan
            
            guessed_keys = guessResp.keys
            if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                updateTrialWin(currentLoop.name, runDataframe, currentN,
                               raResponseTopText, guessRespFirstKey)
            else:
                updateTrialWin(currentLoop.name, runDataframe, currentN,
                               raResponseTopText, guessRespFirstKey, 
                               raLast5Text, raTotalText)
            
            
            
            
            #------Prepare to start Routine "jitter2"-------
            t = 0
            jitter2Clock.reset()  # clock 
            frameN = -1
            routineTimer.add(1.500000)
            # update component parameters for each repeat
            # Force a non-slip builder routine of "unknown" duration
            # Base fixed duration is .5s, then add the remaining amount here.
            routineTimer.add(ISI2 - 1.5)
            presses, rts = [], []
            # keep track of which components have finished
            jitter2Components = []
            jitter2Components.append(jitteredISI_2)
            for thisComponent in jitter2Components:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "jitter2"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = jitter2Clock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                # *jitteredISI_2* updates
                if t >= 0.0 and jitteredISI_2.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    jitteredISI_2.tStart = t  # underestimates by a little under one frame
                    jitteredISI_2.frameNStart = frameN  # exact frame index
                    jitteredISI_2.setAutoDraw(True)
                if jitteredISI_2.status == STARTED and frameN >= 0:
                    jitteredISI_2.setAutoDraw(False)
                for prompt in buttonPrompts:
                    prompt.draw()
                jitteredISI_2.draw()
                
                if frameN == 0:
                    jitter2Components.append(extraISIStim)
                    extraISIStim.status = STARTED
                    isi2StartTime = fmriClock.getTime()
                    winRecorder.keyframe()
                
                
                isiPresses = event.getKeys(keyList=allowed_keys, timeStamped=fmriClock)
                if isiPresses:
                    presses.extend([p[0] for p in isiPresses])
                    rts.extend([p[1] - isi2StartTime for p in isiPresses])
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in jitter2Components:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "jitter2"-------
            for thisComponent in jitter2Components:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            isi2EndTime = fmriClock.getTime()
            currentLoop.addData('isiPress2.keys', presses)
            currentLoop.addData('isiPress2.rt', rts)
            currentLoop.addData('isi2StartTime', isi2StartTime)
            currentLoop.addData('isi2EndTime', isi2EndTime)
            runDataframe.loc[currentN, 'isiPress2.keys'] = str(presses)
            runDataframe.loc[currentN, 'isiPress2.rt'] = str(rts)
            runDataframe.loc[currentN, 'isi2StartTime'] = isi2StartTime
            runDataframe.loc[currentN, 'isi2EndTime'] = isi2EndTime
            
            #------Prepare to start Routine "feedback"-------
            t = 0
            feedbackClock.reset()  # clock 
            frameN = -1
            routineTimer.add(1.000000)
            # update component parameters for each repeat
            # calculate feedback and reward and account for missed responses
            fbkStims = set([dime1_fbk, dime2_fbk, quarter1_fbk, quarter2_fbk,
                            quarter3_fbk, quarter4_fbk])
            feedbackPos = [0, 2]
            
            usedStims.clear()
            
            if valueCondition == "low":  # DIME
              if Feedback == 1:  # Win
                feedbackName = "Win"
                rewardAmount = u"20¢"
                rewardAmountNum = .20
                dime1_fbk.pos = [-1 * xscale, yoffset]
                dime2_fbk.pos = [1 * xscale, yoffset]
                usedStims.update([dime1_fbk, dime2_fbk])
                xStim1.opacity, xStim2.opacity = 0, 0
            
              else:  # Lose
                feedbackName = "Lose"
                rewardAmount = u"10¢"
                rewardAmountNum = -.1
                dime1_fbk.pos = [0, yoffset]
                
                xStim1.pos = [0, yoffset]
                xStim1.opacity, xStim2.opacity = 1, 0
            
                usedStims.update([dime1_fbk])
            
            elif valueCondition == "high":  # QUARTER
              if Feedback == 1:  # Win
                feedbackName="Win"
                rewardAmount = "$1.00"
                rewardAmountNum = 1
                quarter1_fbk.pos = [-3 * highXScale, yoffset]
                quarter2_fbk.pos = [-1 * highXScale, yoffset]
                quarter3_fbk.pos = [1 * highXScale, yoffset]
                quarter4_fbk.pos = [3 * highXScale, yoffset]
                xStim1.opacity, xStim2.opacity = 0, 0
                usedStims.update([quarter1_fbk, quarter2_fbk, 
                                  quarter3_fbk, quarter4_fbk])
            
              else: # Lose
                rewardAmount = u"50¢"
                rewardAmountNum = -.5
                feedbackName="Lose"
                quarter1_fbk.pos = [-1.5 * highXScale, yoffset]
                quarter2_fbk.pos = [1.5 * highXScale, yoffset]
                xStim1.pos = [-1.5 * highXScale, yoffset]
                xStim2.pos = [1.5 * highXScale, yoffset]
                xStim1.opacity = 1
                xStim2.opacity = 1
                usedStims.update([quarter1_fbk, quarter2_fbk])
                xStim1.opacity, xStim2.opacity = 1, 1
            
            feedbackMsg = "%s %s" %(feedbackName, rewardAmount)
            RewardMsg="%s" %(rewardAmount)
            feedbackDisplayComponents = []
            
            # Allow for Too Slow on practice trials
            if currentLoop.name == 'practiceTrial' and not guessed_keys:
                feedbackName = 'Too slow'
                feedbackMsg = 'Too slow!'
                rewardAmount = 0
                usedStims.clear()
            
            for stim in usedStims:
              stim.opacity = 1
            
            for stim in fbkStims - usedStims:
              stim.opacity = 0
            
            if feedbackName == 'Win':
                for frame in rewFrames:
                    frame.opacity = 1
                for frame in unrewFrames:
                    frame.opacity = 0
            elif feedbackName == 'Lose':
                for frame in rewFrames:
                    frame.opacity = 0
                for frame in unrewFrames:
                    frame.opacity = 1
            else:  # Don't draw frame on practice no response trial
                for frame in rewFrames + unrewFrames + [xStim1, xStim2]:
                    frame.opacity = 0
            
            if not guessed_keys:
                nNoResp += 1
            
            record_expt_window = True
            fdback.setText(feedbackMsg)
            fdback.setPos(feedbackPos)
            
            
            
            # keep track of which components have finished
            feedbackComponents = []
            feedbackComponents.append(circleFrame)
            feedbackComponents.append(outerSquareFrame)
            feedbackComponents.append(innerSquareFrame)
            feedbackComponents.append(fdback)
            feedbackComponents.append(quarter1_fbk)
            feedbackComponents.append(quarter2_fbk)
            feedbackComponents.append(quarter3_fbk)
            feedbackComponents.append(quarter4_fbk)
            feedbackComponents.append(dime1_fbk)
            feedbackComponents.append(dime2_fbk)
            feedbackComponents.append(xStim1)
            feedbackComponents.append(xStim2)
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "feedback"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = feedbackClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                if currentLoop.name == 'practiceTrial' or currentLoop.name == 'kidPracLoop':
                    if frameN == 0:
                        routineTimer.add(3.0)  # add more time for practice feedback (4s total)
                        feedbackComponents.append(extraFeedbackStim)
                        extraFeedbackStim.status = STARTED  
                        # Draw this forever (so components don't shut off), but maintain the non-slip timing
                
                    # n.b. Some of these stims (e.g. xStim1) may have opacity == 0
                    # and won't be visible, but "draw" them as active stims anyway
                    # for consistency with the rest of the routine. 
                    # Continue drawing stims even if the Builder response thinks
                    # they're done.
                    # Also, must be split up for order (frame in the back, then
                    # feedback, followed by X's on top.
                    for stim in [circleFrame, outerSquareFrame, innerSquareFrame]:
                #        if stim.status == FINISHED:
                        stim.draw()
                    for stim in set([fdback]).union(fbkStims):
                        stim.draw()
                    for stim in [xStim1, xStim2]:
                        stim.draw()
                
                
                # *circleFrame* updates
                if t >= 0.0 and circleFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    circleFrame.tStart = t  # underestimates by a little under one frame
                    circleFrame.frameNStart = frameN  # exact frame index
                    circleFrame.setAutoDraw(True)
                if circleFrame.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    circleFrame.setAutoDraw(False)
                
                # *outerSquareFrame* updates
                if t >= 0.0 and outerSquareFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    outerSquareFrame.tStart = t  # underestimates by a little under one frame
                    outerSquareFrame.frameNStart = frameN  # exact frame index
                    outerSquareFrame.setAutoDraw(True)
                if outerSquareFrame.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    outerSquareFrame.setAutoDraw(False)
                
                # *innerSquareFrame* updates
                if t >= 0.0 and innerSquareFrame.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    innerSquareFrame.tStart = t  # underestimates by a little under one frame
                    innerSquareFrame.frameNStart = frameN  # exact frame index
                    innerSquareFrame.setAutoDraw(True)
                if innerSquareFrame.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    innerSquareFrame.setAutoDraw(False)
                
                # *fdback* updates
                if t >= 0.0 and fdback.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    fdback.tStart = t  # underestimates by a little under one frame
                    fdback.frameNStart = frameN  # exact frame index
                    fdback.setAutoDraw(True)
                if fdback.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    fdback.setAutoDraw(False)
                
                # *quarter1_fbk* updates
                if t >= 0.0 and quarter1_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter1_fbk.tStart = t  # underestimates by a little under one frame
                    quarter1_fbk.frameNStart = frameN  # exact frame index
                    quarter1_fbk.setAutoDraw(True)
                if quarter1_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter1_fbk.setAutoDraw(False)
                
                # *quarter2_fbk* updates
                if t >= 0.0 and quarter2_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter2_fbk.tStart = t  # underestimates by a little under one frame
                    quarter2_fbk.frameNStart = frameN  # exact frame index
                    quarter2_fbk.setAutoDraw(True)
                if quarter2_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter2_fbk.setAutoDraw(False)
                
                # *quarter3_fbk* updates
                if t >= 0.0 and quarter3_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter3_fbk.tStart = t  # underestimates by a little under one frame
                    quarter3_fbk.frameNStart = frameN  # exact frame index
                    quarter3_fbk.setAutoDraw(True)
                if quarter3_fbk.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter3_fbk.setAutoDraw(False)
                
                # *quarter4_fbk* updates
                if t >= 0.0 and quarter4_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    quarter4_fbk.tStart = t  # underestimates by a little under one frame
                    quarter4_fbk.frameNStart = frameN  # exact frame index
                    quarter4_fbk.setAutoDraw(True)
                if quarter4_fbk.status == STARTED and t >= (0.0 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                    quarter4_fbk.setAutoDraw(False)
                
                # *dime1_fbk* updates
                if t >= 0.0 and dime1_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime1_fbk.tStart = t  # underestimates by a little under one frame
                    dime1_fbk.frameNStart = frameN  # exact frame index
                    dime1_fbk.setAutoDraw(True)
                if dime1_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime1_fbk.setAutoDraw(False)
                
                # *dime2_fbk* updates
                if t >= 0.0 and dime2_fbk.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    dime2_fbk.tStart = t  # underestimates by a little under one frame
                    dime2_fbk.frameNStart = frameN  # exact frame index
                    dime2_fbk.setAutoDraw(True)
                if dime2_fbk.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    dime2_fbk.setAutoDraw(False)
                
                # *xStim1* updates
                if t >= 0.0 and xStim1.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim1.tStart = t  # underestimates by a little under one frame
                    xStim1.frameNStart = frameN  # exact frame index
                    xStim1.setAutoDraw(True)
                if xStim1.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim1.setAutoDraw(False)
                
                # *xStim2* updates
                if t >= 0.0 and xStim2.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    xStim2.tStart = t  # underestimates by a little under one frame
                    xStim2.frameNStart = frameN  # exact frame index
                    xStim2.setAutoDraw(True)
                if xStim2.status == STARTED and t >= (0.0 + (1-win.monitorFramePeriod*0.75)): #most of one frame period left
                    xStim2.setAutoDraw(False)
                if frameN == 0:
                    feedbackStartTime = fmriClock.getTime()
                
                
                #for prompt in buttonPrompts:
                #    prompt.draw()
                
                if record_expt_window:
                    winRecorder.keyframe()
                    record_expt_window = False
                
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in feedbackComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "feedback"-------
            for thisComponent in feedbackComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            if currentLoop.name != 'practiceTrial' and currentLoop.name != 'kidPracLoop':
                cumulativeReward += rewardAmountNum
            
                currentLoop.addData('guessResp.firstKey', guessRespFirstKey)
                currentLoop.addData('guessResp.firstRt', guessRespFirstRt)
            
                currentLoop.addData('rewardAmount', rewardAmountNum)
                currentLoop.addData('feedbackName', feedbackName)
                currentLoop.addData('cumulativeReward', cumulativeReward)
                currentLoop.addData('cumulativeNoResp', nNoResp)
            
                trialCondition = valueCondition + feedbackName
                currentLoop.addData('trialCondition', trialCondition)
            
                runDataframe.loc[currentN, 'rewardAmount'] = rewardAmountNum
                runDataframe.loc[currentN, 'feedbackName'] = feedbackName
                runDataframe.loc[currentN, 'cumulativeReward'] = cumulativeReward
                runDataframe.loc[currentN, 'cumulativeNoResp'] = nNoResp
                runDataframe.loc[currentN, 'trialCondition'] = trialCondition
            
            usedStims.clear()
            
            xStim1.setAutoDraw(False)
            xStim2.setAutoDraw(False)
            feedbackEndTime = fmriClock.getTime()
            
            if Feedback:
                feedback_str = 'correct'
            else:
                feedback_str = 'incorrect'
            
            if not guessed_keys:
                feedback_str += 'noresponse'
            
            if currentLoop.name == 'trials':
                feedbackOffsetTime = fmriClock.getTime()
                currentLoop.addData('feedbackStartTime', feedbackStartTime)
                currentLoop.addData('feedbackEndTime', feedbackEndTime)
                events.append({'run': runs.thisN + runNumber,
                               'condition': trialCondition + 'Feedback',
                               'onset': feedbackStartTime,
                               'duration': 1,})
                trialInfo = None
                currentLoop.addData('trialNum', trialNum)
                updateRunLog(thisExp.entries)
            
            
            thisExp.nextEntry()
            
        # completed 1 repeats of 'trials'
        
        
        #------Prepare to start Routine "Break"-------
        t = 0
        BreakClock.reset()  # clock 
        frameN = -1
        routineTimer.add(8.000000)
        # update component parameters for each repeat
        
        # keep track of which components have finished
        BreakComponents = []
        BreakComponents.append(breakJitter)
        for thisComponent in BreakComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "Break"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = BreakClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *breakJitter* updates
            if t >= 0.0 and breakJitter.status == NOT_STARTED:
                # keep track of start time/frame for later
                breakJitter.tStart = t  # underestimates by a little under one frame
                breakJitter.frameNStart = frameN  # exact frame index
                breakJitter.setAutoDraw(True)
            if breakJitter.status == STARTED and t >= (0.0 + (8-win.monitorFramePeriod*0.75)): #most of one frame period left
                breakJitter.setAutoDraw(False)
            if expInfo['runNumber'] == 'practice' and t > 3:
                continueRoutine = False
            for prompt in buttonPrompts:
                prompt.draw()
            if frameN == 0:
                winRecorder.keyframe()
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in BreakComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "Break"-------
        for thisComponent in BreakComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        if blocks.nRemaining == 0:
            runEndTime = fmriClock.getTime()
        
            trials.addData('runEndTime', runEndTime )
            runs.addData('runEndTime', runEndTime )
        
            # Add block end event
            events.append({'run': runNum,
                           'condition': 'runEnd',
                           'onset': runEndTime,
                           'duration': 0.})
        
            events_df = pd.DataFrame(events)
            designOutfile = '%s_design.csv' % filename
            events_df.to_csv(designOutfile, index=False)
        
            runDataframe['runEndTime'] = runEndTime
        
            trialEntries = thisExp.entries
            trialEntries[-1]['runEndTime'] = runEndTime
            updateRunLog(trialEntries)
        
        thisExp.nextEntry()
        
    # completed nBlocks repeats of 'blocks'
    
    
    #------Prepare to start Routine "end_run"-------
    t = 0
    end_runClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    if runNumber == 0:
        if cumulativeReward > 1.00:  # Got at least $1 
            shouldContinue = False
            runs.finished = True
            endRunMsg = ''
        else:
            shouldContinue = True
            endRunMsg = "You won less than $1.\nLet's try again."
    else:
        endRunMsg = "You finished this round."
        shouldContinue = True
        totalCumulativeReward += cumulativeReward
    
    endRunText2.setText(endRunMsg)
    endRunResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    endRunResp.status = NOT_STARTED
    raText.pos = [0, -4]
    raText.text = 'Press <space> to continue.'
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    end_runComponents = []
    end_runComponents.append(endRunText1)
    end_runComponents.append(endRunText2)
    end_runComponents.append(endRunResp)
    for thisComponent in end_runComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "end_run"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = end_runClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if frameN == 0:
            continueRoutine = shouldContinue
        
        
        # *endRunText1* updates
        if t >= 0.0 and endRunText1.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunText1.tStart = t  # underestimates by a little under one frame
            endRunText1.frameNStart = frameN  # exact frame index
            endRunText1.setAutoDraw(True)
        if endRunText1.status == STARTED and t >= (0.0 + (2.0-win.monitorFramePeriod*0.75)): #most of one frame period left
            endRunText1.setAutoDraw(False)
        
        # *endRunText2* updates
        if t >= 2.0 and endRunText2.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunText2.tStart = t  # underestimates by a little under one frame
            endRunText2.frameNStart = frameN  # exact frame index
            endRunText2.setAutoDraw(True)
        
        # *endRunResp* updates
        if t >= 0.0 and endRunResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            endRunResp.tStart = t  # underestimates by a little under one frame
            endRunResp.frameNStart = frameN  # exact frame index
            endRunResp.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(endRunResp.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if endRunResp.status == STARTED:
            theseKeys = event.getKeys(keyList=['y', 'n', 'left', 'right', 'space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                endRunResp.keys = theseKeys[-1]  # just the last key pressed
                endRunResp.rt = endRunResp.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in end_runComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "end_run"-------
    for thisComponent in end_runComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # check responses
    if endRunResp.keys in ['', [], None]:  # No response was made
       endRunResp.keys=None
    # store data for runs (TrialHandler)
    runs.addData('endRunResp.keys',endRunResp.keys)
    if endRunResp.keys != None:  # we had a response
        runs.addData('endRunResp.rt', endRunResp.rt)
    
    # the Routine "end_run" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed nRuns repeats of 'runs'


#------Prepare to start Routine "End"-------
t = 0
EndClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat
if expInfo['runNumber'] == 'practice':
    endMsg = 'You finished the practice!'
else:
    endMsg = 'Congratulations, you finished!\n\nYou made $%1.02f.' % totalCumulativeReward
endText.setText(endMsg)
endResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
endResp.status = NOT_STARTED
if expInfo['runNumber'] == 'practice':
    raVerbalText.text='"Congratulations! You finished the practice!'
else:
    raVerbalText.text='"Congratulations! You finished the Guessing Game and made $%1.02f! We will give you this money in cash at the end of today\'s session. Good job!"' % totalCumulativeReward

raText.text = 'Press <space> to continue.'
raVerbalText.draw()
raText.draw()
raWinRecorder.keyframe()
raWin.flip()

# keep track of which components have finished
EndComponents = []
EndComponents.append(endText)
EndComponents.append(endResp)
for thisComponent in EndComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "End"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = EndClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    
    # *endText* updates
    if t >= 0 and endText.status == NOT_STARTED:
        # keep track of start time/frame for later
        endText.tStart = t  # underestimates by a little under one frame
        endText.frameNStart = frameN  # exact frame index
        endText.setAutoDraw(True)
    
    # *endResp* updates
    if t >= 0 and endResp.status == NOT_STARTED:
        # keep track of start time/frame for later
        endResp.tStart = t  # underestimates by a little under one frame
        endResp.frameNStart = frameN  # exact frame index
        endResp.status = STARTED
        # keyboard checking is just starting
        win.callOnFlip(endResp.clock.reset)  # t=0 on next screen flip
        event.clearEvents(eventType='keyboard')
    if endResp.status == STARTED:
        theseKeys = event.getKeys(keyList=['space'])
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            endResp.keys = theseKeys[-1]  # just the last key pressed
            endResp.rt = endResp.clock.getTime()
            # a response ends the routine
            continueRoutine = False
    if frameN == 0:
        winRecorder.keyframe()
    
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in EndComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "End"-------
for thisComponent in EndComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)

# check responses
if endResp.keys in ['', [], None]:  # No response was made
   endResp.keys=None
# store data for thisExp (ExperimentHandler)
thisExp.addData('endResp.keys',endResp.keys)
if endResp.keys != None:  # we had a response
    thisExp.addData('endResp.rt', endResp.rt)
thisExp.nextEntry()

# the Routine "End" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()

raWin.close()

#outerFrame.autoDraw = False
save_movie(filename, winRecorder, raWinRecorder)





























































# these shouldn't be strictly necessary (should auto-save)
thisExp.saveAsPickle(filename)
logging.flush()
# make sure everything is closed down
thisExp.abort() # or data files will save again on exit
win.close()
core.quit()
